@@ -1,6 +1,6 @@
 /***************************************************************************
  * base.h is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -23,8 +23,6 @@
 #include "mgl2/abstract.h"
 
 #ifdef __cplusplus
-#include <vector>
-#include <string>
 
 #if (MGL_HAVE_PTHREAD|MGL_HAVE_PTHR_WIDGET)
 #include <pthread.h>
@@ -35,9 +33,6 @@
 #else
 #define MGL_PUSH(a,v,m)	a.push_back(v);
 #endif
-#if MGL_HAVE_OMP
-#include <omp.h>
-#endif
 //-----------------------------------------------------------------------------
 inline mreal mgl_d(mreal v,mreal v1,mreal v2) { return v2!=v1?(v-v1)/(v2-v1):NAN; }
 //-----------------------------------------------------------------------------
@@ -59,13 +54,13 @@ public:
 	mglStack(const mglStack<T> &st)
 	{	np=st.np;	dat = (T**)malloc(np*sizeof(T*));
 		pb=st.pb;	m=n=0;	reserve(st.n);
-		for(size_t i=0;i<m;i++)	memcpy(dat[i],st.dat[i],(1L<<pb)*sizeof(T));
+		for(size_t i=0;i<m;i++)	memcpy(dat[i],st.dat[i],((size_t)1<<pb)*sizeof(T));
 		n=st.n;		mutex = 0;	}
 	mglStack(size_t Pbuf=10)
 	{	np=16;	pb=Pbuf;	dat = (T**)malloc(np*sizeof(T*));
-		dat[0] = new T[1L<<pb];	n=0;	m=1;	mutex = 0;	}
+		dat[0] = new T[(size_t)1<<pb];	n=0;	m=1;	mutex = 0;	}
 	~mglStack()	{	clear();	delete [](dat[0]);	free(dat);	}
-	inline void set_mutex(void *m)	{	mutex = m;	}
+	inline void set_mutex(void *mtx)	{	mutex = mtx;	}
 	void reserve(size_t num)
 	{
 		num+=n;
@@ -74,44 +69,30 @@ public:
 			num = 1+ (num>>pb);
 			if(num>np)
 			{	dat = (T**)realloc(dat, num*sizeof(T*));	np=num;	}
-			for(size_t i=m;i<num;i++)	dat[i] = new T[1L<<pb];
+			for(size_t i=m;i<num;i++)	dat[i] = new T[(size_t)1<<pb];
 			m = num;
 		}
 	}
 	void clear()
 	{
-		if(mutex)
-		{
-#if MGL_HAVE_PTHREAD
-			pthread_mutex_lock((pthread_mutex_t *)mutex);
-#elif MGL_HAVE_OMP
-			omp_set_lock((omp_lock_t *)mutex);
-#endif
-		}
+		if(mutex)	mgl_mutex_lock(mutex);
 		for(size_t i=0;i<m;i++)	delete [](dat[i]);
-		dat[0] = new T[1L<<pb];	n=0;	m=1;
-		if(mutex)
-		{
-#if MGL_HAVE_PTHREAD
-			pthread_mutex_unlock((pthread_mutex_t *)mutex);
-#elif MGL_HAVE_OMP
-			omp_unset_lock((omp_lock_t *)mutex);
-#endif
-		}
+		dat[0] = new T[(size_t)1<<pb];	n=0;	m=1;
+		if(mutex)	mgl_mutex_unlock(mutex);
 	}
-	T &operator[](size_t i)	{	register size_t d=i>>pb;	return dat[d][i-(d<<pb)];	}
-	const T &operator[](size_t i)	const	{	register size_t d=i>>pb;	return dat[d][i-(d<<pb)];	}
+	T &operator[](size_t i)	{	size_t d=i>>pb;	return dat[d][i-(d<<pb)];	}
+	const T &operator[](size_t i)	const	{	size_t d=i>>pb;	return dat[d][i-(d<<pb)];	}
 	void push_back(const T &t)
 	{
 		if(n>=(m<<pb))	reserve(1);
-		register size_t d=n>>pb;
+		size_t d=n>>pb;
 		dat[d][n-(d<<pb)] = t;	n++;
 	}
 	size_t size()	const	{	return n;	}
 	const mglStack<T> &operator=(const mglStack<T> &st)
 	{
 		pb=st.pb;	clear();	reserve(st.n);
-		for(size_t i=0;i<st.m && i<m;i++)	memcpy(dat[i],st.dat[i],(1L<<pb)*sizeof(T));
+		for(size_t i=0;i<st.m && i<m;i++)	memcpy(dat[i],st.dat[i],((size_t)1<<pb)*sizeof(T));
 		n = st.n;	return st;
 	}
 };
@@ -167,6 +148,8 @@ struct MGL_EXPORT mglLight
 {
 	mglLight():n(false),a(0),b(0)	{}
 	mglLight(const mglLight &aa) : n(aa.n),d(aa.d),r(aa.r),q(aa.q),p(aa.p),a(aa.a),b(aa.b),c(aa.c)	{}
+	const mglLight &operator=(const mglLight &aa)
+	{	memcpy(this,&aa,sizeof(mglLight));	return aa;	}
 
 	bool n;			///< Availability of light sources
 	mglPoint d;		///< Direction of light sources
@@ -238,6 +221,7 @@ struct MGL_EXPORT mglPnt	// NOTE: use float for reducing memory size
 	mglPnt():x(0),y(0),z(0),u(0),v(0),w(0),r(0),g(0),b(0),a(0),xx(0),yy(0),zz(0),c(0),t(0),ta(0),sub(0)	{}
 	mglPnt(const mglPnt &aa) : sub(aa.sub)	{	memcpy(dat,aa.dat,16*sizeof(float));	}
 	inline const mglPnt&operator=(const mglPnt &aa)	{ sub=aa.sub;	memcpy(dat,aa.dat,16*sizeof(float));	return aa;	}
+	inline bool same(const mglPnt &p, mreal d)	const {	return fabs(x-p.x)<d && fabs(y-p.y)<d;	}
 };
 inline mglPnt operator+(const mglPnt &a, const mglPnt &b)
 {	mglPnt p;	for(long i=0;i<10;i++)	p.dat[i] = a.dat[i]+b.dat[i];	p.sub=a.sub;	return p;	}
@@ -254,46 +238,49 @@ inline mglPnt operator*(float b, const mglPnt &a)
 //-----------------------------------------------------------------------------
 /// Structure for glyph representation
 struct MGL_EXPORT mglGlyph
-{
-	long nt, nl;			///< number of triangles and lines
+{	// NOTE nt<0 is used to set char id for user-defined glyphs
+	long nt, nl;		///< number of triangles and lines
 	short *trig, *line;	///< vertexes of triangles and lines
 
 	mglGlyph():nt(0),nl(0),trig(0),line(0)	{}
 	mglGlyph(const mglGlyph &a):nt(0),nl(0),trig(0),line(0)	{	*this=a;	}
 	mglGlyph(long Nt, long Nl):nt(0),nl(0),trig(0),line(0)	{	Create(Nt,Nl);	}
-#if MGL_HAVE_RVAL
-	mglGlyph(mglGlyph &&aa) : nt(aa.nt),nl(aa.nl),trig(aa.trig), line(aa.line)	{	aa.trig=aa.line=0;	}
-#endif
 	~mglGlyph()	{	if(trig)	delete []trig;	if(line)	delete []line;	}
 
 	void Create(long Nt, long Nl);
-	bool operator==(const mglGlyph &g) MGL_FUNC_PURE;
-	inline bool operator!=(const mglGlyph &g)	{	return !(*this==g);	}
+	bool operator==(const mglGlyph &g) const MGL_FUNC_PURE;
+	inline bool operator!=(const mglGlyph &g) const MGL_FUNC_PURE
+	{	return !(*this==g);	}
 	inline const mglGlyph &operator=(const mglGlyph &a)
-	{	Create(a.nt, a.nl);	memcpy(trig, a.trig, 6*nt*sizeof(short));
-		memcpy(line, a.line, 2*nl*sizeof(short));	return a;	}
+	{	Create(a.nt, a.nl);
+		if(a.trig)	memcpy(trig, a.trig, 6*nt*sizeof(short));
+		if(a.line)	memcpy(line, a.line, 2*nl*sizeof(short));
+		return a;	}
 };
 //-----------------------------------------------------------------------------
 #define MGL_TEXTURE_COLOURS 512
 /// Structure for texture (color scheme + palette) representation
 struct MGL_EXPORT mglTexture
 {
-	mglColor col[MGL_TEXTURE_COLOURS];	///< Colors itself
+	mglColor *col;	///< Colors itself
 	long n;				///< Number of initial colors along u
 
 	char Sch[260];		///< Color scheme used
 	int Smooth;			///< Type of texture (smoothing and so on)
 	mreal Alpha;			///< Transparency
 
-	mglTexture():n(0),Smooth(0),Alpha(1)	{}
+	mglTexture():n(0),Smooth(0),Alpha(1)
+	{	col = new mglColor[MGL_TEXTURE_COLOURS];	}
 	mglTexture(const char *cols, int smooth=0,mreal alpha=1):n(0)
-	{	Set(cols,smooth,alpha);	}
+	{	col = new mglColor[MGL_TEXTURE_COLOURS];	Set(cols,smooth,alpha);	}
 	mglTexture(const mglTexture &aa) : n(aa.n),Smooth(aa.Smooth),Alpha(aa.Alpha)
-	{	memcpy(col,aa.col,MGL_TEXTURE_COLOURS*sizeof(mglColor));	memcpy(Sch,aa.Sch,260);	}
+	{	col = new mglColor[MGL_TEXTURE_COLOURS];	memcpy(Sch,aa.Sch,260);
+		memcpy(col,aa.col,MGL_TEXTURE_COLOURS*sizeof(mglColor));	}
 #if MGL_HAVE_RVAL
 	mglTexture(mglTexture &&aa) : n(aa.n),Smooth(aa.Smooth),Alpha(aa.Alpha)
-	{	memcpy(col,aa.col,MGL_TEXTURE_COLOURS*sizeof(mglColor));	memcpy(Sch,aa.Sch,260);	}
+	{	col = aa.col;	memcpy(Sch,aa.Sch,260);	aa.col=0;	}
 #endif
+	~mglTexture()	{	if(col)	delete []col;	}
 	void Clear()	{	n=0;	}
 	void Set(const char *cols, int smooth=0,mreal alpha=1);
 	void Set(HCDT val, const char *cols);
@@ -323,6 +310,18 @@ struct MGL_EXPORT mglActivePos
 	int n;		///< position of active point in command (object id)
 };
 //-----------------------------------------------------------------------------
+#if defined(_MSC_VER)
+template class MGL_EXPORT mglStack<mglPnt>;
+template class MGL_EXPORT mglStack<mglPrim>;
+template class MGL_EXPORT std::vector<mglGroup>;
+template class MGL_EXPORT std::vector<mglText>;
+template class MGL_EXPORT std::vector<mglTexture>;
+template class MGL_EXPORT std::vector<mglGlyph>;
+template class MGL_EXPORT std::vector<mglBlock>;
+template class MGL_EXPORT std::vector<mglMatrix>;
+template class MGL_EXPORT mglStack<mglActivePos>;
+#endif
+//-----------------------------------------------------------------------------
 /// Base class for canvas which handle all basic drawing
 class MGL_EXPORT mglBase
 {
@@ -336,9 +335,11 @@ public:
 	std::string Mess;	///< Buffer for receiving messages
 	int ObjId;			///< object id for mglPrim
 	int HighId;			///< object id to be highlited
-	mglStack<mglGroup> Grp;		///< List of groups with names -- need for export
+	std::vector<mglGroup> Grp;	///< List of groups with names -- need for export
 	mglStack<mglActivePos> Act;	///< Position of active points
 	std::string PlotId;	///< Id of plot for saving filename (in GLUT window for example)
+	int BBoxX1, BBoxY1, BBoxX2, BBoxY2;	///< BBox region for exporting 2d graphics
+	std::vector<mglGlyph> UserGlf;	///< User-defined glyphs data
 
 	mreal CDef;			///< Default (current) color in texture
 	mreal AlphaDef;		///< Default value of alpha channel (transparency)
@@ -424,6 +425,7 @@ public:
 	/// Set default palette
 	inline void SetPalette(const char *colors)
 	{	Txt[0].Set(mgl_have_color(colors)?colors:MGL_DEF_PAL,-1);	}
+	inline void ResetPal()	{	CurrPal=0;	}
 	inline long GetNumPal(long id) const	{	return Txt[labs(id)/256].n;	}
 	/// Set default color scheme
 	inline void SetDefScheme(const char *colors)
@@ -476,6 +478,8 @@ public:
 	virtual mreal GetRatio() const MGL_FUNC_CONST;
 	virtual int GetWidth() const MGL_FUNC_CONST;
 	virtual int GetHeight() const MGL_FUNC_CONST;
+	/// Add user-defined glyph
+	void DefineGlyph(HCDT x, HCDT y, unsigned char id=0);
 
 	/// Set to use or not text rotation
 	inline void SetRotatedText(bool val)	{	set(val,MGL_ENABLE_RTEXT);	}
@@ -511,6 +515,8 @@ public:
 	inline void SetReduceAcc(bool val)	{	set(val, MGL_REDUCEACC);	}
 	/// Add glyph of current font to the Glf and return its position
 	long AddGlyph(int s, long j);
+	/// Add glyph to the Glf and return its position
+	long AddGlyph(unsigned char id);
 	/// Add active point as k-th element of Pnt
 	void AddActive(long k,int n=0);
 	/// Clear unused points and primitives
@@ -530,6 +536,13 @@ public:
 	inline long GetGlfNum() const		{	return Glf.size();	}
 	inline const mglPnt &GetPnt(long i) const	{	return Pnt[i];		}
 	inline long GetPntNum() const		{	return Pnt.size();	}
+	inline bool SamePnt(long i, long j) const
+	{
+		if(i<0 || j<0)	return true;
+		const mglPnt &p=Pnt[i], &q=Pnt[j];
+//		return GetWidth()>1 ? (long(p.x)==long(q.x) && long(p.y)==long(q.y)): (p.x==q.x && p.y==q.y);
+		return p.x==q.x && p.y==q.y;	
+	}
 //	inline mglPrim &GetPrm(long i)		{	return Prm[i];		}
 	inline mglPrim &GetPrm(long i, bool sort=true)
 	{	return (sort && PrmInd) ? Prm[PrmInd[i]]:Prm[i];	}
@@ -570,14 +583,20 @@ public:
 	virtual void line_plot(long p1, long p2)=0;
 	virtual void trig_plot(long p1, long p2, long p3)=0;
 	virtual void quad_plot(long p1, long p2, long p3, long p4)=0;
+	virtual void smbl_plot(long p1, char id, double size)=0;
 	virtual void Glyph(mreal x, mreal y, mreal f, int style, long icode, mreal col)=0;
 	virtual float GetGlyphPhi(const mglPnt &q, float phi)=0;
 	virtual mreal text_plot(long p,const wchar_t *text,const char *fnt,mreal size=-1,mreal sh=0,mreal  col=-('k'),bool rot=true)=0;
 	void vect_plot(long p1, long p2, mreal s=1);
+	
+	// check if visible
+	virtual bool trig_vis(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3) const =0;
+	virtual bool quad_vis(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, const mglPnt &p4) const =0;
+
 	inline mreal mark_size()	{	return MarkSize*font_factor;	}
 //	inline char last_color()	{	return last_style[1];	}
 	inline const char *last_line()	{	return last_style;	}
-	int PrmCmp(long i, long j) const MGL_FUNC_PURE;	// compare 2 primitives with indexes i,j
+	int PrmCmp(size_t i, size_t j) const MGL_FUNC_PURE;	// compare 2 primitives with indexes i,j
 	/// Check if plot termination is asked
 	bool NeedStop()	{	if(event_cb)	event_cb(event_par);	return Stop;	}
 	/// Ask to stop drawing
@@ -598,29 +617,27 @@ protected:
 	mglPoint FMax;		///< Actual upper edge after transformation formulas.
 	mglPoint Org;		///< Center of axis cross section.
 	int WarnCode;		///< Warning code
-	long *PrmInd;		///< Indexes of sorted primitives
+	size_t *PrmInd;		///< Indexes of sorted primitives
 	mglStack<mglPnt> Pnt; 	///< Internal points
 	mglStack<mglPrim> Prm;	///< Primitives (lines, triangles and so on) -- need for export
 	std::vector<mglBlock> Sub;	///< InPlot regions
 	std::vector<mglText> Ptx;	///< Text labels for mglPrim
 	std::vector<mglText> Leg;	///< Text labels for legend
 	std::vector<mglGlyph> Glf;	///< Glyphs data
-	mglStack<mglTexture> Txt;	///< Pointer to textures
+	std::vector<mglTexture> Txt;	///< Pointer to textures
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_t mutexPnt, mutexTxt, mutexLeg, mutexGlf, mutexAct, mutexDrw;
 	pthread_mutex_t mutexSub, mutexPrm, mutexPtx, mutexStk, mutexGrp, mutexClf;
 #endif
-#if MGL_HAVE_OMP
-	omp_lock_t lockClf;
-#endif
+	void *lockClf;		///< pointer to mutex for mglStack
 
 	int TernAxis;		///< Flag that Ternary axis is used
 	unsigned PDef;		///< Pen bit mask
 	mreal pPos;			///< Current position in pen mask
 	mreal PenWidth;		///< Pen width for further line plotting (must be >0 !!!)
 //	long numT;			///< Number of textures
-	mreal AmbBr;		///< Default ambient light brightness	// TODO move to mglBlock
-	mreal DifBr;		///< Default diffusive light brightness	// TODO move to mglBlock
+	mreal AmbBr;		///< Default ambient light brightness
+	mreal DifBr;		///< Default diffusive light brightness
 
 	mreal persp;		///< Original value for perspective
 	mglMatrix Bp;		///< Transformation matrix for View() and Zoom()
