@@ -1,6 +1,6 @@
 /***************************************************************************
  * canvas.h is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -31,10 +31,13 @@ struct MGL_EXPORT mglAxis
 #if MGL_HAVE_RVAL
 	mglAxis(mglAxis &&aa) : dv(aa.dv),ds(aa.ds),d(aa.d),ns(aa.ns),	t(aa.t),fact(aa.fact),stl(aa.stl),	dir(aa.dir),a(aa.a),b(aa.b),org(aa.org), v0(aa.v0),v1(aa.v1),v2(aa.v2),o(aa.o),	f(aa.f),txt(aa.txt),	ch(aa.ch),	pos(aa.pos),sh(aa.sh),inv(aa.inv)	{}
 #endif
-	inline void AddLabel(const wchar_t *lbl, mreal v)
-	{	txt.push_back(mglText(lbl,"",v));	}
+
+	const mglAxis &operator=(const mglAxis &aa)
+	{	dv=aa.dv; ds=aa.ds; d=aa.d; ns=aa.ns; t=aa.t; fact=aa.fact; stl=aa.stl;
+		dir=aa.dir; a=aa.a; b=aa.b; org=aa.org; v0=aa.v0; v1=aa.v1; v2=aa.v2; o=aa.o;
+		f=aa.f; txt=aa.txt; 	ch=aa.ch; pos=aa.pos; sh=aa.sh; inv=aa.inv;	return aa;	}
 	inline void AddLabel(const std::wstring &lbl, mreal v)
-	{	txt.push_back(mglText(lbl,v));	}
+	{	if(mgl_isfin(v))	txt.push_back(mglText(L' '+lbl+L' ',v));	}
 	inline void Clear()
 	{	dv=ds=d=v0=v1=v2=sh=0;	o=NAN;	ns=f=0;	pos = 't';	inv=false;
 		fact.clear();	stl.clear();	t.clear();	txt.clear();	}
@@ -70,6 +73,9 @@ struct MGL_EXPORT mglDrawReg
 #if MGL_HAVE_RVAL
 	mglDrawReg(mglDrawReg &&aa) : PDef(aa.PDef),angle(aa.angle),ObjId(aa.ObjId),PenWidth(aa.PenWidth),pPos(aa.pPos) ,x1(aa.x1),x2(aa.x2),y1(aa.y1),y2(aa.y2)	{}
 #endif
+	inline void copy(const mglPrim &p)
+	{	PDef = p.n3;	pPos = p.s;	ObjId = p.id;	PenWidth=p.w;	angle = p.angl;
+		if(p.type==2 || p.type==3) PDef = p.m;	}
 	inline const mglDrawReg &operator=(const mglDrawReg &aa)
 	{	memcpy(this,&aa,sizeof(mglDrawReg));	return aa;	}
 	union
@@ -99,8 +105,11 @@ struct MGL_EXPORT mglDrawDat
 	std::vector<mglBlock> Sub;	///< InPlot regions
 	std::vector<mglText> Ptx;	///< Text labels for mglPrim
 	std::vector<mglGlyph> Glf;	///< Glyphs data
-	mglStack<mglTexture> Txt;	///< Pointer to textures
+	std::vector<mglTexture> Txt;	///< Pointer to textures
 };
+#if defined(_MSC_VER)
+template class MGL_EXPORT std::vector<mglDrawDat>;
+#endif
 //-----------------------------------------------------------------------------
 union mglRGBA	{	uint32_t c; unsigned char r[4];	};
 //-----------------------------------------------------------------------------
@@ -143,6 +152,8 @@ using mglBase::Light;
 
 	/// Put further plotting in cell of stick rotated on angles tet, phi
 	void StickPlot(int num, int i, mreal tet, mreal phi);
+	/// Put further plotting in cell of stick sheared on sx, sy
+	void ShearPlot(int num, int i, mreal sx, mreal sy, mreal xd, mreal yd);
 	/// Put further plotting in some region of whole frame surface.
 	inline void InPlot(mreal x1,mreal x2,mreal y1,mreal y2,bool rel=true)
 	{	InPlot(B,x1,x2,y1,y2,rel);	}
@@ -153,6 +164,8 @@ using mglBase::Light;
 	void Title(const wchar_t *title,const char *stl="#",mreal size=-2);
 	/// Set aspect ratio for further plotting.
 	void Aspect(mreal Ax,mreal Ay,mreal Az);
+	/// Shear a further plotting.
+	void Shear(mreal Sx,mreal Sy);
 	/// Rotate a further plotting.
 	void Rotate(mreal TetX,mreal TetZ,mreal TetY=0);
 	/// Rotate a further plotting around vector {x,y,z}.
@@ -179,6 +192,9 @@ using mglBase::Light;
 	int GetHeight() const	{	return Height;	}
 	/// Combine plots from 2 canvases. Result will be saved into this.
 	void Combine(const mglCanvas *gr);
+	/// Set boundary box for export graphics into 2D file formats
+	void SetBBox(int x1=0, int y1=0, int x2=-1, int y2=-1)
+	{	BBoxX1=x1;	BBoxY1=y1;	BBoxX2=x2;	BBoxY2=y2;	}
 
 	/// Rasterize current plot and set it as background image
 	void Rasterize();
@@ -198,8 +214,8 @@ using mglBase::Light;
 	/// Set object/subplot id
 	inline void SetObjId(long id)	{	ObjId = id;	}
 	/// Get object id
-	inline int GetObjId(long xs,long ys) const	
-	{	register long i=xs+Width*ys;	return (i>=0 && i<Width*Height)?OI[i]:-1;	}
+	inline int GetObjId(long xs,long ys) const
+	{	long i=xs+Width*ys;	return (i>=0 && i<Width*Height)?OI[i]:-1;	}
 	/// Get subplot id
 	int GetSplId(long xs,long ys) const MGL_FUNC_PURE;
 	/// Check if there is active point or primitive (n=-1)
@@ -331,6 +347,8 @@ using mglBase::Light;
 	void PreparePrim(int fast);
 	inline uint32_t GetPntCol(long i) const	{	return pnt_col[i];	}
 	inline uint32_t GetPrmCol(long i, bool sort=true) const	{	return GetColor(GetPrm(i, sort));	}
+	/// Set the size of semi-transparent area around lines, marks, ...
+	inline void SetPenDelta(float d)	{	pen_delta = 1.5*fabs(d);	}
 
 protected:
 	mreal Delay;		///< Delay for animation in seconds
@@ -358,7 +376,7 @@ protected:
 	int Depth;			///< Depth of the image
 	mreal inW, inH;		///< Width and height of last InPlot
 	mreal inX, inY;		///< Coordinates of last InPlot
-	mglLight light[10];	///< Light sources	// TODO move to mglBlock
+	mglLight light[10];	///< Light sources
 	mreal FogDist;		///< Inverse fog distance (fog ~ exp(-FogDist*Z))
 	mreal FogDz;		///< Relative shift of fog
 
@@ -397,6 +415,7 @@ protected:
 	void trig_plot(long p1, long p2, long p3);
 	void quad_plot(long p1, long p2, long p3, long p4);
 	void Glyph(mreal x, mreal y, mreal f, int style, long icode, mreal col);
+	void smbl_plot(long p1, char id, double size);
 	mreal text_plot(long p,const wchar_t *text,const char *fnt,mreal size=-1,mreal sh=0,mreal  col=-('k'), bool rot=true);
 
 	void add_prim(mglPrim &a);	///< add primitive to list
@@ -409,8 +428,18 @@ protected:
 	void arrow_draw(long n1, long n2, char st, float ll);
 	void arrow_plot_3d(long n1, long n2, char st, float ll);
 	void glyph_draw(const mglPrim &P, mglDrawReg *d);
+	void glyph_draw_new(const mglPrim &P, mglDrawReg *d);
 	bool IsSame(const mglPrim &pr,mreal wp,mglColor cp,int st);
 
+	// check if visible
+	bool trig_vis(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3) const;
+	bool quad_vis(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, const mglPnt &p4) const;
+
+	// functions for glyph drawing
+	virtual void glyph_fill(mreal phi, const mglPnt &p, mreal f, const mglGlyph &g, const mglDrawReg *d);
+	void glyph_wire(mreal phi, const mglPnt &p, mreal f, const mglGlyph &g, const mglDrawReg *d);
+	void glyph_line(mreal phi, const mglPnt &p, mreal f, bool solid, const mglDrawReg *d);
+
 	// restore normalized coordinates from screen ones
 	mglPoint RestorePnt(mglPoint ps, bool norm=false) const MGL_FUNC_PURE;
 
@@ -438,9 +467,10 @@ private:
 	std::vector<mglMatrix> stack;	///< stack for transformation matrices
 	GifFileType *gif;
 	mreal fscl,ftet;	///< last scale and rotation for glyphs
-	long forg;		///< original point (for directions)
+	long forg;			///< original point (for directions)
 	size_t grp_counter;	///< Counter for StartGroup(); EndGroup();
-	mglMatrix Bt;	///< temporary matrix for text
+	mglMatrix Bt;		///< temporary matrix for text
+	float pen_delta;	///< delta pen width (dpw) -- the size of semi-transparent region for lines, marks, ...
 
 	/// Draw generic colorbar
 	void colorbar(HCDT v, const mreal *s, int where, mreal x, mreal y, mreal w, mreal h, bool text);
@@ -465,11 +495,6 @@ private:
 	/// Set coordinate and add the point, return its id
 	long setPp(mglPnt &q, const mglPoint &p);
 
-	// functions for glyph drawing
-	void glyph_fill(const mglMatrix *M, const mglPnt &p, mreal f, const mglGlyph &g, const mglDrawReg *d);
-	void glyph_wire(const mglMatrix *M, const mglPnt &p, mreal f, const mglGlyph &g, const mglDrawReg *d);
-	void glyph_line(const mglMatrix *M, const mglPnt &p, mreal f, bool solid, const mglDrawReg *d);
-
 	// fill pixel for given primitive
 	void mark_pix(long i,long j,const mglPnt &p, char type, mreal size, mglDrawReg *d);
 	void arrow_pix(long i,long j,const mglPnt &p1, const mglPnt &p2, char st, mreal size, const mglDrawReg *d);
@@ -481,8 +506,6 @@ private:
 	void glyph_fpix(long i,long j,const mglMatrix *M, const mglPnt &p, mreal f, const mglGlyph &g, const mglDrawReg *d);
 	void glyph_wpix(long i,long j,const mglMatrix *M, const mglPnt &p, mreal f, const mglGlyph &g, const mglDrawReg *d);
 	void glyph_lpix(long i,long j,const mglMatrix *M, const mglPnt &p, mreal f, bool solid, const mglDrawReg *d);
-
-
 };
 //-----------------------------------------------------------------------------
 struct mglThreadG
@@ -496,4 +519,11 @@ struct mglThreadG
 /// Start several thread for the task
 void mglStartThread(void (mglCanvas::*func)(long i, long n), mglCanvas *gr, long n);
 //-----------------------------------------------------------------------------
+inline mreal get_persp(float pf, float z, float Depth)
+//{	return (1-pf)/(1-pf*z/Depth);	}
+{	return (1-pf/1.37)/(1-pf*z/Depth);	}
+inline mreal get_pfact(float pf, float Depth)
+//{	return pf/(1-pf)/Depth;	}
+{	return pf/(1-pf/1.37)/Depth;	}
+//-----------------------------------------------------------------------------
 #endif
