@@ -163,15 +163,18 @@ using mglDataA::Momentum;
 	/// Crop the data
 	inline void Crop(long n1, long n2,char dir='x')
 	{	mgl_data_crop(this,n1,n2,dir);	}
+	/// Crop the data to be most optimal for FFT (i.e. to closest value of 2^n*3^m*5^l)
+	inline void Crop(const char *how="235x")
+	{	mgl_data_crop_opt(this, how);	}
 	/// Insert data rows/columns/slices
 	inline void Insert(char dir, long at=0, long num=1)
 	{	mgl_data_insert(this,dir,at,num);	}
 	/// Delete data rows/columns/slices
 	inline void Delete(char dir, long at=0, long num=1)
 	{	mgl_data_delete(this,dir,at,num);	}
-	/// Remove rows with duplicate values in column id
-	inline void Clean(long id)
-	{	mgl_data_clean(this,id);	}
+	/// Remove rows with duplicate values in column clmn
+	inline void Clean(long clmn)
+	{	mgl_data_clean(this,clmn);	}
 	/// Join with another data array
 	inline void Join(const mglDataA &d)
 	{	mgl_data_join(this,&d);	}
@@ -276,6 +279,13 @@ using mglDataA::Momentum;
 	{	return mglData(true,mgl_data_subdata_ext(this,&xx,&yy,0));	}
 	inline mglData SubData(const mglDataA &xx) const
 	{	return mglData(true,mgl_data_subdata_ext(this,&xx,0,0));	}
+	/// Get data from sections ids, separated by value val along specified direction.
+	/** If section id is negative then reverse order is used (i.e. -1 give last section). */
+	inline mglData Section(const mglDataA &ids, char dir='y', mreal val=NAN) const
+	{	return mglData(true,mgl_data_section(this,&ids,dir,val));	}
+	inline mglData Section(long id, char dir='y', mreal val=NAN) const
+	{	return mglData(true,mgl_data_section_val(this,id,dir,val));	}
+
 	/// Get trace of the data array
 	inline mglData Trace() const
 	{	return mglData(true,mgl_data_trace(this));	}
@@ -308,14 +318,19 @@ using mglDataA::Momentum;
 	inline mglData Evaluate(const mglData &idat, const mglData &jdat, const mglData &kdat, bool norm=true) const
 	{	return mglData(true,mgl_data_evaluate(this,&idat,&jdat,&kdat,norm));	}
 	/// Find roots for set of nonlinear equations defined by textual formula
-	inline mglData Roots(const char *func, char var='x') const
-	{	return mglData(true,mgl_data_roots(func, this, var));	}
+	inline mglData Roots(const char *eq, char var='x') const
+	{	return mglData(true,mgl_data_roots(eq, this, var));	}
 	/// Find correlation with another data arrays
 	inline mglData Correl(const mglDataA &dat, const char *dir) const
 	{	return mglData(true,mgl_data_correl(this,&dat,dir));	}
 	/// Find auto correlation function
 	inline mglData AutoCorrel(const char *dir) const
 	{	return mglData(true,mgl_data_correl(this,this,dir));	}
+	/// Get curves, separated by NAN, for maximal values of array d as function of x coordinate.
+	/** Noises below lvl amplitude are ignored.
+	 * Parameter dy \in [0,ny] set the "attraction" distance of points to curve. */
+	inline mglData Detect(mreal lvl, mreal dj, mreal di=0, mreal min_len=0) const
+	{	return mglData(true,mgl_data_detect(this,lvl,dj,di,min_len));	}
 
 	/// Cumulative summation the data in given direction or directions
 	inline void CumSum(const char *dir)	{	mgl_data_cumsum(this,dir);	}
@@ -323,6 +338,9 @@ using mglDataA::Momentum;
 	inline void Integral(const char *dir)	{	mgl_data_integral(this,dir);	}
 	/// Differentiate the data in given direction or directions
 	inline void Diff(const char *dir)	{	mgl_data_diff(this,dir);	}
+	/// Differentiate the parametrically specified data along direction v1
+	inline void Diff(const mglDataA &v1)
+	{	mgl_data_diff_par(this,&v1,0,0);	}
 	/// Differentiate the parametrically specified data along direction v1 with v2=const
 	inline void Diff(const mglDataA &v1, const mglDataA &v2)
 	{	mgl_data_diff_par(this,&v1,&v2,0);	}
@@ -340,6 +358,12 @@ using mglDataA::Momentum;
 	inline void Mirror(const char *dir)		{	mgl_data_mirror(this,dir);	}
 	/// Sort rows (or slices) by values of specified column
 	inline void Sort(long idx, long idy=-1)	{	mgl_data_sort(this,idx,idy);	}
+	/// Return dilated array of 0 or 1 for data values larger val
+	inline void Dilate(mreal val=1, long step=1)
+	{	mgl_data_dilate(this, val, step);	}
+	/// Return eroded array of 0 or 1 for data values larger val
+	inline void Erode(mreal val=1, long step=1)
+	{	mgl_data_erode(this, val, step);	}
 
 	/// Set as the data envelop
 	inline void Envelop(char dir='x')
@@ -372,7 +396,12 @@ using mglDataA::Momentum;
 	inline void SinFFT(const char *dir)	{	mgl_data_sinfft(this,dir);	}
 	/// Apply Cos-Fourier transform
 	inline void CosFFT(const char *dir)	{	mgl_data_cosfft(this,dir);	}
-	/// Fill data by 'x'/'k' samples for Hankel ('h') or Fourier ('f') transform
+	/// Fill data by coordinates/momenta samples for Hankel ('h') or Fourier ('f') transform
+	/** Parameter \a how may contain:
+	 * ‘x‘,‘y‘,‘z‘ for direction (only one will be used),
+	 * ‘k‘ for momenta samples,
+	 * ‘h‘ for Hankel samples,
+	 * ‘f‘ for Cartesian/Fourier samples (default). */
 	inline void FillSample(const char *how)
 	{	mgl_data_fill_sample(this,how);	}
 	/// Apply wavelet transform
@@ -442,13 +471,13 @@ using mglDataA::Momentum;
 	mreal vthr(long i) const {	return a[i];	}
 	// add for speeding up !!!
 	mreal dvx(long i,long j=0,long k=0) const
-	{   register long i0=i+nx*(j+ny*k);
+	{   long i0=i+nx*(j+ny*k);
 		return i>0? (i<nx-1? (a[i0+1]-a[i0-1])/2:a[i0]-a[i0-1]) : a[i0+1]-a[i0];	}
 	mreal dvy(long i,long j=0,long k=0) const
-	{   register long i0=i+nx*(j+ny*k);
+	{   long i0=i+nx*(j+ny*k);
 		return j>0? (j<ny-1? (a[i0+nx]-a[i0-nx])/2:a[i0]-a[i0-nx]) : a[i0+nx]-a[i0];}
 	mreal dvz(long i,long j=0,long k=0) const
-	{   register long i0=i+nx*(j+ny*k), n=nx*ny;
+	{   long i0=i+nx*(j+ny*k), n=nx*ny;
 		return k>0? (k<nz-1? (a[i0+n]-a[i0-n])/2:a[i0]-a[i0-n]) : a[i0+n]-a[i0];	}
 };
 //-----------------------------------------------------------------------------
@@ -512,9 +541,9 @@ inline mglData mglQO3d(const char *ham, const mglDataA &ini_re, const mglDataA &
 /// Finds ray with starting point r0, p0 (and prepares ray data for mglQO2d)
 inline mglData mglRay(const char *ham, mglPoint r0, mglPoint p0, mreal dt=0.1, mreal tmax=10)
 {	return mglData(true, mgl_ray_trace(ham, r0.x, r0.y, r0.z, p0.x, p0.y, p0.z, dt, tmax));	}
-/// Saves result of ODE solving (|u|^2) for "Hamiltonian" ham with initial conditions ini
-inline mglData mglODE(const char *df, const char *var, const mglDataA &ini, mreal dt=0.1, mreal tmax=10)
-{	return mglData(true, mgl_ode_solve_str(df,var, &ini, dt, tmax));	}
+/// Saves result of ODE solving for var complex variables with right part func (separated by ';') and initial conditions x0 over time interval [0,tmax] with time step dt
+inline mglData mglODE(const char *func, const char *var, const mglDataA &ini, mreal dt=0.1, mreal tmax=10)
+{	return mglData(true, mgl_ode_solve_str(func,var, &ini, dt, tmax));	}
 //-----------------------------------------------------------------------------
 /// Get array as solution of tridiagonal system of equations a[i]*x[i-1]+b[i]*x[i]+c[i]*x[i+1]=d[i]
 /** String \a how may contain:
@@ -536,6 +565,12 @@ inline mglData mglTriangulation(const mglDataA &x, const mglDataA &y, const mglD
 {	return mglData(true,mgl_triangulation_3d(&x,&y,&z));	}
 inline mglData mglTriangulation(const mglDataA &x, const mglDataA &y)
 {	return mglData(true,mgl_triangulation_2d(&x,&y));	}
+/// Get curves, separated by NAN, for maximal values of array d as function of x coordinate.
+/** Noises below lvl amplitude are ignored.
+ * Parameter dy \in [0,ny] set the "attraction" distance of points to curve. */
+inline mglData mglDetect(const mglDataA &d, mreal lvl, mreal dj, mreal di=0, mreal min_len=0)
+{	return mglData(true,mgl_data_detect(&d, lvl, dj, di, min_len));	}
+//-----------------------------------------------------------------------------
 /// Get array which is n-th pairs {x[i],y[i]} for iterated function system (fractal) generated by A
 inline mglData mglIFS2d(const mglDataA &A, long n, long skip=20)
 {	return mglData(true,mgl_data_ifs_2d(&A,n,skip));	}
@@ -545,7 +580,6 @@ inline mglData mglIFS3d(const mglDataA &A, long n, long skip=20)
 /// Get array which is n-th points {x[i],y[i],z[i]} for iterated function system (fractal) defined in *.ifs file 'fname' and named as 'name'
 inline mglData mglIFSfile(const char *fname, const char *name, long n, long skip=20)
 {	return mglData(true,mgl_data_ifs_file(fname,name,n,skip));	}
-/// Get array which is n-th pairs {x[i],y[i]} for flame fractal generated by A with functions F
 /// Get array which is n-th pairs {x[i],y[i]} for Flame fractal generated by A with functions F
 /** NOTE: A.nx must be >= 7 and F.nx >= 2 and F.nz=A.ny.
  * F[0,i,j] denote function id. F[1,i,j] give function weight, F(2:5,i,j) provide function parameters.
@@ -598,7 +632,7 @@ public:
 #endif
 };
 //-----------------------------------------------------------------------------
-/// Class which present variable as data array
+/// Class which present equidistantly distributed data
 class MGL_EXPORT mglDataV : public mglDataA
 {
 	long nx;	///< number of points in 1st dimensions ('x' dimension)
@@ -656,7 +690,7 @@ public:
 	mreal value(mreal x,mreal y=0,mreal z=0) const	{	return a0+di*x+dj*y+dk*z;	}
 	mreal v(long i,long j=0,long k=0) const		{	return a0+di*i+dj*j+dk*k;	}
 	mreal vthr(long ii) const
-	{	register long i=ii%nx, j=(ii/nx)%ny, k=ii/(nx*ny);	return a0+di*i+dj*j+dk*k;	}
+	{	long i=ii%nx, j=(ii/nx)%ny, k=ii/(nx*ny);	return a0+di*i+dj*j+dk*k;	}
 	// add for speeding up !!!
 	mreal dvx(long ,long =0,long =0) const	{	return di;	}
 	mreal dvy(long ,long =0,long =0) const	{	return dj;	}
@@ -717,7 +751,7 @@ public:
 	mreal v(long i,long j=0,long k=0) const
 	{	return di*(i<nx/2?i:i-nx)+dj*(j<ny/2?j:j-ny)+dk*(k<nz/2?k:k-nz);	}
 	mreal vthr(long ii) const
-	{	register long i=ii%nx, j=(ii/nx)%ny, k=ii/(nx*ny);
+	{	long i=ii%nx, j=(ii/nx)%ny, k=ii/(nx*ny);
 		return di*(i<nx/2?i:i-nx)+dj*(j<ny/2?j:j-ny)+dk*(k<nz/2?k:k-nz);	}
 	// add for speeding up !!!
 	mreal dvx(long ,long =0,long =0) const	{	return di;	}
@@ -725,7 +759,7 @@ public:
 	mreal dvz(long ,long =0,long =0) const	{	return dk;	}
 };
 //-----------------------------------------------------------------------------
-/// Class which present variable as data array
+/// Class which present function as data array
 class MGL_EXPORT mglDataF : public mglDataA
 {
 	long nx;	///< number of points in 1st dimensions ('x' dimension)
@@ -778,7 +812,9 @@ public:
 	mreal valueD(mreal i,mreal j=0,mreal k=0, mreal *di=0,mreal *dj=0,mreal *dk=0) const
 	{
 		mreal res=0, x=v1.x+dx*i, y=v1.y+dy*j, z=v1.z+dz*k;
-		if(di)	*di = 0;	if(dj)	*dj = 0;	if(dk)	*dk = 0;
+		if(di)	*di = 0;
+		if(dj)	*dj = 0;
+		if(dk)	*dk = 0;
 		if(dfunc)
 		{
 			res = dfunc(x,y,z, par);
@@ -805,7 +841,7 @@ public:
 	}
 	/// Copy data from other mglDataV variable
 	inline const mglDataF &operator=(const mglDataF &d)
-	{	nx=d.nx;	ny=d.ny;	nz=d.nz;	v1=d.v1;	v2=d.v2;	setD();
+	{	nx=d.nx;	ny=d.ny;	nz=d.nz;	v1=d.v1;	v2=d.v2;	setD();	mgl_delete_expr(ex);
 		str=d.str;	ex = mgl_create_expr(str.c_str());	dfunc=d.dfunc;	par=d.par;	return d;	}
 	/// Get the value in given cell of the data without border checking
 	mreal v(long i,long j=0,long k=0) const
@@ -846,14 +882,16 @@ public:
 	}
 };
 //-----------------------------------------------------------------------------
-/// Class which present variable as data array
+/// Class which present column of another data as data array
 class MGL_EXPORT mglDataT : public mglDataA
 {
 	const mglDataA &dat;
 	long ind;
+	const mglDataT &operator=(const mglDataT &d)	{	return d;	}
 public:
 	mglDataT(const mglDataT &d) : dat(d.dat), ind(d.ind)	{	s = d.s;	}
 	mglDataT(const mglDataA &d, long col=0) : dat(d), ind(col)	{}
+	mglDataT(HCDT d, long col=0) : dat(*d), ind(col)	{}
 #if MGL_HAVE_RVAL
 	mglDataT(mglDataT &&d):dat(d.dat),ind(d.ind)
 	{	s=d.s;	temp=d.temp;	func=d.func;	o=d.o;	d.func=0;	}
@@ -894,13 +932,16 @@ public:
 	{	return 0;	}
 };
 //-----------------------------------------------------------------------------
+/// Class which present row of another data as data array
 class MGL_EXPORT mglDataR : public mglDataA
 {
 	const mglDataA &dat;
 	long ind;
+	const mglDataR &operator=(const mglDataR &d)	{	return d;	}
 public:
 	mglDataR(const mglDataR &d) : dat(d.dat), ind(d.ind)	{	s = d.s;	}
 	mglDataR(const mglDataA &d, long row=0) : dat(d), ind(row)	{}
+	mglDataR(HCDT d, long row=0) : dat(*d), ind(row)	{}
 #if MGL_HAVE_RVAL
 	mglDataR(mglDataR &&d):dat(d.dat),ind(d.ind)
 	{	s=d.s;	temp=d.temp;	func=d.func;	o=d.o;	d.func=0;	}
@@ -941,7 +982,7 @@ public:
 	{	return 0;	}
 };
 //-----------------------------------------------------------------------------
-/// Class for replacement of std::vector
+/// Class which present std::vector as data array
 class MGL_EXPORT mglDataS : public mglDataA
 {
 public:
@@ -967,14 +1008,14 @@ public:
 	{	return mglSpline3s(dat.data(),dat.size(),1,1,x,0,0);	}
 
 	mreal v(long i,long =0,long =0) const		{	return dat[i];	}
-	mreal vthr(long i) const	{	return dat[i];	};
+	mreal vthr(long i) const	{	return dat[i];	}
 	long GetNx() const	{	return dat.size();	}
 	long GetNy() const	{	return 1;	}
 	long GetNz() const	{	return 1;	}
 	mreal dvx(long i,long =0,long =0) const
 	{	return i>0? (i<long(dat.size()-1)? (dat[i+1]-dat[i-1])/2:dat[i]-dat[i-1]) : dat[i+1]-dat[i];	}
-	mreal dvy(long ,long =0,long =0) const	{	return 1;	}
-	mreal dvz(long ,long =0,long =0) const	{	return 1;	}
+	mreal dvy(long ,long =0,long =0) const	{	return 0;	}
+	mreal dvz(long ,long =0,long =0) const	{	return 0;	}
 };
 //-----------------------------------------------------------------------------
 #endif
