@@ -19,7 +19,7 @@
 #include <FL/Fl_Int_Input.H>
 #include <FL/Fl_Value_Slider.H>
 #include <FL/fl_draw.H>
-#include "udav.h"
+#include "mgllab.h"
 //-----------------------------------------------------------------------------
 Fl_Callback input_cb;
 //-----------------------------------------------------------------------------
@@ -33,6 +33,7 @@ Fl_Data_Table::Fl_Data_Table(int x, int y, int w, int h, const char *l) : Fl_Tab
 	input->callback(input_cb, (void*)this);
 	input->when(FL_WHEN_ENTER_KEY_ALWAYS);
 	input->maximum_size(16);
+	sl = 0;	row=col=-1;
 //	(new Fl_Box(9999,9999,0,0))->hide();  // HACK: prevent flickering in Fl_Scroll
 	end();
 }
@@ -40,7 +41,7 @@ Fl_Data_Table::Fl_Data_Table(int x, int y, int w, int h, const char *l) : Fl_Tab
 // Handle drawing all cells in table
 void Fl_Data_Table::draw_cell(TableContext context, int R, int C, int X, int Y, int W, int H)
 {
-	static char s[32];
+	static char s[64];
 	fl_push_clip(X, Y, W, H);
 	switch ( context )
 	{
@@ -62,9 +63,25 @@ void Fl_Data_Table::draw_cell(TableContext context, int R, int C, int X, int Y,
 		fl_pop_clip();
 		fl_push_clip(X+3, Y+3, W-6, H-6);
 		fl_font(FL_HELVETICA, 14);
-		fl_color(FL_BLACK);
-		if(mgl_isnan(data[C+nx*R]))	strcpy(s,"nan");
-		else	snprintf(s,32,"%g",data[C+nx*R]);
+		if(mgl_isnan(data->v(C,R,sl)))	strcpy(s,"nan");
+		else if(mgl_isbad(data->v(C,R,sl)))	strcpy(s,data->v(C,R,sl)>0?"inf":"-inf");
+		else	strncpy(s,mgl_str_num(data->vc(C,R,sl)).c_str(),64);
+		{	dual vc = data->vc(C,R,sl);
+			mreal v = data->v(C,R,sl);
+			std::vector<mreal> vn;
+			if(C>0)	vn.push_back(data->v(C-1,R,sl));
+			if(R>0)	vn.push_back(data->v(C,R-1,sl));
+			if(C<data->GetNx()-1)	vn.push_back(data->v(C+1,R,sl));
+			if(R<data->GetNy()-1)	vn.push_back(data->v(C,R+1,sl));
+			bool v1=true, v2=true;
+			for(size_t i=0;i<vn.size();i++)	{	if(vn[i]<=v)	v1=false;	if(vn[i]>=v)	v2=false;	}
+			if(v2)	fl_color(FL_MAGENTA);
+			else if(v1)	fl_color(FL_CYAN);
+			else if(real(vc)>0)		fl_color(FL_RED);
+			else if(real(vc)<0)	fl_color(FL_BLUE);
+			else if(imag(vc)>0)	fl_color(FL_DARK_MAGENTA);
+			else if(imag(vc)<0)	fl_color(FL_DARK_CYAN);
+			else	fl_color(FL_BLACK);	}
 		fl_draw(s, X+3, Y+3, W-6, H-6, FL_ALIGN_RIGHT);
 		break;
 	case CONTEXT_RC_RESIZE:
@@ -85,19 +102,16 @@ void Fl_Data_Table::cell_click()
 
     if(context==CONTEXT_CELL)
 	{
-		if (input->visible())	//input->do_callback();
-		{
-			const char *s = input->value();
-			data[col + nx*row] = (s[0]==0 || !strcmp(s,"nan")) ? NAN : atof(s);
-		}
-		row = R;		col = C;
+		if (input->visible())	set_value();
+		row = R;	col = C;
 		int XX,YY,WW,HH;
 		find_cell(CONTEXT_CELL, R, C, XX, YY, WW, HH);
 		input->resize(XX,YY,WW,HH);
-		char s[32];
-		if(mgl_isnan(data[C+nx*R]))	strcpy(s,"nan");
-		else	snprintf(s,32,"%g",data[C+nx*R]);
-		input->value(s);	input->show();
+		std::string s;
+		if(mgl_isnan(data->v(C,R,sl)))	s = "nan";
+		else if(mgl_isbad(data->v(C,R,sl)))	s = data->v(C,R,sl)>0?"inf":"-inf";
+		else	s = mgl_str_num(data->vc(C,R,sl));
+		input->value(s.c_str());	input->show();
 		input->take_focus();
 	}
 }
@@ -105,6 +119,19 @@ void Fl_Data_Table::cell_click()
 void Fl_Data_Table::set_value()
 {
 	const char *s = input->value();
-	data[col + nx*row] = (s[0]==0 || !strcmp(s,"nan")) ? NAN : atof(s);
+	if(s[0]==0 || !strcmp(s,"nan"))	data->set_v(NAN, col,row,sl);
+	else if(!strcmp(s,"inf"))	data->set_v(INFINITY, col,row,sl);
+	else if(!strcmp(s,"-inf"))	data->set_v(-INFINITY, col,row,sl);
+	else
+	{
+		dual v = mgl_atoc(s,true);
+		if(imag(v)==0)	data->set_v(real(v), col,row,sl);
+		else
+		{
+			HADT c = dynamic_cast<HADT>(data);
+			if(c)	c->a[col+c->nx*(row+c->ny*sl)] = v;
+			else	data->set_v(abs(v),col,row,sl);
+		}
+	}
 }
 //-----------------------------------------------------------------------------
