@@ -32,13 +32,14 @@ MGL_NO_EXPORT inline struct tm *mgl_localtime (const time_t *clock, tm *result,
 //-----------------------------------------------------------------------------
 long MGL_EXPORT mgl_have_color(const char *stl)
 {
-	long j=0;
-	if(stl)	for(long i=0;stl[i];i++)
-	{
-		if(strchr(MGL_COLORS,stl[i]))	j++;
-		if(stl[i]=='{' && stl[i+1]=='x')	j++;
-	}
-	return j;
+	return mgl_get_num_color(stl,0);
+// 	long j=0;
+// 	if(stl)	for(long i=0;stl[i];i++)
+// 	{
+// 		if(strchr(MGL_COLORS,stl[i]))	j++;
+// 		if(stl[i]=='{' && stl[i+1]=='x')	j++;
+// 	}
+// 	return j;
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_wcstrim(wchar_t *str)
@@ -47,17 +48,10 @@ void MGL_EXPORT mgl_wcstrim(wchar_t *str)
 	size_t n=mgl_wcslen(str), k, i;
 	for(k=0;k<n;k++)	if(str[k]>' ')	break;
 	for(i=n;i>k;i--)	if(str[i-1]>' ')	break;
-	memmove(str, str+k, (i-k)*sizeof(wchar_t));
+	for(size_t j=0;j<i-k;j++)	str[j]=str[j+k];
 	str[i-k]=0;
 }
 //-----------------------------------------------------------------------------
-size_t MGL_EXPORT mgl_wcslen(const wchar_t *str)
-{
-	long i=0;
-	if(str)	while(str[i])	i++;
-	return i;
-}
-//-----------------------------------------------------------------------------
 //		Ticks setup
 //-----------------------------------------------------------------------------
 void mglCanvas::SetAxisStl(const char *stl, const char *tck, const char *sub)
@@ -110,8 +104,8 @@ void mglCanvas::SetTicksVal(char dir, HCDT v, const wchar_t *lbl, bool add)
 	else	aa.txt.clear();
 	if(!v || !lbl || !lbl[0])	{	aa.f = 0;	return;	}
 	aa.f = 2;	aa.ns=0;	aa.ds=0;
-	register long i,j,l=0,n=v->GetNx();
-	for(i=j=l=0;i<n && lbl[j];j++)
+	long i=0,l=0,n=v->GetNx();
+	for(long j=0;i<n && lbl[j];j++)
 	{
 		if(lbl[j]=='\n')
 		{
@@ -132,8 +126,8 @@ void mglCanvas::SetTicksVal(char dir, HCDT v, const char *lbl, bool add)
 //-----------------------------------------------------------------------------
 void mglCanvas::SetTicksVal(char dir, const wchar_t *lbl, bool add)
 {
-	register long i,j,len=mgl_wcslen(lbl);
-	for(i=0,j=1;j<len;j++)
+	long i=0,len=mgl_wcslen(lbl);
+	for(long j=1;j<len;j++)
 		if(lbl[j]=='\n' || (lbl[j]=='n' && lbl[j-1]=='\\'))	i++;
 	if(i>63)	i=63;
 	mglData val(i+1);	val.Fill(Min.x,Max.x);
@@ -142,8 +136,8 @@ void mglCanvas::SetTicksVal(char dir, const wchar_t *lbl, bool add)
 //-----------------------------------------------------------------------------
 void mglCanvas::SetTicksVal(char dir, const char *lbl, bool add)
 {
-	register long i,j,len=strlen(lbl);
-	for(i=0,j=1;j<len;j++)
+	long i=0,len=strlen(lbl);
+	for(long j=1;j<len;j++)
 		if(lbl[j]=='\n' || (lbl[j]=='n' && lbl[j-1]=='\\'))	i++;
 	if(i>63)	i=63;
 	mglData val(i+1);	val.Fill(Min.x,Max.x);
@@ -160,8 +154,8 @@ void mglCanvas::SetTicksVal(char dir, HCDT v, const wchar_t **lbl, bool add)
 	else	aa.txt.clear();
 	if(!v || !lbl)	{	aa.f = 0;	return;	}
 	aa.f = 2;	aa.ns=0;	aa.ds=0;
-	register long i,n=v->GetNx();
-	for(i=0;i<n;i++)	aa.AddLabel(lbl[i],v->v(i));
+	long n=v->GetNx();
+	for(long i=0;i<n;i++)	aa.AddLabel(lbl[i],v->v(i));
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::SetTicksVal(char dir, HCDT v, const char **lbl, bool add)
@@ -237,26 +231,32 @@ void mglCanvas::SetTickTime(char dir, mreal d, const char *t)
 		t = abs(t1.tm_yday-t2.tm_yday)>1 ? "%x" : "%X";
 		if(abs(t1.tm_year-t2.tm_year)>3)	t = "%Y";
 	}
-	if(d==0)	// try to select opimal step
+	mreal ds=0;
+	if(d==0)	// try to select optimal step
 	{
-		// TODO add subticks for drawing
-		if(abs(t1.tm_year-t2.tm_year)>1)
-			d = 365.25*24*3600*mgl_adj_val(abs(t1.tm_year-t2.tm_year));	// number of second in year NOTE: improve it
+		if(abs(t1.tm_year-t2.tm_year)>1)	// number of second in year NOTE: improve it
+		{	d = 365.25*24*3600*mgl_adj_val(abs(t1.tm_year-t2.tm_year),&ds);
+			ds *= 365.25*24*3600;	}
 		// NOTE here should be months ... but it is too unregular ... so omit it now
 // 		else if(t1.tm_mon!=t2.tm_mon)	d = 30*24*3600;	// number of second in month
+		else if(abs(t1.tm_yday-t2.tm_yday)>=14)	// number of second in week 
+		{	d = mgl_adj_val(abs(t1.tm_yday-t2.tm_yday)/7,&ds);
+			ds = ((ds<1)?1./7:ds)*7*24*3600;	d = ((d>=1)?d:1)*7*24*3600;	}
 		else if(abs(t1.tm_yday-t2.tm_yday)>1)	// localtime("%x") cannot print time < 1 day
-		{	d = 24*3600.*mgl_adj_val(abs(t1.tm_yday-t2.tm_yday));	d = d>24*3600?d:24*3600;	}
+		{	d = 24*3600.*mgl_adj_val(abs(t1.tm_yday-t2.tm_yday),&ds);
+			ds *= 24*3600;	if(d<24*3600)	{	d=24*3600;	ds=d/2;}	}
 		else if(abs(t1.tm_hour-t2.tm_hour)>1)
-			d = 3600.*mgl_adj_val(abs(t1.tm_hour-t2.tm_hour));
+		{	d = 3600.*mgl_adj_val(abs(t1.tm_hour-t2.tm_hour),&ds);	ds *=3600;	}
 		else if(abs(t1.tm_min-t2.tm_min)>1)
-			d = 60*mgl_adj_val(abs(t1.tm_min-t2.tm_min));
+		{	d = 60*mgl_adj_val(abs(t1.tm_min-t2.tm_min),&ds);	ds *=60;	}
 		else if(abs(t1.tm_sec-t2.tm_sec)>1)	// localtime("%X") cannot print time < 1 sec
-		{	d = mgl_adj_val(abs(t1.tm_sec-t2.tm_sec));	d = d>1?d:1;	}
+		{	d = mgl_adj_val(abs(t1.tm_sec-t2.tm_sec),&ds);
+			if(d<1)	{	d=1;	ds=0.5;}	}
 		else	// adjust msec. NOTE: this is not supported by localtime() !!!
-			d = mgl_adj_val(fabs(aa.v2-aa.v1));
+			d = mgl_adj_val(fabs(aa.v2-aa.v1),&ds);
 	}
 
-	aa.ds = 0;	aa.dv = d;	aa.f = 1;	aa.txt.clear();
+	aa.ds = ds;	aa.dv = d;	aa.f = 1;	aa.txt.clear();
 	MGL_TO_WCS(t,aa.t=wcs);
 
 	if(strchr("xyztuvw",aa.ch))
@@ -584,7 +584,7 @@ void mglCanvas::DrawLabels(mglAxis &aa, bool inv, const mglMatrix *M)
 	if(aa.ch=='c')	a = aa.a;
 
 	long n = aa.txt.size();
-	mreal *w=new mreal[n];
+	mreal *w=new mreal[n], wsp = 2*TextWidth(" ",FontDef,-1);
 	long *kk=new long[n];
 	for(long i=0;i<n;i++)	// fill base label properties
 	{
@@ -599,7 +599,7 @@ void mglCanvas::DrawLabels(mglAxis &aa, bool inv, const mglMatrix *M)
 			continue;
 		if(kk[i]<0 || kk[i+1]<0)	continue;
 		mreal v = (GetPntP(kk[i+1])-GetPntP(kk[i])).norm();	// distance between ticks
-		mreal vv = (w[i]+w[i+1])/2;	// length of labels
+		mreal vv = (w[i]+w[i+1])/2-wsp;	// length of labels
 		if(v>0 && l < vv/v)	l = vv/v;
 		if(c>v)	c = v;
 	}
@@ -614,10 +614,10 @@ void mglCanvas::DrawLabels(mglAxis &aa, bool inv, const mglMatrix *M)
 		mreal t2 = r2*1.21>1 ? asin((l*sqrt(r2-1/1.21)+h/1.1)/r2):M_PI/2;
 		tet = t1<t2 ? t1:t2;
 	}
-	mreal sn = sin(tet);
+	mreal sn = sin(tet), cs=cos(tet);
 	if(sn)
 	{
-		mreal l1=h/fabs(sn), l2=fabs(l*cos(tet)+h*sn);
+		mreal l1=h/fabs(sn), l2=fabs(l*cs+h*sn);
 		l = l2>l1?l1:l2;
 	}
 	char *align=new char[n], *up=new char[n];
@@ -626,13 +626,16 @@ void mglCanvas::DrawLabels(mglAxis &aa, bool inv, const mglMatrix *M)
 		mglPoint p(a),r(o+d*aa.txt[i].val);
 		ScalePoint(M, r, p, false);
 		mglPnt &pp = Pnt[kk[i]];
-		mreal ux=pp.u*cos(tet) + pp.v*sin(tet), uy=pp.v*cos(tet) - pp.u*sin(tet);
-		bool algn = tet!=0;
-		if(!get(MGL_ENABLE_RTEXT) || !get(MGL_TICKS_ROTATE))	{	ux=1;	uy=0;	algn=true;	}
+		mreal ux=pp.u*cs + pp.v*sn, uy=pp.v*cs - pp.u*sn;
+		bool Nrot = !get(MGL_ENABLE_RTEXT) || !get(MGL_TICKS_ROTATE);
+		bool Vcnt = ux==0 && uy!=0 && Nrot;
+		bool algn = tet!=0;		// TODO add proper align for arbitrary tet!
+		if(ux*ux+uy*uy!=0 && Nrot)	{	ux=1;	uy=0;	algn=true;	}
 		if(ux<0 || (ux==0 && uy<0))	{	ux=-ux;	uy=-uy;	pp.w=-pp.w;	}
 		pp.u = ux;	pp.v = uy;
 		mreal pu = p.x*ux+p.y*uy, pv = p.y*ux-p.x*uy; /*, su = ps.x*ux+ps.y*uy;*/
-		if(aa.ch!='c')	up[i] = ((pv>0) ^ inv) ? 'T':'t';
+		if(Vcnt)	up[i]='V';
+		else if(aa.ch!='c')	up[i] = ((pv>0) ^ inv) ? 'T':'t';
 		else		up[i]=(aa.ns==0 || aa.ns==3)?'t':'T';
 		int t=0;
 		if(algn)
@@ -848,8 +851,9 @@ void mglCanvas::Labelw(char dir, const wchar_t *text, mreal pos, const char *opt
 	if(aa)
 	{
 		char font[64],ff[3]=":C";	memset(font,0,64);
-		if(pos<-0.2)	ff[1]='L';	if(pos>0.2)	ff[1]='R';
-		strncpy(font,FontDef,63);	strcat(font,ff);
+		if(pos<-0.2)	ff[1]='L';
+		if(pos>0.2)	ff[1]='R';
+		strncpy(font,FontDef,32);	strcat(font,ff);
 		long kk = AddPnt(&B, p,-1,q,0,7);	ff[1]=0;
 		if(kk>=0)
 		{
@@ -857,7 +861,7 @@ void mglCanvas::Labelw(char dir, const wchar_t *text, mreal pos, const char *opt
 			if(pp.u<0 || (pp.u==0 && pp.v<0))
 			{	pp.u=-pp.u;	pp.v=-pp.v;	pp.w=-pp.w;	}
 			ff[0] = GetLabelPos(t, kk, *aa);	strcat(font,ff);
-			text_plot(kk,text,font,-1.4,0.35+shift);
+			text_plot(kk,text,font,-1.4,(ff[0]=='T'?0.3:0.35)+shift);
 		}
 	}
 	LoadState();
@@ -931,14 +935,14 @@ void mglCanvas::Box(const char *col, bool ticks)
 			mreal dx = (Max.x-Min.x)/30, dy = (Max.y-Min.y)/30, dz = (Max.z-Min.z)/30;
 			for(long i=0;i<31;i++)	for(long j=0;j<31;j++)
 			{
-				register long i0=3*(i+31*j);
+				long i0=3*(i+31*j);
 				pos[i0]   = AddPnt(mglPoint(oo[im].x,Min.y+dy*i,Min.z+dz*j));
 				pos[i0+1] = AddPnt(mglPoint(Min.x+dx*i,oo[im].y,Min.z+dz*j));
 				pos[i0+2] = AddPnt(mglPoint(Min.x+dx*i,Min.y+dy*j,oo[im].z));
 			}
 			for(long i=0;i<30;i++)	for(long j=0;j<30;j++)
 			{
-				register long i0=3*(i+31*j);
+				long i0=3*(i+31*j);
 				quad_plot(pos[i0],pos[i0+3],pos[i0+93],pos[i0+96]);
 				quad_plot(pos[i0+1],pos[i0+4],pos[i0+94],pos[i0+97]);
 				quad_plot(pos[i0+2],pos[i0+5],pos[i0+95],pos[i0+98]);
@@ -1072,7 +1076,7 @@ void mglCanvas::colorbar(HCDT vv, const mreal *c, int where, mreal x, mreal y, m
 		if(ac.t.empty())
 			for(long i=0;i<n;i++)
 			{
-				register mreal d = vv->v(i);
+				mreal d = vv->v(i);
 				ac.AddLabel(mgl_ftoa(d,ac.stl.c_str()),d);
 			}
 		else
@@ -1080,7 +1084,7 @@ void mglCanvas::colorbar(HCDT vv, const mreal *c, int where, mreal x, mreal y, m
 			wchar_t buf[64];
 			for(long i=0;i<n;i++)
 			{
-				register mreal d = vv->v(i);
+				mreal d = vv->v(i);
 				mglprintf(buf,64,ac.t.c_str(),d);
 				ac.AddLabel(buf,d);
 			}
