@@ -64,15 +64,15 @@ void MGL_EXPORT mgl_create_cpp_font(HMGL gr, const wchar_t *how)
 	for(i=l=n=0;i<s.size();i++)
 	{
 		ch = f->Internal(s[i]);
-		l += 2*f->GetNl(0,ch);
-		n += 6*f->GetNt(0,ch);
+		if(ch>=0)	{	l += 2*f->GetNl(0,ch);	n += 6*f->GetNt(0,ch);	}
 	}
-	printf("const long mgl_numg=%lu, mgl_cur=%lu;\n",(unsigned long)s.size(),l+n);
-	printf("float mgl_fact=%g;\n",f->GetFact(0)/mgl_fgen);
+	printf("const unsigned long mgl_numg=%lu, mgl_cur=%lu;\n",(unsigned long)s.size(),l+n);
+	printf("const float mgl_fact=%g;\n",f->GetFact(0)/mgl_fgen);
 	printf("long mgl_gen_fnt[%lu][6] = {\n", (unsigned long)s.size());
 	for(i=m=0;i<s.size();i++)	// first write symbols descriptions
 	{
 		ch = f->Internal(s[i]);
+		if(ch<0)	continue;
 		int m1 = f->GetNl(0,ch), m2 = f->GetNt(0,ch);
 		printf("\t{0x%x,%d,%d,%lu,%d,%lu},\n",unsigned(s[i]),f->GetWidth(0,ch),m1,m,m2,m+2*m1);
 		m += 2*m1+6*m2;
@@ -82,6 +82,7 @@ void MGL_EXPORT mgl_create_cpp_font(HMGL gr, const wchar_t *how)
 	for(i=0;i<s.size();i++)		// now write data itself
 	{
 		ch = f->Internal(s[i]);
+		if(ch<0)	continue;
 		unsigned m1 = f->GetNl(0,ch), m2 = f->GetNt(0,ch);
 		const short *ln = f->GetLn(0,ch), *tr = f->GetTr(0,ch);
 		for(l=0;l<2*m1;l++)	printf("%d,",ln[l]);
@@ -104,8 +105,8 @@ void MGL_EXPORT mgl_strtrim(char *str)
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_strlwr(char *str)
 {
-	register size_t k,l=strlen(str);
-	for(k=0;k<l;k++)
+	size_t l=strlen(str);
+	for(size_t k=0;k<l;k++)
 		str[k] = (str[k]>='A' && str[k]<='Z') ? str[k]+'a'-'A' : str[k];
 }
 //-----------------------------------------------------------------------------
@@ -150,7 +151,7 @@ mglBase::mglBase()
 
 	strcpy(last_style,"__1 {dFFFF}k\0");
 	MinS.Set(-1,-1,-1);	MaxS.Set(1,1,1);
-	fnt = new mglFont;	fnt->gr = this;	PrevState=NAN;	size_opt=NAN;
+	fnt = new mglFont;	fnt->gr = this;	PrevState=size_opt=NAN;
 }
 //-----------------------------------------------------------------------------
 mglBase::~mglBase()
@@ -212,27 +213,27 @@ void mglBase::StartGroup(const char *name, int id)
 	StartAutoGroup(buf);
 }
 //-----------------------------------------------------------------------------
-const char *mglWarn[mglWarnEnd] = {"data dimension(s) is incompatible",	//mglWarnDim
-								"data dimension(s) is too small",		//mglWarnLow
-								"minimal data value is negative",		//mglWarnNeg
-								"no file or wrong data dimensions",		//mglWarnFile
-								"not enough memory", 					//mglWarnMem
-								"data values are zero",					//mglWarnZero
-								"no legend entries",					//mglWarnLeg
-								"slice value is out of range",			//mglWarnSlc
-								"number of contours is zero or negative",//mglWarnCnt
-								"couldn't open file",					//mglWarnOpen
-								"light: ID is out of range",			//mglWarnLId
-								"size(s) is zero or negative",			//mglWarnSize
-								"format is not supported for that build",//mglWarnFmt
-								"axis ranges are incompatible",			//mglWarnTern
-								"pointer is NULL",						//mglWarnNull
-								"not enough space for plot",			//mglWarnSpc
-								"There is wrong argument(s) in script",	//mglScrArg
-								"There is wrong command(s) in script",	//mglScrCmd
-								"There is too long string(s) in script",	//mglScrLong
-								"There is unbalanced ' in script",		//mglScrStr
-								"There is changing temporary data in script"};	//mglScrTemp
+const char *mglWarn[mglWarnEnd] = {_("data dimension(s) is incompatible"),	//mglWarnDim
+								_("data dimension(s) is too small"),		//mglWarnLow
+								_("minimal data value is negative"),		//mglWarnNeg
+								_("no file or wrong data dimensions"),		//mglWarnFile
+								_("not enough memory"), 					//mglWarnMem
+								_("data values are zero"),					//mglWarnZero
+								_("no legend entries"),					//mglWarnLeg
+								_("slice value is out of range"),			//mglWarnSlc
+								_("number of contours is zero or negative"),//mglWarnCnt
+								_("couldn't open file"),					//mglWarnOpen
+								_("light: ID is out of range"),			//mglWarnLId
+								_("size(s) is zero or negative"),			//mglWarnSize
+								_("format is not supported for that build"),//mglWarnFmt
+								_("axis ranges are incompatible"),			//mglWarnTern
+								_("pointer is NULL"),						//mglWarnNull
+								_("not enough space for plot"),			//mglWarnSpc
+								_("There is wrong argument(s) in script"),	//mglScrArg
+								_("There is wrong command(s) in script"),	//mglScrCmd
+								_("There is too long string(s) in script"),	//mglScrLong
+								_("There is unbalanced ' in script"),		//mglScrStr
+								_("There is changing temporary data in script")};	//mglScrTemp
 //-----------------------------------------------------------------------------
 extern bool mglPrintWarn;
 void mglBase::SetWarn(int code, const char *who)
@@ -246,7 +247,7 @@ void mglBase::SetWarn(int code, const char *who)
 	}
 	else if(!code)	Mess="";
 	else if(who && *who)	warn = who;
-	if(mglPrintWarn && !warn.empty())	fprintf(stderr,"MathGL message - %s\n",warn.c_str());
+	if(mglPrintWarn && !warn.empty())	fprintf(stderr,_("MathGL message - %s\n"),warn.c_str());
 	if(code && !warn.empty())	Mess = Mess+(code==-2?"":"\n")+warn;
 	LoadState();
 }
@@ -255,7 +256,7 @@ void mglBase::SetWarn(int code, const char *who)
 //-----------------------------------------------------------------------------
 void mglGlyph::Create(long Nt, long Nl)
 {
-	if(Nt<0 || Nl<0)	return;
+//	if(Nt<0 || Nl<0)	return;
 	nt=Nt;	nl=Nl;
 #pragma omp critical(glf_create)
 	{
@@ -266,7 +267,7 @@ void mglGlyph::Create(long Nt, long Nl)
 	}
 }
 //-----------------------------------------------------------------------------
-bool mglGlyph::operator==(const mglGlyph &g)
+bool mglGlyph::operator==(const mglGlyph &g) const
 {
 	if(nl!=g.nl || nt!=g.nt)	return false;
 	if(trig && memcmp(trig,g.trig,6*nt*sizeof(short)))	return false;
@@ -290,6 +291,45 @@ long mglBase::AddGlyph(int s, long j)
 	{k=Glf.size();	MGL_PUSH(Glf,g,mutexGlf);}	return k;
 }
 //-----------------------------------------------------------------------------
+long mglBase::AddGlyph(unsigned char id)
+{
+	size_t j=0;
+	for(size_t i=0;i<UserGlf.size();i++)
+		if(UserGlf[i].nt==-id)	j=i+1;
+	if(j==0)	return -1;
+	const mglGlyph &g=UserGlf[j-1];
+	// let find the similar glyph
+	for(size_t i=0;i<Glf.size();i++)
+		if(g!=Glf[i])	continue;	else	return i;
+	// if no one then let add it
+	long k;
+#pragma omp critical(glf)
+	{k=Glf.size();	MGL_PUSH(Glf,g,mutexGlf);}	return k;
+}
+//-----------------------------------------------------------------------------
+void mglBase::DefineGlyph(HCDT x, HCDT y, unsigned char id)
+{
+	long n = x->GetNx();
+	if(y->GetNx()!=n || n<2)	return;
+	mglGlyph g(-id,n);
+	mreal x1=1e10,x2=-1e10,y1=1e10,y2=-1e10;
+	for(long i=0;i<n;i++)
+	{
+		mreal xx = x->v(i), yy = y->v(i);
+		x1=x1>xx?xx:x1;	x2=x2<xx?xx:x2;
+		y1=y1>yy?yy:y1;	y2=y2<yy?yy:y2;
+	}
+	mreal scale = 1;
+	if(fabs(x1)<10 && fabs(x2)<10 && fabs(y1)<10 && fabs(y2)<10)
+		scale=1000;
+	for(long i=0;i<n;i++)
+	{
+		short sx = short(x->v(i)*scale), sy = short(y->v(i)*scale);
+		g.line[2*i] = sx;	g.line[2*i+1] = sy;
+	}
+	UserGlf.push_back(g);
+}
+//-----------------------------------------------------------------------------
 //		Add points to the buffer
 //-----------------------------------------------------------------------------
 void inline mgl_put_inbox(mreal a1, mreal a2, mreal &a)
@@ -336,7 +376,7 @@ long mglBase::AddPnt(const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mrea
 		q.x=q.xx=p.x;	q.y=q.yy=p.y;	q.z=q.zz=p.z;
 		q.c=c;	q.t=q.ta=a;	q.u=n.x;	q.v=n.y;	q.w=n.z;
 	}
-	register long ci=long(c);
+	long ci=long(c);
 	if(ci<0 || ci>=(long)Txt.size())	ci=0;	// NOTE never should be here!!!
 	const mglTexture &txt=Txt[ci];
 	txt.GetC(c,a,q);	// RGBA color
@@ -402,12 +442,10 @@ bool mglBase::RecalcCRange()
 	else
 	{
 		FMin.c = INFINITY;	FMax.c = -INFINITY;
-		register int i;
-		mreal a;
 		int n=30;
-		for(i=0;i<=n;i++)
+		for(int i=0;i<=n;i++)
 		{
-			a = fa->Calc(0,0,0,Min.c+i*(Max.c-Min.c)/n);
+			mreal a = fa->Calc(0,0,0,Min.c+i*(Max.c-Min.c)/n);
 			if(mgl_isbad(a))	wrong=true;
 			if(a<FMin.c)	FMin.c=a;
 			if(a>FMax.c)	FMax.c=a;
@@ -426,30 +464,28 @@ void mglBase::RecalcBorder()
 	{
 		FMin.Set( INFINITY, INFINITY, INFINITY);
 		FMax.Set(-INFINITY,-INFINITY,-INFINITY);
-		register int i,j;
 		int n=30;
-		for(i=0;i<=n;i++)	for(j=0;j<=n;j++)	// x range
+		for(int i=0;i<=n;i++)	for(int j=0;j<=n;j++)	// x range
 		{
 			if(SetFBord(Min.x, Min.y+i*(Max.y-Min.y)/n, Min.z+j*(Max.z-Min.z)/n))	wrong=true;
 			if(SetFBord(Max.x, Min.y+i*(Max.y-Min.y)/n, Min.z+j*(Max.z-Min.z)/n))	wrong=true;
 		}
-		for(i=0;i<=n;i++)	for(j=0;j<=n;j++)	// y range
+		for(int i=0;i<=n;i++)	for(int j=0;j<=n;j++)	// y range
 		{
 			if(SetFBord(Min.x+i*(Max.x-Min.x)/n, Min.y, Min.z+j*(Max.z-Min.z)/n))	wrong=true;
 			if(SetFBord(Min.x+i*(Max.x-Min.x)/n, Max.y, Min.z+j*(Max.z-Min.z)/n))	wrong=true;
 		}
-		for(i=0;i<=n;i++)	for(j=0;j<=n;j++)	// x range
+		for(int i=0;i<=n;i++)	for(int j=0;j<=n;j++)	// x range
 		{
 			if(SetFBord(Min.x+i*(Max.x-Min.x)/n, Min.y+j*(Max.y-Min.y)/n, Min.z))	wrong=true;
 			if(SetFBord(Min.x+i*(Max.x-Min.x)/n, Min.y+j*(Max.y-Min.y)/n, Max.z))	wrong=true;
 		}
-		mreal d;
 		if(!fx)	{	FMin.x = Min.x;	FMax.x = Max.x;	}
-		else	{	d=0.01*(FMax.x-FMin.x);	FMin.x-=d;	FMax.x+=d;	}
+		else	{	mreal d=0.01*(FMax.x-FMin.x);	FMin.x-=d;	FMax.x+=d;	}
 		if(!fy)	{	FMin.y = Min.y;	FMax.y = Max.y;	}
-		else	{	d=0.01*(FMax.y-FMin.y);	FMin.y-=d;	FMax.y+=d;	}
+		else	{	mreal d=0.01*(FMax.y-FMin.y);	FMin.y-=d;	FMax.y+=d;	}
 		if(!fz)	{	FMin.z = Min.z;	FMax.z = Max.z;	}
-		else	{	d=0.01*(FMax.z-FMin.z);	FMin.z-=d;	FMax.z+=d;	}
+		else	{	mreal d=0.01*(FMax.z-FMin.z);	FMin.z-=d;	FMax.z+=d;	}
 	}
 	if(RecalcCRange())	wrong=true;
 	if(wrong)	SetWarn(mglWarnTern, "Curved coordinates");
@@ -537,17 +573,17 @@ bool mglBase::ScalePoint(const mglMatrix *, mglPoint &p, mglPoint &n, bool use_n
 	}
 
 	x1=x;	y1=y;	z1=z;
-	register mreal xx=1,xy=0,xz=0,yx=0,yy=1,yz=0,zx=0,zy=0,zz=1;
+	mreal xx=1,xy=0,xz=0,yx=0,yy=1,yz=0,zx=0,zy=0,zz=1;
 	if(fx)	{	x1 = fx->Calc(x,y,z);	xx = fx->CalcD('x',x,y,z);	xy = fx->CalcD('y',x,y,z);	xz = fx->CalcD('z',x,y,z);	}
 	if(fy)	{	y1 = fy->Calc(x,y,z);	yx = fy->CalcD('x',x,y,z);	yy = fy->CalcD('y',x,y,z);	yz = fy->CalcD('z',x,y,z);	}
 	if(fz)	{	z1 = fz->Calc(x,y,z);	zx = fz->CalcD('x',x,y,z);	zy = fz->CalcD('y',x,y,z);	zz = fz->CalcD('z',x,y,z);	}
 	if(mgl_isnan(x1) || mgl_isnan(y1) || mgl_isnan(z1))	{	x=NAN;	return false;	}
 
-	register mreal d;
+	mreal d;
 	d = 1/(FMax.x - FMin.x);	x = (2*x1 - FMin.x - FMax.x)*d;	xx /= d;	xy /= d;	xz /= d;
 	d = 1/(FMax.y - FMin.y);	y = (2*y1 - FMin.y - FMax.y)*d;	yx /= d;	yy /= d;	yz /= d;
 	d = 1/(FMax.z - FMin.z);	z = (2*z1 - FMin.z - FMax.z)*d;	zx /= d;	zy /= d;	zz /= d;
-	register mreal nx=n.x, ny=n.y, nz=n.z;
+	mreal nx=n.x, ny=n.y, nz=n.z;
 	n.x = nx*xx+ny*xy+nz*xz;
 	n.y = nx*yx+ny*yy+nz*yz;
 	n.z = nx*zx+ny*zy+nz*zz;
@@ -802,8 +838,10 @@ void mglBase::Ternary(int t)
 //-----------------------------------------------------------------------------
 void mglBase::SetFunc(const char *EqX,const char *EqY,const char *EqZ,const char *EqA)
 {
-	if(fa)	delete fa;	if(fx)	delete fx;
-	if(fy)	delete fy;	if(fz)	delete fz;
+	if(fa)	delete fa;
+	if(fx)	delete fx;
+	if(fy)	delete fy;
+	if(fz)	delete fz;
 	if(EqX && *EqX && (EqX[0]!='x' || EqX[1]!=0))
 		fx = new mglFormula(EqX);
 	else	fx = 0;
@@ -867,8 +905,11 @@ void mglBase::ClearEq()
 {
 #pragma omp critical(eq)
 	{
-		if(fx)	delete fx;	if(fy)	delete fy;	if(fz)	delete fz;
-		if(fa)	delete fa;	if(fc)	delete fc;
+		if(fx)	delete fx;
+		if(fy)	delete fy;
+		if(fz)	delete fz;
+		if(fa)	delete fa;
+		if(fc)	delete fc;
 		fx = fy = fz = fc = fa = 0;
 	}
 	RecalcBorder();
@@ -907,22 +948,31 @@ void MGL_EXPORT mgl_chrrgb(char p, float c[3])
 		}
 }
 //-----------------------------------------------------------------------------
+size_t MGL_EXPORT mgl_get_num_color(const char *s, int smooth)
+{
+	if(!s || !s[0])	return 0;
+	size_t l=strlen(s), n=0;	long j=0;
+	for(size_t i=0;i<l;i++)		// find number of colors
+	{
+		if(smooth>=0 && s[i]==':' && j<1)	break;
+		if(s[i]=='{' && strchr(MGL_COLORS"x",s[i+1]) && j<1)	n++;
+		if(s[i]=='[' || s[i]=='{')	j++;
+		if(s[i]==']' || s[i]=='}')	j--;
+		if(strchr(MGL_COLORS,s[i]) && j<1)	n++;
+//		if(smooth && s[i]==':')	break;	// NOTE: should use []
+	}
+	return n;
+}
+//-----------------------------------------------------------------------------
 void mglTexture::Set(const char *s, int smooth, mreal alpha)
 {
 	// NOTE: New syntax -- colors are CCCCC or {CNCNCCCN}; options inside []
 	if(!s || !s[0])	return;
 	strncpy(Sch,s,259);	Smooth=smooth;	Alpha=alpha;
 
-	register long i,j=0,m=0,l=strlen(s);
+	long l=strlen(s);
 	bool map = smooth==2 || mglchr(s,'%'), sm = smooth>=0 && !strchr(s,'|');	// Use mapping, smoothed colors
-	for(i=n=0;i<l;i++)		// find number of colors
-	{
-		if(smooth>=0 && s[i]==':' && j<1)	break;
-		if(s[i]=='{' && strchr(MGL_COLORS"x",s[i+1]) && j<1)	n++;
-		if(s[i]=='[' || s[i]=='{')	j++;	if(s[i]==']' || s[i]=='}')	j--;
-		if(strchr(MGL_COLORS,s[i]) && j<1)	n++;
-//		if(smooth && s[i]==':')	break;	// NOTE: should use []
-	}
+	n = mgl_get_num_color(s,smooth);
 	if(!n)
 	{
 		if(strchr(s,'|') && !smooth)	// sharp colors
@@ -934,11 +984,13 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 	bool man=sm;
 	mglColor *c = new mglColor[2*n];		// Colors itself
 	mreal *val = new mreal[n];
-	for(i=j=n=0;i<l;i++)	// fill colors
+	for(long i=0, m=0, j=n=0;i<l;i++)	// fill colors
 	{
 		if(smooth>=0 && s[i]==':' && j<1)	break;
-		if(s[i]=='[')	j++;	if(s[i]==']')	j--;
-		if(s[i]=='{')	m++;	if(s[i]=='}')	m--;
+		if(s[i]=='[')	j++;
+		if(s[i]==']')	j--;
+		if(s[i]=='{')	m++;
+		if(s[i]=='}')	m--;
 		if(strchr(MGL_COLORS,s[i]) && j<1 && (m==0 || s[i-1]=='{'))	// {CN,val} format, where val in [0,1]
 		{
 			if(m>0 && s[i+1]>'0' && s[i+1]<='9')// ext color
@@ -988,10 +1040,10 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 	val[0]=0;	val[n-1]=1;	// boundary have to be [0,1]
 	for(long i=0;i<n;i++) if(val[i]>0 && val[i]<1) 	def.push_back(i);
 	def.push_back(n-1);
-	long i1=0,i2;
-	for(size_t j=0;j<def.size();j++)	for(i=i1+1;i<def[j];i++)
+	long i1=0;
+	for(size_t j=0;j<def.size();j++)	for(long i=i1+1;i<def[j];i++)
 	{
-		i1 = j>0?def[j-1]:0;	i2 = def[j];
+		i1 = j>0?def[j-1]:0;	long i2 = def[j];
 		v1 = val[i1];	v2 = val[i2];
 		v2 = i2-i1>1?(v2-v1)/(i2-i1):0;
 		val[i]=v1+v2*(i-i1);
@@ -1000,12 +1052,12 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 	mreal v=sm?(n-1)/255.:n/256.;
 	if(!sm)	for(long i=0;i<256;i++)
 	{
-		register long j = 2*long(v*i);	//u-=j;
+		long j = 2*long(v*i);	//u-=j;
 		col[2*i] = c[j];	col[2*i+1] = c[j+1];
 	}
-	else	for(i=i1=0;i<256;i++)
+	else	for(long i=i1=0;i<256;i++)
 	{
-		register mreal u = v*i;	j = long(u);	//u-=j;
+		mreal u = v*i;	long j = long(u);	//u-=j;
 		if(j<n-1)	// advanced scheme using val
 		{
 			for(;i1<n-1 && i>=255*val[i1];i1++);
@@ -1023,7 +1075,7 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 mglColor mglTexture::GetC(mreal u,mreal v) const
 {
 	u -= long(u);
-	register long i=long(255*u);	u = u*255-i;
+	long i=long(255*u);	u = u*255-i;
 	const mglColor *s=col+2*i;
 	return (s[0]*(1-u)+s[2]*u)*(1-v) + (s[1]*(1-u)+s[3]*u)*v;
 }
@@ -1031,7 +1083,7 @@ mglColor mglTexture::GetC(mreal u,mreal v) const
 void mglTexture::GetC(mreal u,mreal v,mglPnt &p) const
 {
 	u -= long(u);
-	register long i=long(255*u);	u = u*255-i;
+	long i=long(255*u);	u = u*255-i;
 	const mglColor &s0=col[2*i], &s1=col[2*i+1], &s2=col[2*i+2], &s3=col[2*i+3];
 	p.r = (s0.r*(1-u)+s2.r*u)*(1-v) + (s1.r*(1-u)+s3.r*u)*v;
 	p.g = (s0.g*(1-u)+s2.g*u)*(1-v) + (s1.g*(1-u)+s3.g*u)*v;
@@ -1110,7 +1162,7 @@ MGL_EXPORT const char *mglchr(const char *str, char ch)
 	size_t l=strlen(str),k=0;
 	for(size_t i=0;i<l;i++)
 	{
-		register char c = str[i];
+		char c = str[i];
 		if(c=='{')	k++;
 		if(c=='}')	k--;
 		if(c==ch && k==0)	return str+i;
@@ -1139,7 +1191,8 @@ char mglBase::SetPenPal(const char *p, long *Id, bool pal)
 		size_t l=strlen(p);
 		for(size_t i=0;i<l;i++)
 		{
-			if(p[i]=='{')	m++;	if(p[i]=='}')	m--;
+			if(p[i]=='{')	m++;
+			if(p[i]=='}')	m--;
 			if(m>0 && p[i]=='d')	PDef = strtol(p+i+1,0,16);
 			if(m>0)	continue;
 			s = mglchr(stl,p[i]);
@@ -1155,7 +1208,8 @@ char mglBase::SetPenPal(const char *p, long *Id, bool pal)
 				else	Arrow1 = p[i];
 			}
 		}
-		if(!Arrow1)	Arrow1='_';		if(!Arrow2)	Arrow2='_';
+		if(!Arrow1)	Arrow1='_';
+		if(!Arrow2)	Arrow2='_';
 		if(mglchr(p,'#'))
 		{
 			s = mglchr(mrk,mk);
@@ -1164,11 +1218,13 @@ char mglBase::SetPenPal(const char *p, long *Id, bool pal)
 		}
 		if((s=strstr(p,"{&"))!=0)
 		{	mk = last_style[3] = p[3];	strcpy(last_style+11,s);	}
+		else if(mk && mglchr(p,'&'))
+		{	mk += 128;	last_style[3] = mk;	}
 		last_style[0] = Arrow1;	last_style[1] = Arrow2;
 	}
 	if(pal)
 	{
-		if((s=strstr(p,"{&"))!=0)
+		if(p && (s=strstr(p,"{&"))!=0)
 		{
 			CDef = atof(s+2);
 //			if(Id)	*Id=long(tt)*256+(n+CurrPal-1)%n;
@@ -1176,22 +1232,23 @@ char mglBase::SetPenPal(const char *p, long *Id, bool pal)
 		else
 		{
 			long tt, n;
-			tt = AddTexture(p,-1);	n=Txt[tt].n;
+			tt = AddTexture(p?p:MGL_DEF_PAL,-1);	n=Txt[tt].n;
 			CDef = tt+((n+CurrPal-1)%n+0.5)/n;
 			if(Id)	*Id=long(tt)*256+(n+CurrPal-1)%n;
 			sprintf(last_style+11,"{&%g}",CDef);
 		}
 	}
-	if(Arrow1=='_')	Arrow1=0;	if(Arrow2=='_')	Arrow2=0;
+	if(Arrow1=='_')	Arrow1=0;
+	if(Arrow2=='_')	Arrow2=0;
 	return mk;
 }
 //-----------------------------------------------------------------------------
 // keep this for restore default mask
-uint64_t mgl_mask_def[16]={0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000007700000000,
+MGL_EXPORT uint64_t mgl_mask_def[16]={0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000007700000000,
 							0x0000182424180000,	0x0000183C3C180000,	0x00003C24243C0000,	0x00003C3C3C3C0000,
 							0x0000060990600000,	0x0060584658600000,	0x00061A621A060000,	0x0000005F00000000,
 							0x0008142214080000,	0x00081C3E1C080000,	0x8142241818244281,	0x0000001824420000};
-uint64_t mgl_mask_val[16]={0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000007700000000,
+MGL_EXPORT uint64_t mgl_mask_val[16]={0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000007700000000,
 							0x0000182424180000,	0x0000183C3C180000,	0x00003C24243C0000,	0x00003C3C3C3C0000,
 							0x0000060990600000,	0x0060584658600000,	0x00061A621A060000,	0x0000005F00000000,
 							0x0008142214080000,	0x00081C3E1C080000,	0x8142241818244281,	0x0000001824420000};
@@ -1206,7 +1263,8 @@ void mglBase::SetMask(const char *p)
 		long m=0, l=strlen(p);
 		for(long i=0;i<l;i++)
 		{
-			if(p[i]=='{')	m++;	if(p[i]=='}')	m--;
+			if(p[i]=='{')	m++;
+			if(p[i]=='}')	m--;
 			if(m>0 && p[i]=='s')	mask = strtoull(p+i+1,0,16);
 			if(m>0)	continue;
 			if(p[i]==':')	break;
@@ -1374,7 +1432,7 @@ void mglBase::AddLegend(const char *str,const char *style)
 bool MGL_EXPORT mgl_check_dim2(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, const char *name, bool less)
 {
 //	if(!gr || !x || !y || !z)	return true;		// if data is absent then should be segfault!!!
-	register long n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,name);	return true;	}
 	if(a && z->GetNN()!=a->GetNN())
 	{	gr->SetWarn(mglWarnDim,name);	return true;	}
@@ -1398,7 +1456,7 @@ bool MGL_EXPORT mgl_check_dim2(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, const ch
 bool MGL_EXPORT mgl_check_dim0(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT r, const char *name, bool less)
 {
 //	if(!gr || !x || !y)	return true;		// if data is absent then should be segfault!!!
-	register long n=y->GetNx();
+	long n=y->GetNx();
 	if(less)
 	{
 		if(x->GetNx()<n)		{	gr->SetWarn(mglWarnDim,name);	return true;	}
@@ -1417,7 +1475,7 @@ bool MGL_EXPORT mgl_check_dim0(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT r, const ch
 bool MGL_EXPORT mgl_check_dim1(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT r, const char *name, bool less)
 {
 //	if(!gr || !x || !y)	return true;		// if data is absent then should be segfault!!!
-	register long n=y->GetNx();
+	long n=y->GetNx();
 	if(n<2)	{	gr->SetWarn(mglWarnLow,name);	return true;	}
 	if(less)
 	{
@@ -1437,7 +1495,7 @@ bool MGL_EXPORT mgl_check_dim1(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT r, const ch
 bool MGL_EXPORT mgl_check_dim3(HMGL gr, bool both, HCDT x, HCDT y, HCDT z, HCDT a, HCDT b, const char *name)
 {
 // 	if(!gr || !x || !y || !z || !a)	return true;		// if data is absent then should be segfault!!!
-	register long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
 	if(n<2 || m<2 || l<2)
 	{	gr->SetWarn(mglWarnLow,name);	return true;	}
 	if(!both && (x->GetNx()!=n || y->GetNx()!=m || z->GetNx()!=l))
@@ -1459,20 +1517,20 @@ bool MGL_EXPORT mgl_check_trig(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCDT
 //-----------------------------------------------------------------------------
 bool MGL_EXPORT mgl_isnboth(HCDT x, HCDT y, HCDT z, HCDT a)
 {
-	register long n=a->GetNN();
+	long n=a->GetNN();
 	return x->GetNN()!=n || y->GetNN()!=n || z->GetNN()!=n;
 }
 //-----------------------------------------------------------------------------
 bool MGL_EXPORT mgl_isboth(HCDT x, HCDT y, HCDT z, HCDT a)
 {
-	register long n=a->GetNN();
+	long n=a->GetNN();
 	return x->GetNN()==n && y->GetNN()==n && z->GetNN()==n;
 }
 //-----------------------------------------------------------------------------
 bool MGL_EXPORT mgl_check_vec3(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay, HCDT az, const char *name)
 {
 // 	if(!gr || !x || !y || !z || !ax || !ay || !az)	return true;		// if data is absent then should be segfault!!!
-	register long n=ax->GetNx(),m=ax->GetNy(),l=ax->GetNz(), nn=n*m*l;
+	long n=ax->GetNx(),m=ax->GetNy(),l=ax->GetNz(), nn=n*m*l;
 	if(nn!=ay->GetNN() || nn!=az->GetNN())
 	{	gr->SetWarn(mglWarnDim,name);	return true;	}
 	if(n<2 || m<2 || l<2)	{	gr->SetWarn(mglWarnLow,name);	return true;	}
@@ -1501,11 +1559,13 @@ void mglBase::ClearUnused()
 			used[p.n1] = 1;
 			switch(p.type)
 			{
-			case 1:	case 4:	if(p.n2>=0)	used[p.n2] = 1;	break;
-			case 2:	if(p.n2>=0 && p.n3>=0)
-				used[p.n2] = used[p.n3] = 1;	break;
+			case 1:	case 4:	if(p.n2>=0)	used[p.n2] = 1;
+				break;
+			case 2:	if(p.n2>=0 && p.n3>=0)	used[p.n2] = used[p.n3] = 1;
+				break;
 			case 3:	if(p.n2>=0 && p.n3>=0 && p.n4>=0)
-				used[p.n2] = used[p.n3] = used[p.n4] = 1;	break;
+					used[p.n2] = used[p.n3] = used[p.n4] = 1;
+				break;
 			}
 		}
 		// now add proper indexes
