@@ -19,6 +19,7 @@
  ***************************************************************************/
 #include <time.h>
 #include <ctype.h>
+#include <wchar.h>
 #include "mgl2/base.h"
 #include "mgl2/parser.h"
 #if MGL_HAVE_GSL
@@ -259,7 +260,7 @@ HADT mglApplyOperDivC(std::wstring a1, std::wstring a2, mglParser *arg, const st
 //-----------------------------------------------------------------------------
 bool mglCheck(std::wstring str)
 {
-	register long s = 0,i,n=str.length();
+	long s = 0,i,n=str.length();
 	for(i=0;i<n;i++)
 	{
 		if(str[i]=='(')	s++;
@@ -269,10 +270,10 @@ bool mglCheck(std::wstring str)
 	return (s==0) ? true : false;
 }
 //-----------------------------------------------------------------------------
-int mglFindInText(std::wstring str,const char *lst)
+long mglFindInText(const std::wstring &str,const char *lst)
 {
-	register long l=0,r=0,i;//,j,len=strlen(lst);
-	for(i=str.length()-1;i>=0;i--)
+	long l=0,r=0;
+	for(long i=str.length()-1;i>=0;i--)
 	{
 		if(str[i]=='(') l++;
 		if(str[i]==')') r++;
@@ -305,7 +306,7 @@ double MGL_LOCAL_CONST mgl_fmax(double a,double b);
 // It seems that standard wcstombs() have a bug. So, I replace by my own.
 void MGL_EXPORT mgl_wcstombs(char *dst, const wchar_t *src, int size)
 {
-	register int j;
+	int j;
 	for(j=0;j<size-1 && src[j]!=0;j++)
 		dst[j] = src[j]<0x7f ? src[j] : ' ';
 	dst[j] = 0;
@@ -320,11 +321,33 @@ MGL_LOCAL_PURE const mglDataA *FindVar(const std::vector<mglDataA*> &head, const
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_wcslwr(wchar_t *str)
 {
-	register size_t l=mgl_wcslen(str);
+	size_t l=mgl_wcslen(str);
 	for(size_t k=0;k<l;k++)
 		str[k] = (str[k]>='A' && str[k]<='Z') ? str[k]+'a'-'A' : str[k];
 }
 //-----------------------------------------------------------------------------
+mreal mgl_gettime(const std::wstring &s)
+{
+	mreal t=NAN;
+	tm a;	memset(&a,0,sizeof(tm));
+	if(swscanf(s.c_str(),L"%u-%u-%u_%u.%u.%d", &a.tm_hour,&a.tm_min,&a.tm_sec, &a.tm_mday,&a.tm_mon,&a.tm_year)==6)
+	{	a.tm_year-=1900;	a.tm_mon -= 1;
+		if(a.tm_hour<24 && a.tm_min<60 && a.tm_sec<60 && a.tm_mday>0 && a.tm_mday<32 && a.tm_mon<12)
+			t = mktime(&a);
+	}
+	else if(swscanf(s.c_str(),L"%d.%d.%d", &a.tm_mday,&a.tm_mon,&a.tm_year)==3)
+	{	a.tm_year-=1900;	a.tm_mon -= 1;
+		if(a.tm_mday>0 && a.tm_mday<32 && a.tm_mon<12)
+			t = mktime(&a);
+	}
+	else if(swscanf(s.c_str(),L"%d-%d-%d", &a.tm_hour,&a.tm_min,&a.tm_sec)==3)
+	{	a.tm_mday=1;	a.tm_mon=0;	a.tm_year=70;
+		if(a.tm_hour<24 && a.tm_min<60 && a.tm_sec<60)
+			t = mktime(&a);
+	}
+	return t;
+}
+//-----------------------------------------------------------------------------
 /// Parse string and substitute the script argument
 // All numbers are presented as mglData(1). Do boundary checking.
 // NOTE: In any case where number is required the mglData::a[0] is used.
@@ -336,6 +359,10 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 #endif
 	if(str.empty())	return new mglData;	// nothing to parse
 	str = mgl_trim_ws(str);
+	mreal tval = mgl_gettime(str);
+	if(mgl_isnum(tval))
+	{	mglData *r=new mglData;	r->a[0] = tval;	return r;	}
+
 	long n,len=str.length();
 	if(str[0]=='(' && mglCheck(str.substr(1,len-2)))	// remove braces
 	{	str = str.substr(1,len-2);	len-=2;	}
@@ -456,9 +483,15 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 		{
 			if(ch=='a' && p[2]=='x')	v = d->Maximal();
 			else if(ch=='i' && p[2]=='n')	v = d->Minimal();
-			else if(ch=='x')	{	v = d->Maximal(x,y,z);	v = x/ns[0];	}
-			else if(ch=='y')	{	v = d->Maximal(x,y,z);	v = y/ns[1];	}
-			else if(ch=='z')	{	v = d->Maximal(x,y,z);	v = z/ns[2];	}
+			else if(ch=='x' && p[2]=='f')	v = d->Maximal('x',0)/mreal(ns[0]);
+			else if(ch=='x' && p[2]=='l')	v = d->Maximal('x',-1)/mreal(ns[0]);
+			else if(ch=='x')	{	d->Maximal(x,y,z);	v = x/ns[0];	}
+			else if(ch=='y' && p[2]=='f')	v = d->Maximal('y',0)/mreal(ns[1]);
+			else if(ch=='y' && p[2]=='l')	v = d->Maximal('y',-1)/mreal(ns[1]);
+			else if(ch=='y')	{	d->Maximal(x,y,z);	v = y/ns[1];	}
+			else if(ch=='z' && p[2]=='f')	v = d->Maximal('z',0)/mreal(ns[2]);
+			else if(ch=='z' && p[2]=='l')	v = d->Maximal('z',-1)/mreal(ns[2]);
+			else if(ch=='z')	{	d->Maximal(x,y,z);	v = z/ns[2];	}
 		}
 		else if(c0=='s')
 		{
@@ -679,7 +712,8 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 		else if(!nm.compare(L"int"))	return mglApplyFunc(str, arg, head, floor);
 		else if(!nm.compare(L"random"))
 		{	HMDT res=mglFormulaCalc(str, arg, head);	mreal *a = res->a;
-			for(long i=0;i<res->GetNN();i++)	a[i] = mgl_rnd();	return res;	}
+			for(long i=0;i<res->GetNN();i++)	a[i] = mgl_rnd();
+			return res;	}
 		else if(!nm.compare(L"real"))
 		{
 			HADT a1 = mglFormulaCalcC(str, arg, head);
@@ -730,9 +764,11 @@ dual MGL_LOCAL_CONST cgtc(dual a,dual b);	//{return real(a-b)>0?1:0;}
 dual MGL_LOCAL_CONST ipwc(dual a,dual b);	//{return mgl_ipowc(a,int(b.real()));}
 dual MGL_LOCAL_CONST powc(dual a,dual b);	//{return exp(b*log(a));	}
 dual MGL_LOCAL_CONST llgc(dual a,dual b);	//{return log(a)/log(b);	}
+dual MGL_LOCAL_CONST cmplxc(dual a,dual b);	//{return a+dual(0,1)*b;	}
 dual MGL_LOCAL_CONST expi(dual a);	//{	return exp(dual(0,1)*a);	}
 dual MGL_LOCAL_CONST expi(double a);	//{	return dual(cos(a),sin(a));	}
 //-----------------------------------------------------------------------------
+dual MGL_LOCAL_CONST hypotc(dual x, dual y);	//{	return sqrt(x*x+y*y);	}
 dual MGL_LOCAL_CONST asinhc(dual x);	//{	return log(x+sqrt(x*x+mreal(1)));	}
 dual MGL_LOCAL_CONST acoshc(dual x);	//{	return log(x+sqrt(x*x-mreal(1)));	}
 dual MGL_LOCAL_CONST atanhc(dual x);	//{	return log((mreal(1)+x)/(mreal(1)-x))/mreal(2);	}
@@ -767,6 +803,10 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 #endif
 	if(str.empty())	return new mglDataC;	// nothing to parse
 	str = mgl_trim_ws(str);
+	mreal tval = mgl_gettime(str);
+	if(mgl_isnum(tval))
+	{	mglDataC *r=new mglDataC;	r->a[0] = tval;	return r;	}
+	
 	long n,len=str.length();
 	if(str[0]=='(' && mglCheck(str.substr(1,len-2)))	// remove braces
 	{	str = str.substr(1,len-2);	len-=2;	}
@@ -882,9 +922,9 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 			mreal x,y,z;
 			if(ch=='a' && p[2]=='x')	v = d->Maximal();
 			else if(ch=='i' && p[2]=='n')	v = d->Minimal();
-			else if(ch=='x')	{	v = d->Maximal(x,y,z);	v = x/ns[0];	}
-			else if(ch=='y')	{	v = d->Maximal(x,y,z);	v = y/ns[1];	}
-			else if(ch=='z')	{	v = d->Maximal(x,y,z);	v = z/ns[2];	}
+			else if(ch=='x')	{	d->Maximal(x,y,z);	v = x/ns[0];	}
+			else if(ch=='y')	{	d->Maximal(x,y,z);	v = y/ns[1];	}
+			else if(ch=='z')	{	d->Maximal(x,y,z);	v = z/ns[2];	}
 		}
 		else if(c0=='s')
 		{
@@ -990,6 +1030,8 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 			if(!nm.compare(L"cos"))	return mglApplyFuncC(str, arg, head, cosc);
 			else if(!nm.compare(L"cosh") || !nm.compare(L"ch"))	return mglApplyFuncC(str, arg, head, coshc);
 			else if(!nm.compare(L"conj"))	return mglApplyFuncC(str, arg, head, conjc);
+			else if(!nm.compare(L"cmplx") && n>0)
+				return mglApplyOperC(str.substr(0,n),str.substr(n+1),arg, head, cmplxc);
 		}
 		else if(!nm.compare(L"exp"))	return mglApplyFuncC(str, arg, head, expc);
 		else if(nm[0]=='l')
@@ -1012,7 +1054,10 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 			return mglApplyOperC(str.substr(0,n),str.substr(n+1),arg, head, powc);
 		else if(!nm.compare(L"random"))
 		{	HADT res=mglFormulaCalcC(str, arg, head);	dual *a = res->a;
-			for(long i=0;i<res->GetNN();i++)	a[i] = dual(mgl_rnd(), mgl_rnd());	return res;	}
+			for(long i=0;i<res->GetNN();i++)	a[i] = dual(mgl_rnd(), mgl_rnd());
+			return res;	}
+		else if(!nm.compare(L"hypot"))
+			return mglApplyOperC(str.substr(0,n),str.substr(n+1),arg, head, hypotc);
 		else if(!nm.compare(L"real"))	return mglApplyFuncC(str, arg, head, realc);
 		else if(!nm.compare(L"imag"))	return mglApplyFuncC(str, arg, head, imagc);
 		else if(!nm.compare(L"norm"))	return mglApplyFuncC(str, arg, head, normc);
