@@ -36,7 +36,7 @@ void mglTexture::GetRGBA(unsigned char *f) const
 		mglColor c1 = col[2*i], c2 = col[2*i+1];
 		for(long j=0;j<256;j++)
 		{
-			register long i0 = 4*(j+256*i);
+			long i0 = 4*(j+256*i);
 			mglColor c = c1 + (c2-c1)*(j/255.);
 			f[i0]   = int(255*c.r);
 			f[i0+1] = int(255*c.g);
@@ -287,13 +287,13 @@ void MGL_EXPORT mgl_write_obj_old(HMGL gr, const char *fname,const char *descr,
 	if(gr->GetPrmNum()==0)	return;	// nothing to do
 	long m1=0,m2=0;
 	for(size_t i=0;i<gr->Grp.size();i++)	// prepare array of indirect indexing
-	{	register long m = gr->Grp[i].Id;	if(m<m1) m1=m;	if(m>m2) m2=m;	}
+	{	long m = gr->Grp[i].Id;	if(m<m1) m1=m;	if(m>m2) m2=m;	}
 	long *ng = new long[m2-m1+1];
 	for(size_t i=0;i<gr->Grp.size();i++)	ng[gr->Grp[i].Id-m1] = i;
 	for(long i=0;i<gr->GetPrmNum();i++)	// collect data for groups
 	// it is rather expensive (extra 4b per primitive) but need for export to 3D
 	{
-		register long m = gr->GetPrm(i,false).id-m1;
+		long m = gr->GetPrm(i,false).id-m1;
 		if(m>=0 && m<m2-m1+1)	gr->Grp[ng[m]].p.push_back(i);
 	}
 	delete []ng;
@@ -302,7 +302,7 @@ void MGL_EXPORT mgl_write_obj_old(HMGL gr, const char *fname,const char *descr,
 	FILE *fp=fopen(fname,"wt");
 	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
 	// vertices definition
-	const std::string loc = setlocale(LC_NUMERIC, NULL);	setlocale(LC_NUMERIC, "C");
+	const std::string loc = setlocale(LC_NUMERIC, "C");
 	fprintf(fp,"# Created by MathGL library\n# Title: %s\n",(descr && *descr) ? descr : fname);
 	for(long i=0;i<gr->GetPntNum();i++)
 	{
@@ -363,7 +363,7 @@ void MGL_EXPORT mgl_write_stl(HMGL gr, const char *fname,const char *descr)
 	if(gr->GetPrmNum()==0)	return;	// nothing to do
 	FILE *fp = fopen(fname,"wt");
 	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
-	const std::string loc = setlocale(LC_NUMERIC, NULL);	setlocale(LC_NUMERIC, "C");
+	const std::string loc = setlocale(LC_NUMERIC, "C");
 	fprintf(fp,"solid %s\n",(descr && *descr)?descr:"mathgl");
 	mglPnt pp;
 	for(long i=0;i<gr->GetPrmNum();i++)
@@ -416,7 +416,7 @@ void MGL_EXPORT mgl_write_xyz(HMGL gr, const char *fname,const char *descr)
 
 	FILE *fp=fopen(fname,"wt"), *ff;	// vertices definition
 	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
-	const std::string loc = setlocale(LC_NUMERIC, NULL);	setlocale(LC_NUMERIC, "C");
+	const std::string loc = setlocale(LC_NUMERIC, "C");
 	fprintf(fp,"# Created by MathGL library\n# Title: %s\n",(descr && *descr) ? descr : fname);
 	fprintf(fp,"# List of Vertices, with (x,y,z) coordinates.\n");
 	for(long i=0;i<gr->GetPntNum();i++)
@@ -462,7 +462,7 @@ void MGL_EXPORT mgl_write_off(HMGL gr, const char *fname,const char *descr, int
 
 	FILE *fp=fopen(fname,"wt");
 	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
-	const std::string loc = setlocale(LC_NUMERIC, NULL);	setlocale(LC_NUMERIC, "C");
+	const std::string loc = setlocale(LC_NUMERIC, "C");
 	// vertices definition
 	if(colored)
 		fprintf(fp,"COFF\n# Created by MathGL library\n# Title: %s\n",(descr && *descr) ? descr : fname);
@@ -542,7 +542,7 @@ std::string mglCanvas::GetJSON()
 	std::string res, buf;
 	long ll=0,l=(long)Pnt.size();
 	long factor = Width>1?10:10000;
-	const std::string loc = setlocale(LC_NUMERIC, NULL);	setlocale(LC_NUMERIC, "C");
+	const std::string loc = setlocale(LC_NUMERIC, "C");
 	res = res + mgl_sprintf("{\n\"width\":%d,\t\"height\":%d,\t\"depth\":%d,\t\"plotid\":\"%s\",\t\"npnts\":%ld,\t\"pnts\":[\n",
 			factor*Width, factor*Height, factor*Depth, PlotId.c_str(), l);
 	for(long i=0;i<l;i++)
@@ -575,7 +575,7 @@ std::string mglCanvas::GetJSON()
 			n3 = p.n3;	n4 = p.n4;
 		}
 		if(p.type==1 && n1>n2)	{	n1=p.n2;	n2=p.n1;	}
-		register long ps=p.s==p.s?long(100*factor*p.s):0, pw=p.w==p.w?long(100*p.w):0, pp=p.p==p.p?mgl_int(1e5*p.p):0;
+		long ps=p.s==p.s?long(100*factor*p.s):0, pw=p.w==p.w?long(100*p.w):0, pp=p.p==p.p?mgl_int(1e5*p.p):0;
 		if(cp.r[3]==255 || p.type==0 || p.type==1 || p.type==4 || p.type==6)
 			buf = mgl_sprintf("[%d,%ld,%ld,%ld,%ld,%d,%ld,%ld,%ld,0,\"#%02x%02x%02x\"],\n",
 				p.type, n1, n2, n3, n4, p.id, ps,pw,pp, int(cp.r[0]),int(cp.r[1]),int(cp.r[2]));
@@ -594,7 +594,7 @@ std::string mglCanvas::GetJSON()
 	{
 		const mglPoint &p=xy[i];
 		const mglPnt &q=Pnt[int(0.5+p.z)];
-		register long px=long(100*p.x), py=long(100*p.y);
+		long px=long(100*p.x), py=long(100*p.y);
 		if(q.u==q.u && q.v==q.v && q.w==q.w)
 			res = res + mgl_sprintf("[%ld,%ld,%ld,%ld,%ld]%c\n", px, py, long(100*q.u), long(100*q.v), long(100*q.w), i+1<l?',':' ');
 		else
@@ -634,7 +634,7 @@ bool mglCanvas::ExportMGLD(const char *fname, const char *descr)
 	if(Pnt.size()<1 || Prm.size()<1)	return true;
 	FILE *fp=fopen(fname,"wt");
 	if(!fp)	return true;
-	const std::string loc = setlocale(LC_NUMERIC, NULL);	setlocale(LC_NUMERIC, "C");
+	const std::string loc = setlocale(LC_NUMERIC, "C");
 	// NOTE: I'll save Ptx. So prim type=6 is useless,and no LaTeX
 	fprintf(fp,"MGLD %lu %lu %lu %lu %d %d\n# %s\n", (unsigned long)Pnt.size(), (unsigned long)Prm.size(), (unsigned long)Txt.size(), (unsigned long)Glf.size(), Width, Height, (descr && *descr) ? descr : fname);
 	fprintf(fp,"# Vertexes: x y z c t ta u v w r g b a\n");
@@ -697,7 +697,7 @@ bool mglCanvas::ImportMGLD(const char *fname, bool add)
 	if(w<=0 || h<=0)	{	w=Width;	h=Height;	}
 	d = long(sqrt(double(w*h)));
 	if(n==0 || m==0 || l==0)	{	delete []buf;	fclose(fp);	return true;	}
-	const std::string loc = setlocale(LC_NUMERIC, NULL);	setlocale(LC_NUMERIC, "C");
+	const std::string loc = setlocale(LC_NUMERIC, "C");
 	if(!add)	{	Clf();	Txt.clear();	}
 	else	{	ClfZB();	npnt=Pnt.size();	nglf=Glf.size();	}
 	LightScale(&B);
@@ -781,7 +781,7 @@ void MGL_EXPORT mgl_import_mgld_(uintptr_t *gr, const char *fname, int *add, int
 /*void MGL_EXPORT mgl_xgl_prim(const mglPrim &q, const mglPnt &p, FILE *fp, mreal size)
 {
 	char type = q.n4;	mreal ss=size*0.35;
-	register long i=q.n1,j;
+	long i=q.n1;
 	switch(q.type)
 	{
 	case 0:
@@ -950,8 +950,7 @@ void mglCanvas::WriteXGL(const char *fname,const char *descr)
 	fprintf(fp,"<WORLD>\n<NAME>%s</NAME>\n", (descr && *descr)?descr:fname);
 	fprintf(fp,"<BACKGROUND><BACKCOLOR>%g, %g, %g</BACKCOLOR></BACKGROUND>\n", BDef[0]/255., BDef[1]/255., BDef[2]/255.);
 	fprintf(fp,"<LIGHTING>\n<AMBIENT>%g, %g, %g</AMBIENT>\n",AmbBr, AmbBr, AmbBr);
-	register size_t i,j;
-	if(get(MGL_ENABLE_LIGHT))	for(i=0;i<10;i++)
+	if(get(MGL_ENABLE_LIGHT))	for(size_t i=0;i<10;i++)
 		if(light[i].n && mgl_isnan(light[i].r.x))
 		{
 			fprintf(fp, "<DIRECTIONALLIGHT>\n<DIRECTION>%g, %g, %g</DIRECTION>\n", light[i].d.x, light[i].d.y, light[i].d.z);
@@ -962,11 +961,11 @@ void mglCanvas::WriteXGL(const char *fname,const char *descr)
 	// TODO: add textures
 
 	long m1=0,m2=0,m;
-	for(i=0;i<Grp.size();i++)	// prepare array of indirect indexing
+	for(size_t i=0;i<Grp.size();i++)	// prepare array of indirect indexing
 	{	m = Grp[i].Id;	if(m<m1) m1=m;	if(m>m2) m2=m;	}
 	long *ng = new long[m2-m1+1];
-	for(i=0;i<Grp.size();i++)	ng[gr->Grp[i].Id-m1] = i;
-	for(i=0;i<GetPrmNum();i++)	// collect data for groups
+	for(size_t i=0;i<Grp.size();i++)	ng[gr->Grp[i].Id-m1] = i;
+	for(size_t i=0;i<GetPrmNum();i++)	// collect data for groups
 	// it is rather expensive (extra 4b per primitive) but need for export to 3D
 	{
 		m = GetPrm(i,false).id-m1;
@@ -977,25 +976,25 @@ void mglCanvas::WriteXGL(const char *fname,const char *descr)
 	std::vector<long> p;
 	mglPrim q;
 	char *pg=new char[GetPntNum()];
-	for(i=0;i<Grp.size();i++)	// first write objects
+	for(size_t i=0;i<Grp.size();i++)	// first write objects
 	{
 		p = Grp[i].p;	memset(pg,0,GetPntNum());
 		fprintf(fp,"<OBJECT>\n<NAME>%s</NAME>\n<MESH>\n",Grp[i].Lbl.c_str());
-		for(j=0;j<p.size();j++)		// collect Pnt for this object
+		for(size_t j=0;j<p.size();j++)		// collect Pnt for this object
 		{
 			const mglPrim q=GetPrm(p[j],false);	pg[q.n1]=1;
 			if(q.type==3)	{	pg[q.n2]=1;	pg[q.n3]=1;	pg[q.n4]=1;	}
 			else if(q.type==1)	pg[q.n2]=1;
 			else if(q.type==2)	{	pg[q.n2]=1;	pg[q.n3]=1;	}
 		}
-		for(j=0;j<GetPntNum();j++)	if(pg[j])	// write Pnt for this object
+		for(size_t j=0;j<GetPntNum();j++)	if(pg[j])	// write Pnt for this object
 		{
 			const mglPnt s=Pnt[j];
 			fprintf(fp,"<P ID=\"%u\">%g, %g, %g</P>\n",j, s.x, s.y, s.z);
 			fprintf(fp,"<N ID=\"%u\">%g, %g, %g</N>\n",j, s.x, s.y, s.z);
 		}
 		// TODO: add line styles
-		for(j=0;j<p.size();j++)	// now write primitives itself
+		for(size_t j=0;j<p.size();j++)	// now write primitives itself
 		{
 			const mglPrim q=GetPrm(p[j],false);
 			mgl_xgl_prim(q, GetPnt(q.n1), fp, q.s*FontFactor());
@@ -1024,17 +1023,26 @@ void MGL_EXPORT mgl_x3d_mdef(HMGL gr, void *fp, bool gz)
 	for(long i=0;i<gr->GetPrmNum();i++)
 	{
 		const mglPrim q = gr->GetPrm(i,false);
-		if(q.type>0)	continue;		if(q.n4=='+')	m_p = true;
-		if(q.n4=='x')	m_x = true;		if(q.n4=='s')	m_s = true;
-		if(q.n4=='d')	m_d = true;		if(q.n4=='v')	m_v = true;
-		if(q.n4=='^')	m_t = true;		if(q.n4=='*')	m_a = true;
+		if(q.type>0)	continue;
+		if(q.n4=='+')	m_p = true;
+		if(q.n4=='x')	m_x = true;
+		if(q.n4=='s')	m_s = true;
+		if(q.n4=='d')	m_d = true;
+		if(q.n4=='v')	m_v = true;
+		if(q.n4=='^')	m_t = true;
+		if(q.n4=='*')	m_a = true;
 		if(q.n4=='o' || q.n4=='O' || q.n4=='C')	m_o = true;
-		if(q.n4=='S')	m_S = true;		if(q.n4=='D')	m_D = true;
-		if(q.n4=='V')	m_V = true;		if(q.n4=='T')	m_T = true;
-		if(q.n4=='<')	m_l = true;		if(q.n4=='L')	m_L = true;
-		if(q.n4=='>')	m_r = true;		if(q.n4=='R')	m_R = true;
+		if(q.n4=='S')	m_S = true;
+		if(q.n4=='D')	m_D = true;
+		if(q.n4=='V')	m_V = true;
+		if(q.n4=='T')	m_T = true;
+		if(q.n4=='<')	m_l = true;
+		if(q.n4=='L')	m_L = true;
+		if(q.n4=='>')	m_r = true;
+		if(q.n4=='R')	m_R = true;
 		if(q.n4=='Y')	m_Y = true;
-		if(q.n4=='P')	m_P = true;		if(q.n4=='X')	m_X = true;
+		if(q.n4=='P')	m_P = true;
+		if(q.n4=='X')	m_X = true;
 	}
 	if(m_P)	{	m_p=true;	m_s=true;	}
 	if(m_X)	{	m_x=true;	m_s=true;	}
@@ -1158,7 +1166,7 @@ void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
 	bool gz = fname[strlen(fname)-1]=='z';
 	void *fp = gz ? (void*)gzopen(fname,"wt") : (void*)fopen(fname,"wt");
 	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
-	const std::string loc = setlocale(LC_NUMERIC, NULL);	setlocale(LC_NUMERIC, "C");
+	const std::string loc = setlocale(LC_NUMERIC, "C");
 	mgl_printf(fp, gz, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
 	mgl_printf(fp, gz, "<!DOCTYPE X3D PUBLIC \"ISO//Web3D//DTD X3D 3.0//EN\" \"http://www.web3d.org/specifications/x3d-3.0.dtd\">\n");
 	mgl_printf(fp, gz, "<X3D profile='Immersive'>\n<head>\n<meta name='filename' content='%s'/>\n",fname);
