@@ -37,96 +37,44 @@ void MGL_NO_EXPORT mgl_operator_exp(long n, dual *h, dual *a, dual *f)
 	for(long j=0;j<n;j++)
 	{
 		long jp = (j+1)%n;
-		mreal h1=real(h[n*j]), h2=real(h[n-1+n*j]);
-		mreal g1=(h1+real(h[n*jp]))/2, g2=(h2+real(h[n-1+n*jp]))/2;
+		dual h1=h[n*j]*dual(0,1),		g1=(h1+h[n*jp]*dual(0,1))/mreal(2);
+		dual h2=h[n-1+n*j]*dual(0,1),	g2=(h2+h[n-1+n*jp]*dual(0,1))/mreal(2);
 		mreal k1=M_PI*2*j/n, k2 = M_PI*(2*j+1)/n;
 		for(long i=0;i<i1;i++)
 		{
-			f[2*j] += a[i]*exp(dual(0,h1+i*k1));
-			f[2*j+1] += a[i]*exp(dual(0,g1+i*k2));
+			f[2*j] += a[i]*exp(h1+dual(0,i*k1));
+			f[2*j+1] += a[i]*exp(g1+dual(0,i*k2));
 		}
 		for(long i=i1;i<i2;i++)
 		{
-			mreal hh = real(h[i-i1+n*j]);
-			f[2*j] += a[i]*exp(dual(0,hh+i*k1));
-			f[2*j+1] += a[i]*exp(dual(0,(hh+real(h[i-i1+n*jp]))/2+i*k2));
+			dual hh = h[i-i1+n*j];
+			f[2*j] += a[i]*exp(hh*dual(0,1)+dual(0,i*k1));
+			f[2*j+1] += a[i]*exp((hh+h[i-i1+n*jp])*dual(0,0.5)+dual(0,i*k2));
 		}
 		for(long i=i2;i<2*n;i++)
 		{
-			f[2*j] += a[i]*exp(dual(0,h2+i*k1));
-			f[2*j+1] += a[i]*exp(dual(0,g2+i*k2));
+			f[2*j] += a[i]*exp(h2+dual(0,i*k1));
+			f[2*j+1] += a[i]*exp(g2+dual(0,i*k2));
 		}
 	}
 	memset(a,0,2*n*sizeof(dual));
 #pragma omp parallel for
 	for(long i=0;i<2*n;i++)
 	{
-		register long ii=i-i1;
-		if(ii<0)	ii=0;	if(ii>n-1)	ii=n-1;
+		long ii=i-i1;
+		if(ii<0)	ii=0;
+		if(ii>n-1)	ii=n-1;
 		double kk=M_PI*2*i/n;
 		for(long j=0;j<n;j++)
 		{
-			mreal h1 = real(h[ii+n*j]), g1 = (h1+real(h[ii+n*((j+1)%n)]))/2;
-			a[i] += f[2*j]*exp(dual(0,h1-kk*j));
-			a[i] += f[2*j+1]*exp(dual(0,g1-kk*(j+0.5)));
+			dual h1 = h[ii+n*j], g1 = (h1+h[ii+n*((j+1)%n)])*dual(0,0.5);
+			a[i] += f[2*j]*exp(h1*dual(0,1)-dual(0,kk*j));
+			a[i] += f[2*j+1]*exp(g1-dual(0,kk*(j+0.5)));
 		}
 	}
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_operator_hrm(long n, dual *h, dual *a, dual *f, dual *g, dual *o)
-{
-	memset(f,0,2*n*sizeof(dual));
-	memset(g,0,2*n*sizeof(dual));
-	const long i1=n/2, i2=3*n/2-1;
-	#pragma omp parallel for
-	for(long j=0;j<n;j++)
-	{
-		long jp = (j+1)%n;
-		mreal h1=real(h[n*j]), h2=real(h[n-1+n*j]);
-		mreal g1=h1+real(h[n*jp])/2., g2=h2+real(h[n-1+n*jp])/2.;
-		mreal k1=M_PI*2*j/n, k2 = M_PI*(2*j+1)/n;
-		for(long i=0;i<i1;i++)
-		{
-			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
-			f[2*j] += a[i]*h1*e1;	f[2*j+1] += a[i]*g1*e2;
-			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
-		}
-		for(long i=i1;i<i2;i++)
-		{
-			mreal hh = real(h[i-i1+n*j]);
-			mreal gg = hh+real(h[i-i1+n*jp])/2.;
-			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
-			f[2*j] += a[i]*hh*e1;	f[2*j+1] += a[i]*gg*e2;
-			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
-		}
-		for(long i=i2;i<2*n;i++)
-		{
-			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
-			f[2*j] += a[i]*h2*e1;	f[2*j+1] += a[i]*g2*e2;
-			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
-		}
-	}
-	memset(o,0,2*n*sizeof(dual));
-	#pragma omp parallel for
-	for(long i=0;i<2*n;i++)
-	{
-		register long ii=i-i1;
-		if(ii<0)	ii=0;	if(ii>n-1)	ii=n-1;
-		double kk=M_PI*2*i/n;
-		for(long j=0;j<n;j++)
-		{
-			mreal h1 = real(h[ii+n*j]);
-			mreal g1 = h1+real(h[ii+n*((j+1)%n)])/2.;
-			dual e1=exp(dual(0,-j*kk)), e2=exp(dual(0,-kk*(j+0.5)));
-			o[i] += f[2*j]*e1 + f[2*j+1]*e2;
-			o[i] += g[2*j]*h1*e1 + g[2*j+1]*g1*e2;
-		}
-	}
-}
-//-----------------------------------------------------------------------------
-inline dual isqrt(dual a)
-{	mreal b=imag(a);	return b>=0?dual(sqrt(b),0):dual(0,sqrt(-b));	}
-void MGL_NO_EXPORT mgl_operator_lin(long n, dual *h, dual *a, dual *f, dual *g, dual *o)
+void MGL_NO_EXPORT mgl_operator_lin(long n, mreal *h, dual *a, dual *f, dual *g, dual *o)
 {
 	memset(f,0,2*n*sizeof(dual));
 	memset(g,0,2*n*sizeof(dual));
@@ -135,8 +83,8 @@ void MGL_NO_EXPORT mgl_operator_lin(long n, dual *h, dual *a, dual *f, dual *g,
 	for(long j=0;j<n;j++)
 	{
 		long jp = (j+1)%n;
-		dual h1=tanh(isqrt(h[n*j])), h2=tanh(isqrt(h[n-1+n*j]));
-		dual g1=tanh(h1+isqrt(h[n*jp]))/2., g2=tanh(h2+isqrt(h[n-1+n*jp]))/2.;
+		mreal h1=tanh(h[n*j]), g1=(h1+tanh(h[n*jp]))/2;
+		mreal h2=tanh(h[n-1+n*j]), g2=(h2+tanh(h[n-1+n*jp]))/2;
 		mreal k1=M_PI*2*j/n, k2 = M_PI*(2*j+1)/n;
 		for(long i=0;i<i1;i++)
 		{
@@ -146,8 +94,8 @@ void MGL_NO_EXPORT mgl_operator_lin(long n, dual *h, dual *a, dual *f, dual *g,
 		}
 		for(long i=i1;i<i2;i++)
 		{
-			dual hh = tanh(isqrt(h[i-i1+n*j]));
-			dual gg = tanh(hh+isqrt(h[i-i1+n*jp]))/2.;
+			mreal hh = tanh(h[i-i1+n*j]);
+			mreal gg = (hh+tanh(h[i-i1+n*jp]))/2;
 			dual e1=exp(dual(0,i*k1)), e2=exp(dual(0,i*k2));
 			f[2*j] += a[i]*hh*e1;	f[2*j+1] += a[i]*gg*e2;
 			g[2*j] += a[i]*e1;		g[2*j+1] += a[i]*e2;
@@ -163,13 +111,14 @@ void MGL_NO_EXPORT mgl_operator_lin(long n, dual *h, dual *a, dual *f, dual *g,
 #pragma omp parallel for
 	for(long i=0;i<2*n;i++)
 	{
-		register long ii=i-i1;
-		if(ii<0)	ii=0;	if(ii>n-1)	ii=n-1;
+		long ii=i-i1;
+		if(ii<0)	ii=0;
+		if(ii>n-1)	ii=n-1;
 		double kk=M_PI*2*i/n;
 		for(long j=0;j<n;j++)
 		{
-			dual h1 = tanh(isqrt(h[ii+n*j]));
-			dual g1 = tanh(h1+isqrt(h[ii+n*((j+1)%n)]))/2.;
+			mreal h1 = tanh(h[ii+n*j]);
+			mreal g1 = (h1+tanh(h[ii+n*((j+1)%n)]))/2;
 			dual e1=exp(dual(0,-j*kk)), e2=exp(dual(0,-kk*(j+0.5)));
 			o[i] += f[2*j]*e1 + f[2*j+1]*e2;
 			o[i] += g[2*j]*h1*e1 + g[2*j+1]*g1*e2;
@@ -177,6 +126,47 @@ void MGL_NO_EXPORT mgl_operator_lin(long n, dual *h, dual *a, dual *f, dual *g,
 	}
 }
 //-----------------------------------------------------------------------------
+HADT MGL_NO_EXPORT mgl_apde_calc_ham(HMDT hs, bool old, const char *func, std::vector<mglDataA*> list, const mreal dd)
+{
+	HADT ham = mglFormulaCalcC(func, list);	mgl_datac_mul_num(ham,dd);
+	const long nx = ham->nx;
+	if(old)
+	{
+		mreal hh = ham->Imag().Minimal();
+		if(hh>0)	hh=0;
+#pragma omp parallel for
+		for(long i=0;i<nx*nx;i++)
+		{
+			hs->a[i] = sqrt(imag(ham->a[i])-hh);	// non-additive term
+			ham->a[i] = dual(real(ham->a[i]),hh);	// additive terms
+		}
+	}
+	else
+	{
+		mglData xIm(nx), pIm(nx);
+#pragma omp parallel for
+		for(long i=0;i<nx;i++)	// first find minimal values along x and p
+		{
+			dual *ax=ham->a+i, *ay=ham->a+i*nx;
+			mreal mx=imag(ax[0]), my=imag(ay[0]);
+			for(long j=1;j<nx;j++)	my = (my<imag(ay[j]))?my:imag(ay[j]);
+			for(long j=1;j<nx;j++)	mx = (mx<imag(ax[j*nx]))?mx:imag(ax[j*nx]);
+			xIm.a[i] = mx;	pIm.a[i]=my;
+		}
+		mreal mIm=xIm.a[0];	mreal *aa=xIm.a;	// global minimum
+		for(long j=1;j<nx;j++)	mIm = (mIm<aa[j])?mIm:aa[j];
+#pragma omp parallel for collapse(2)
+		for(long j=0;j<nx;j++)	for(long i=0;i<nx;i++)
+		{
+			mreal hh = xIm.a[i]+pIm.a[j]-mIm;
+			long i0=i+nx*j;
+			hs->a[i0] = sqrt(fabs(imag(ham->a[i0])-hh));	// non-additive term. NOTE: fabs() guarantee absence of negative values due to rounding error
+			ham->a[i0] = dual(real(ham->a[i0]),hh);	// additive terms
+		}
+	}
+	return ham;
+}
+//-----------------------------------------------------------------------------
 // Solve equation dx/dy = func(p,x,y,|u|)[u] where p=d/dx. There are no assumptions about form of func().
 HADT MGL_EXPORT mgl_pde_adv_c(HMGL gr, const char *func, HCDT ini_re, HCDT ini_im, mreal dt, mreal k0, const char *opt)
 {
@@ -187,9 +177,11 @@ HADT MGL_EXPORT mgl_pde_adv_c(HMGL gr, const char *func, HCDT ini_re, HCDT ini_i
 	if(ini_im->GetNx() != nx)		{	gr->SetWarn(mglWarnDim,"PDE");	return 0;	}	// Wrong dimensions
 
 	mglDataC *res=new mglDataC(nx, nt);
+	mglData hIm(nx,nx);	// for advanced damping calculation
 	mglDataC u(nx);	u.s = L"u";
 	mglDataV x(nx,nx), y(nx,nx), r(nx,nx);
 	mglDataW p(nx,nx);	p.s = L"p";
+	bool old = func[0]==';';	if(old)	func=func+1;
 	x.s = L"x";	y.s = L"y";	r.s=L"#$mgl";
 	const mreal dp = 2*M_PI/(Max.x-Min.x), dd = k0*dt/2;
 	x.Fill(Min.x,Max.x,'x');	p.Freq(dp/k0,'y');
@@ -210,20 +202,19 @@ HADT MGL_EXPORT mgl_pde_adv_c(HMGL gr, const char *func, HCDT ini_re, HCDT ini_i
 	}
 	bool have_y = mglchr(func,'y');
 	HADT ham;
-	if(!have_y)
-	{	ham = mglFormulaCalcC(func, list);	mgl_datac_mul_num(ham,dd);	}
+	if(!have_y)		ham = mgl_apde_calc_ham(&hIm, old, func, list, dd);
 	for(long k=0;k<nt;k++)
 	{
 		memcpy(u.a,a+nx/2,nx*sizeof(dual));
 		memcpy(res->a+k*nx,a+nx/2,nx*sizeof(dual));
 		if(have_y)
-		{	y.Fill(k*dt);	ham = mglFormulaCalcC(func, list);	mgl_datac_mul_num(ham,dd);	}
+		{	y.Fill(k*dt);	ham = mgl_apde_calc_ham(&hIm, old, func, list, dd);	}
 		mgl_operator_exp(nx,ham->a,a,f);
-		mgl_operator_lin(nx,ham->a,a,f,g,s);
-		mgl_operator_lin(nx,ham->a,s,f,g,s);
+		mgl_operator_lin(nx,hIm.a,a,f,g,s);
+		mgl_operator_lin(nx,hIm.a,s,f,g,s);
 #pragma omp parallel for
 		for(long i=0;i<2*nx;i++)
-			a[i] = (a[i]-s[i]/mreal(8*nx*nx))*exp(-dmp[i]*dt)/mreal(2*nx);
+			a[i] = (a[i]-s[i]/mreal(8*nx*nx))*mreal(exp(-dmp[i]*dt)/2/nx);
 		if(have_y)	delete ham;
 	}
 	delete []a;	delete []f;	delete []dmp;
@@ -277,7 +268,7 @@ uintptr_t MGL_EXPORT mgl_pde_adv_(uintptr_t* gr, const char *ham, uintptr_t* ini
 #pragma omp parallel for collapse(2)
 	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)	// Initial conditions
 	{
-		register long i0 = i+nx/2+2*nx*(j+ny/2);
+		long i0 = i+nx/2+2*nx*(j+ny/2);
 		a[i0] = dual(ini_re->v(i,j), ini_im->v(i,j));
 		res->a[nz*(i+nx*j)] = a[i0];
 	}
@@ -286,7 +277,7 @@ uintptr_t MGL_EXPORT mgl_pde_adv_(uintptr_t* gr, const char *ham, uintptr_t* ini
 #pragma omp parallel for collapse(2)
 	for(long j=0;j<2*ny;j++)	for(long i=0;i<2*nx;i++)	// step 1
 	{
-		register long i0 = i+2*nx*j;
+		long i0 = i+2*nx*j;
 		if(i<nx/2)		dmp[i0] += gamma*mgl_ipow((nx/2-i)/(nx/2.),2);
 		if(i>3*nx/2)	dmp[i0] += gamma*mgl_ipow((i-3*nx/2-1)/(nx/2.),2);
 		if(j<ny/2)		dmp[i0] += gamma*mgl_ipow((ny/2-j)/(ny/2.),2);
@@ -355,7 +346,7 @@ void MGL_NO_EXPORT mgl_pde_hprep(const mgl_pde_ham *f)
 	}
 }
 //-----------------------------------------------------------------------------
-// Solve equation dx/dz = func(p,q,x,y,z,|u|)[u] where p=d/dx, q=d/dy. At this moment simplified form of ham is supported: ham = f(p,q,z) + g(x,y,z,'u'), where variable 'u'=|u| (for allowing solve nonlinear problems). You may specify imaginary part like ham = p^2 + i*x*(x>0) but only if dependence on variable 'i' is linear (i.e. ham = hre+i*him).
+// Solve equation dx/dz = func(p,q,x,y,z,|u|)[u] where p=d/dx, q=d/dy. At this moment simplified form of ham is supported: ham = f(p,q,z) + g(x,y,z,'u'), where variable 'u'=|u| (for allowing solve nonlinear problems). You may specify imaginary part like ham = p^2 + 1i*x*(x>0).
 HADT MGL_EXPORT mgl_pde_solve_c(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_im, mreal dz, mreal k0, const char *opt)
 {
 	mreal gamma = gr->SaveState(opt);	if(mgl_isnan(gamma))	gamma = GAMMA;
@@ -378,14 +369,14 @@ HADT MGL_EXPORT mgl_pde_solve_c(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_
 #pragma omp parallel for collapse(2)
 	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)	// Initial conditions
 	{
-		register long i0 = i+nx/2+2*nx*(j+ny/2);
+		long i0 = i+nx/2+2*nx*(j+ny/2);
 		a[i0] = dual(ini_re->v(i,j), ini_im->v(i,j));
 		res->a[nz*(i+nx*j)] = a[i0];
 	}
 #pragma omp parallel for collapse(2)
 	for(long j=0;j<2*ny;j++)	for(long i=0;i<2*nx;i++)	// step 1
 	{
-		register long i0 = i+2*nx*j;
+		long i0 = i+2*nx*j;
 		if(i<nx/2)		dmp[i0] += gamma*mgl_ipow((nx/2-i)/(nx/2.),2);
 		if(i>3*nx/2)	dmp[i0] += gamma*mgl_ipow((i-3*nx/2-1)/(nx/2.),2);
 		if(j<ny/2)		dmp[i0] += gamma*mgl_ipow((ny/2-j)/(ny/2.),2);
@@ -420,7 +411,7 @@ HADT MGL_EXPORT mgl_pde_solve_c(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_
 #pragma omp parallel for collapse(2)
 			for(long i=0;i<2*nx;i++) for(long j=0;j<2*ny;j++)
 			{
-				register long i0 = i+2*nx*j;	huv[i0] -= hh0;
+				long i0 = i+2*nx*j;	huv[i0] -= hh0;
 				hxv[i0] -= hx[i]+hv[j]-hh0;
 				huy[i0] -= hu[i]+hy[j]-hh0;
 			}
@@ -502,7 +493,7 @@ struct mglOdeTxt	{	long n;	HMEX *eq;	const char *var;	};
 void MGL_NO_EXPORT mgl_txt_func(const mreal *x, mreal *dx, void *par)
 {
 	mglOdeTxt *p=(mglOdeTxt *)par;
-	mreal vars['z'-'a'+1];
+	mreal vars[MGL_VS];
 	for(long i=0;i<p->n;i++)
 	{	char ch = p->var[i];	if(ch>='a' && ch<='z')	vars[ch-'a']=x[i];	}
 #pragma omp parallel for
@@ -532,6 +523,51 @@ HMDT MGL_EXPORT mgl_ode_solve_str(const char *func, const char *var, HCDT x0, mr
 	delete []par.eq;	delete []buf;	delete []xx;
 	return res;
 }
+//-----------------------------------------------------------------------------
+struct mglOdeTxtC	{	long n;	HAEX *eq;	const char *var;	};
+void MGL_NO_EXPORT mgl_txt_funcC(const mreal *x, mreal *dx, void *par)
+{
+	mglOdeTxtC *p=(mglOdeTxtC *)par;
+	dual vars[MGL_VS];
+	for(long i=0;i<p->n;i++)
+	{	char ch = p->var[i];	if(ch>='a' && ch<='z')	vars[ch-'a']=dual(x[2*i],x[2*i+1]);	}
+#pragma omp parallel for
+	for(long i=0;i<p->n;i++)
+	{
+		dual r = mgl_cexpr_eval_v(p->eq[i], vars);
+		dx[2*i] = real(r);	dx[2*i+1] = imag(r);
+	}
+}
+HADT MGL_EXPORT mgl_ode_solve_str_c(const char *func, const char *var, HCDT x0, mreal dt, mreal tmax)
+{
+	if(!var || !(*var) || !func)	return 0;
+	long len = strlen(func);
+	mglOdeTxtC par;	par.var=var;
+	par.n = strlen(var);
+	par.eq = new HAEX[par.n];
+	char *buf = new char[len+1], *f=buf, *g=f;	memcpy(buf,func,len+1);
+	mreal *xx = new mreal[2*par.n];
+	const mglDataC *c = dynamic_cast<const mglDataC *>(x0);
+	for(long i=0;i<par.n;i++)
+	{
+		if(c)	{	xx[2*i]=real(c->a[i]);	xx[2*i+1]=imag(c->a[i]);	}
+		else	{	xx[2*i] = x0?x0->vthr(i):0;	xx[2*i+1]=0;	}
+		for(long k=0;f[k];k++)	if(f[k]==';')
+		{ g = f+k+1;	f[k]=0;	break;	}
+		if(f==g)	g = f+strlen(f);
+		par.eq[i] = mgl_create_cexpr(f);
+		f = g;
+	}
+	HMDT res = mgl_ode_solve_ex(mgl_txt_funcC,2*par.n,xx,dt,tmax,&par,NULL);
+	for(long i=0;i<par.n;i++)	mgl_delete_cexpr(par.eq[i]);
+	delete []par.eq;	delete []buf;	delete []xx;
+	const long nn=par.n,nt=res->ny;
+	mglDataC *out = new mglDataC(nn, nt);
+#pragma omp parallel for
+	for(long i=0;i<nt*nn;i++)	out->a[i] = dual(res->a[2*i],res->a[2*i+1]);
+	delete res;	return out;
+}
+//-----------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_ode_solve(void (*func)(const mreal *x, mreal *dx, void *par), int n, const mreal *x0, mreal dt, mreal tmax, void *par)
 {	return mgl_ode_solve_ex(func,n,x0,dt,tmax,par,0);	}
 HMDT MGL_EXPORT mgl_ode_solve_ex(void (*func)(const mreal *x, mreal *dx, void *par), int n, const mreal *x0, mreal dt, mreal tmax, void *par, void (*bord)(mreal *x, const mreal *xp, void *par))
@@ -540,22 +576,21 @@ HMDT MGL_EXPORT mgl_ode_solve_ex(void (*func)(const mreal *x, mreal *dx, void *p
 	const long nt = int(tmax/dt+0.5)+1;
 	mglData *res=new mglData(n,nt);
 	mreal *x=new mreal[n], *k1=new mreal[n], *k2=new mreal[n], *k3=new mreal[n], *v=new mreal[n], hh=dt/2;
-	register long i,k;
 	// initial conditions
-	for(i=0;i<n;i++)	x[i] = res->a[i] = x0[i];
+	for(long i=0;i<n;i++)	x[i] = res->a[i] = x0[i];
 	// Runge Kutta scheme of 4th order
-	for(k=1;k<nt;k++)
+	for(long k=1;k<nt;k++)
 	{
 		func(x,k1,par);
-		for(i=0;i<n;i++)	v[i] = x[i]+k1[i]*hh;
+		for(long i=0;i<n;i++)	v[i] = x[i]+k1[i]*hh;
 		func(v,k2,par);
-		for(i=0;i<n;i++)	v[i] = x[i]+k2[i]*hh;
+		for(long i=0;i<n;i++)	v[i] = x[i]+k2[i]*hh;
 		func(v,k3,par);
-		for(i=0;i<n;i++)	{	v[i] = x[i]+k3[i]*dt;	k3[i] += k2[i];	}
+		for(long i=0;i<n;i++)	{	v[i] = x[i]+k3[i]*dt;	k3[i] += k2[i];	}
 		func(v,k2,par);
-		for(i=0;i<n;i++)	x[i] += (k1[i]+k2[i]+2*k3[i])*dt/6;
+		for(long i=0;i<n;i++)	x[i] += (k1[i]+k2[i]+2*k3[i])*dt/6;
 		if(bord)	bord(x,res->a+n*(k-1),par);
-		for(i=0;i<n;i++)	res->a[i+n*k] = x[i];
+		for(long i=0;i<n;i++)	res->a[i+n*k] = x[i];
 	}
 	delete []x;	delete []k1;	delete []k2;	delete []k3;	delete []v;
 	return res;
@@ -600,8 +635,7 @@ struct mgl_ap
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_init_ra(long n, int n7, const mreal *r, mgl_ap *ra)	// prepare some intermediate data for QO (3d case)
 {
-	register double tt;
-	tt = hypot(r[n7]-r[0], r[n7+1]-r[1]);
+	double tt = hypot(r[n7]-r[0], r[n7+1]-r[1]);
 	if(tt)
 	{
 		ra[0].x1 = (r[n7+1]-r[1])/tt;
@@ -679,9 +713,9 @@ MGL_NO_EXPORT void *mgl_qo2d_hprep(void *par)
 	for(long i=t->id;i<nx;i+=mglNumThr)
 	{
 		// x terms
-		register mreal x1 = (2*i-nx+1)*f->dr, hh = 1 - ra->t1*x1;
+		mreal x1 = (2*i-nx+1)*f->dr, hh = 1 - ra->t1*x1;
 		hh = sqrt(sqrt(0.041+hh*hh*hh*hh));
-		register mreal tt = (ra->pt + ra->d1*x1)/hh - ra->pt;
+		mreal tt = (ra->pt + ra->d1*x1)/hh - ra->pt;
 		f->hx[i] = f->ham(abs(f->a[i]), r[0]+ra->x1*x1, r[1]+ra->y1*x1, r[3]+ra->x0*tt, r[4]+ra->y0*tt, f->par) - f->h0/2.;
 		// u-y terms
 		x1 = f->dk/2*(i<nx/2 ? i:i-nx);
@@ -711,7 +745,7 @@ HADT MGL_EXPORT mgl_qo2d_func_c(ddual (*ham)(mreal u, mreal x, mreal y, mreal px
 	memset(dmp,0,2*nx*sizeof(double));
 	for(long i=0;i<nx/2;i++)	// prepare damping
 	{
-		register mreal x1 = (nx/2-i)/(nx/2.);
+		mreal x1 = (nx/2-i)/(nx/2.);
 		dmp[2*nx-1-i] = dmp[i] = 30*GAMMA*x1*x1/k0;
 	}
 	for(long i=0;i<nx;i++)	a[i+nx/2] = dual(ini_re->v(i),ini_im->v(i));	// init
@@ -728,7 +762,7 @@ HADT MGL_EXPORT mgl_qo2d_func_c(ddual (*ham)(mreal u, mreal x, mreal y, mreal px
 			res->a[i+k*nx]=a[i+nx/2]*sqrt(ra[0].ch/ra[k].ch);
 		if(xx && yy)	for(long i=0;i<nx;i++)	// prepare xx, yy
 		{
-			register mreal x1 = (2*i-nx+1)*dr;
+			mreal x1 = (2*i-nx+1)*dr;
 			xx->a[i+k*nx] = ray->a[n7*k] + ra[k].x1*x1;	// new coordinates
 			yy->a[i+k*nx] = ray->a[n7*k+1] + ra[k].y1*x1;
 		}
@@ -830,11 +864,11 @@ MGL_NO_EXPORT void *mgl_qo3d_hprep(void *par)
 #endif
 	for(long ii=t->id;ii<nx*nx;ii+=mglNumThr)
 	{
-		register long i = ii%nx, j = ii/nx;
+		long i = ii%nx, j = ii/nx;
 		// x-y terms
-		register mreal x1 = (2*i-nx+1)*f->dr, x2 = (2*j-nx+1)*f->dr, hh = 1-ra->t1*x1-ra->t2*x2;
+		mreal x1 = (2*i-nx+1)*f->dr, x2 = (2*j-nx+1)*f->dr, hh = 1-ra->t1*x1-ra->t2*x2;
 		hh = sqrt(sqrt(0.041+hh*hh*hh*hh));
-		register mreal tt = (ra->pt + ra->d1*x1 + ra->d2*x2)/hh - ra->pt;
+		mreal tt = (ra->pt + ra->d1*x1 + ra->d2*x2)/hh - ra->pt;
 		f->hxy[ii] = f->ham(abs(f->a[i]), r[0]+ra->x1*x1+ra->x2*x2, r[1]+ra->y1*x1+ra->y2*x2, r[2]+ra->z1*x1+ra->z2*x2, r[3]+ra->x0*tt, r[4]+ra->y0*tt, r[5]+ra->z0*tt, f->par);
 		// x-v terms
 		x1 = (2*i-nx+1)*f->dr;	x2 = f->dk/2*(j<nx/2 ? j:j-nx);	hh = 1-ra->t1*x1;
@@ -863,7 +897,7 @@ MGL_NO_EXPORT void *mgl_qo3d_post(void *par)
 #endif
 	for(long ii=t->id;ii<nx*nx;ii+=mglNumThr)
 	{
-		register long i = ii%nx, j = ii/nx;
+		long i = ii%nx, j = ii/nx;
 		f->hxy[ii] -= (f->hx[i]+f->hy[j]-f->h0/2.)/2.;
 		if(imag(f->hxy[ii])>0)	f->hxy[ii] = f->hxy[ii].real();
 		f->hxv[ii] -= (f->hx[i]+f->hv[j]-f->h0/2.)/2.;
@@ -897,7 +931,7 @@ HADT MGL_EXPORT mgl_qo3d_func_c(ddual (*ham)(mreal u, mreal x, mreal y, mreal z,
 #pragma omp parallel for collapse(2)
 	for(long i=0;i<nx/2;i++)	for(long j=0;j<nx/2;j++)	// prepare damping
 	{
-		register double x1 = (nx/2-i)/(nx/2.), x2 = (nx/2-j)/(nx/2.);
+		double x1 = (nx/2-i)/(nx/2.), x2 = (nx/2-j)/(nx/2.);
 		dmp[2*nx-1-i] = dmp[i] = 30*GAMMA*x1*x1/k0;
 		dmp[(2*nx-1-j)*2*nx] += 30*GAMMA*x2*x2/k0;
 		dmp[j*2*nx] += 30*GAMMA*x2*x2/k0;
@@ -923,7 +957,7 @@ HADT MGL_EXPORT mgl_qo3d_func_c(ddual (*ham)(mreal u, mreal x, mreal y, mreal z,
 #pragma omp parallel for collapse(2)
 			for(long i=0;i<nx;i++)	for(long j=0;j<nx;j++)	// prepare xx, yy, zz
 			{
-				register mreal x1 = (2*i-nx+1)*dr, x2 = (2*j-nx+1)*dr;
+				mreal x1 = (2*i-nx+1)*dr, x2 = (2*j-nx+1)*dr;
 				xx->a[i+nx*(j+k*nx)] = ray->a[n7*k] + ra[k].x1*x1 + ra[k].x2*x2;	// new coordinates
 				yy->a[i+nx*(j+k*nx)] = ray->a[n7*k+1] + ra[k].y1*x1 + ra[k].y2*x2;
 				zz->a[i+nx*(j+k*nx)] = ray->a[n7*k+2] + ra[k].z1*x1 + ra[k].z2*x2;
@@ -1041,12 +1075,12 @@ MGL_NO_EXPORT void *mgl_jacob2(void *par)
 #endif
 	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
-		register long i=i0%nx, j=i0/nx;
-		register long ip = i<nx-1 ? 1:0, jp = j<ny-1 ? nx:0;
-		register long im = i>0 ? -1:0, jm = j>0 ? -nx:0;
+		long i=i0%nx, j=i0/nx;
+		long ip = i<nx-1 ? 1:0, jp = j<ny-1 ? nx:0;
+		long im = i>0 ? -1:0, jm = j>0 ? -nx:0;
 		r[i0] = (x[i0+ip]-x[i0+im])*(y[i0+jp]-y[i0+jm]) -
 				(y[i0+ip]-y[i0+im])*(x[i0+jp]-x[i0+jm]);
-		r[i0] *= mreal((nx-1)*(ny-1)) / ((ip-im)*(jp-jm));
+		r[i0] *= mreal((nx-1)*(ny-1)) / mreal((ip-im)*(jp-jm));
 	}
 	return 0;
 }
@@ -1067,11 +1101,11 @@ HMDT MGL_EXPORT mgl_jacobian_2d(HCDT x, HCDT y)
 #pragma omp parallel for collapse(2)
 		for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 		{
-			register long im = i>0 ? i-1:i, ip = i<nx-1 ? i+1:i;
-			register long jm = j>0 ? j-1:j, jp = j<ny-1 ? j+1:j;
+			long im = i>0 ? i-1:i, ip = i<nx-1 ? i+1:i;
+			long jm = j>0 ? j-1:j, jp = j<ny-1 ? j+1:j;
 			r->a[i+nx*j] = (x->v(ip,j)-x->v(im,j))*(y->v(i,jp)-y->v(i,jm)) -
 						(y->v(ip,j)-y->v(im,j))*(x->v(i,jp)-x->v(i,jm));
-			r->a[i+nx*j] *= mreal((nx-1)*(ny-1)) / ((ip-im)*(jp-jm));
+			r->a[i+nx*j] *= mreal((nx-1)*(ny-1)) / mreal((ip-im)*(jp-jm));
 		}
 	}
 	return r;
@@ -1088,16 +1122,16 @@ MGL_NO_EXPORT void *mgl_jacob3(void *par)
 #endif
 	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
-		register long i=i0%nx, j=(i0/nx)%ny, k=i0/(nx*ny);
-		register long ip = i<nx-1 ? 1:0, jp = j<ny-1 ? nx:0, kp = k<nz-1 ? nx*ny:0;
-		register long im = i>0 ? -1:0, jm = j>0 ? -nx:0, km = k>0 ? -nx*ny:0;
+		long i=i0%nx, j=(i0/nx)%ny, k=i0/(nx*ny);
+		long ip = i<nx-1 ? 1:0, jp = j<ny-1 ? nx:0, kp = k<nz-1 ? nx*ny:0;
+		long im = i>0 ? -1:0, jm = j>0 ? -nx:0, km = k>0 ? -nx*ny:0;
 		r[i0] = (x[i0+ip]-x[i0+im])*(y[i0+jp]-y[i0+jm])*(z[i0+kp]-z[i0+km]) -
 				(x[i0+ip]-x[i0+im])*(y[i0+kp]-y[i0+km])*(z[i0+jp]-z[i0+jm]) -
 				(x[i0+jp]-x[i0+jm])*(y[i0+ip]-y[i0+im])*(z[i0+kp]-z[i0+km]) +
 				(x[i0+jp]-x[i0+jm])*(y[i0+kp]-y[i0+km])*(z[i0+ip]-z[i0+im]) +
 				(x[i0+kp]-x[i0+km])*(y[i0+ip]-y[i0+im])*(z[i0+jp]-z[i0+jm]) -
 				(x[i0+kp]-x[i0+km])*(y[i0+jp]-y[i0+jm])*(z[i0+ip]-z[i0+im]);
-		r[i0] *= mreal((nx-1)*(ny-1)*(nz-1)) / ((ip-im)*(jp-jm)*(kp-km));
+		r[i0] *= mreal((nx-1)*(ny-1)*(nz-1)) / mreal((ip-im)*(jp-jm)*(kp-km));
 	}
 	return 0;
 }
@@ -1120,17 +1154,17 @@ HMDT MGL_EXPORT mgl_jacobian_3d(HCDT x, HCDT y, HCDT z)
 #pragma omp parallel for collapse(3)
 		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 		{
-			register long im = i>0 ? i-1:i, ip = i<nx-1 ? i+1:i;
-			register long jm = j>0 ? j-1:j, jp = j<ny-1 ? j+1:j;
-			register long km = k>0 ? k-1:k, kp = k<nz-1 ? k+1:k;
-			register long i0 = i+nx*(j+ny*k);
+			long im = i>0 ? i-1:i, ip = i<nx-1 ? i+1:i;
+			long jm = j>0 ? j-1:j, jp = j<ny-1 ? j+1:j;
+			long km = k>0 ? k-1:k, kp = k<nz-1 ? k+1:k;
+			long i0 = i+nx*(j+ny*k);
 			r->a[i0] = (x->v(ip,j,k)-x->v(im,j,k))*(y->v(i,jp,k)-y->v(i,jm,k))*(z->v(i,j,kp)-z->v(i,j,km)) -
 					(x->v(ip,j,k)-x->v(im,j,k))*(y->v(i,j,kp)-y->v(i,j,km))*(z->v(i,jp,k)-z->v(i,jm,k)) -
 					(x->v(i,jp,k)-x->v(i,jm,k))*(y->v(ip,j,k)-y->v(im,j,k))*(z->v(i,j,kp)-z->v(i,j,km)) +
 					(x->v(i,jp,k)-x->v(i,jm,k))*(y->v(i,j,kp)-y->v(i,j,km))*(z->v(ip,j,k)-z->v(im,j,k)) +
 					(x->v(i,j,kp)-x->v(i,j,km))*(y->v(ip,j,k)-y->v(im,j,k))*(z->v(i,jp,k)-z->v(i,jm,k)) -
 					(x->v(i,j,kp)-x->v(i,j,km))*(y->v(i,jp,k)-y->v(i,jm,k))*(z->v(ip,j,k)-z->v(im,j,k));
-			r->a[i0] *= mreal((nx-1)*(ny-1)*(nz-1)) / ((ip-im)*(jp-jm)*(kp-km));
+			r->a[i0] *= mreal((nx-1)*(ny-1)*(nz-1)) / mreal((ip-im)*(jp-jm)*(kp-km));
 		}
 
 	}
@@ -1154,7 +1188,7 @@ void MGL_NO_EXPORT mgl_progonka_sr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, l
 	aa[0] = -c0/b0;	bb[0] = d0/b0;
 	for(long i=1;i<n;i++)
 	{
-		register long ii=i0+di*i, dd=id+di*i, tt = id+di*((i+1)%n);
+		long ii=i0+di*i, dd=id+di*i, tt = id+di*((i+1)%n);
 		mreal a=A->vthr(ii), b=B->vthr(ii), c=C->vthr(ii);
 		mreal d=difr?-a*D->vthr(dd-di)+(2.-b)*D->vthr(dd)-c*D->vthr(tt):D->vthr(dd);
 		aa[i] = -c/(b+a*aa[i-1]);
@@ -1171,7 +1205,7 @@ void MGL_NO_EXPORT mgl_progonka_pr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, l
 	aa[0] =-c0/b0;	bb[0] = d0/b0;	gg[0] =-a0/b0;
 	for(long i=1;i<n;i++)
 	{
-		register long ii=i0+di*i, il=id+di*((i+1)%n), dd=id+di*i;
+		long ii=i0+di*i, il=id+di*((i+1)%n), dd=id+di*i;
 		mreal a=A->vthr(ii), b=B->vthr(ii), c=C->vthr(ii);
 		mreal d=difr?-a*D->vthr(dd-di)+(2.-b)*D->vthr(dd)-c*D->vthr(il):D->vthr(dd);
 		aa[i] = -c/(b+a*aa[i-1]);
@@ -1205,7 +1239,7 @@ void MGL_NO_EXPORT mgl_progonka_hr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, l
 		aa[0] = -c0/b0;	bb[0] = d0/b0;
 		for(long i=1;i<=j;i++)
 		{
-			register long ii=i0+j+di*i, dd=id+j+di*i;
+			long ii=i0+j+di*i, dd=id+j+di*i;
 			mreal a=A->vthr(ii),b=B->vthr(ii),c=C->vthr(ii);
 			mreal d=difr?-a*D->vthr(dd-di)+(2.-b)*D->vthr(dd)-c*D->vthr(dd+di):D->vthr(dd);
 			aa[i] = -c/(b+a*aa[i-1]);
@@ -1221,7 +1255,7 @@ void MGL_NO_EXPORT mgl_progonka_hr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, l
 		aa[0] = -c0/b0;	bb[0] = d0/b0;
 		for(long i=1;i<=j;i++)
 		{
-			register long ii=i1+j1-di*i, dd=d1+j1-di*i;
+			long ii=i1+j1-di*i, dd=d1+j1-di*i;
 			mreal a=A->vthr(ii),b=B->vthr(ii),c=C->vthr(ii);
 			mreal d=difr?-a*D->vthr(dd+di)+(2.-b)*D->vthr(dd)-c*D->vthr(dd-di):D->vthr(dd);
 			aa[i] = -c/(b+a*aa[i-1]);
@@ -1310,13 +1344,13 @@ void MGL_NO_EXPORT mgl_progonka_sc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, lo
 {
 	dual *aa=dat, *bb=dat+n, *uu=dat+2*n;
 	dual b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(id);
-	if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(id+di);
+	if(difr)	d0 = (mreal(2)-b0)*d0-c0*D->vcthr(id+di);
 	aa[0] = -c0/b0;	bb[0] = d0/b0;
 	for(long i=1;i<n;i++)
 	{
-		register long ii=i0+di*i, dd=id+di*i, tt = id+di*((i+1)%n);
+		long ii=i0+di*i, dd=id+di*i, tt = id+di*((i+1)%n);
 		dual a=A->vcthr(ii), b=B->vcthr(ii), c=C->vcthr(ii);
-		dual d=difr?-a*D->vcthr(dd-di)+(2.-b)*D->vcthr(dd)-c*D->vcthr(tt):D->vcthr(dd);
+		dual d=difr?-a*D->vcthr(dd-di)+(mreal(2)-b)*D->vcthr(dd)-c*D->vcthr(tt):D->vcthr(dd);
 		aa[i] = -c/(b+a*aa[i-1]);
 		bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 	}
@@ -1327,47 +1361,47 @@ void MGL_NO_EXPORT mgl_progonka_pc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, lo
 {
 	dual *aa=dat, *bb=dat+n, *gg=dat+2*n, *uu=dat+3*n;
 	dual a0=A->vcthr(i0), b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(id);
-	if(difr)	d0 = -a0*D->vcthr(id+di*(n-1))+(2.-b0)*d0-c0*D->vcthr(id+di);
+	if(difr)	d0 = -a0*D->vcthr(id+di*(n-1))+(mreal(2)-b0)*d0-c0*D->vcthr(id+di);
 	aa[0] =-c0/b0;	bb[0] = d0/b0;	gg[0] =-a0/b0;
 	for(long i=1;i<n;i++)
 	{
-		register long ii=i0+di*i, il=id+di*((i+1)%n), dd=id+di*i;
+		long ii=i0+di*i, il=id+di*((i+1)%n), dd=id+di*i;
 		dual a=A->vcthr(ii), b=B->vcthr(ii), c=C->vcthr(ii);
-		dual d=difr?-a*D->vcthr(dd-di)+(2.-b)*D->vcthr(dd)-c*D->vcthr(il):D->vcthr(dd);
+		dual d=difr?-a*D->vcthr(dd-di)+(mreal(2)-b)*D->vcthr(dd)-c*D->vcthr(il):D->vcthr(dd);
 		aa[i] = -c/(b+a*aa[i-1]);
 		bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 		gg[i] = -a*gg[i-1]/(b+a*aa[i-1]);
 	}
-	dual P=bb[n-1]/(1.-gg[n-1]), Q=aa[n-1]/(1.-gg[n-1]);
+	dual P=bb[n-1]/(mreal(1)-gg[n-1]), Q=aa[n-1]/(mreal(1)-gg[n-1]);
 	aa[n-1] = Q;	bb[n-1] = P;
 	for(long i=n-2;i>=0;i--)
 	{
 		bb[i] += aa[i]*bb[i+1]+gg[i]*P;
 		aa[i] = aa[i]*aa[i+1]+gg[i]*Q;
 	}
-	dual u0 = bb[0]/(1.-aa[0]);
+	dual u0 = bb[0]/(mreal(1)-aa[0]);
 	for(long i=0;i<n;i++)	uu[i]=bb[i]+aa[i]*u0;
 }
 void MGL_NO_EXPORT mgl_progonka_hc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long id, long i0, bool difr)
 {
 	dual *aa=dat, *bb=dat+n, *uu=dat+n*n;
 	dual b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(id);
-	uu[0] = d0/b0*(difr?(2.-b0):1.);
+	uu[0] = d0/b0*(difr?(mreal(2)-b0):mreal(1));
 	b0=B->vcthr(i0+n*n-1);	d0=D->vcthr(id+n*n-1);
-	uu[n*n-1] = d0/b0*(difr?(2.-b0):1.);
+	uu[n*n-1] = d0/b0*(difr?(mreal(2)-b0):mreal(1));
 	long di = n-1, i1 = i0+n*(n-1), d1 = id+n*(n-1);
 	// suppose the square grid!
 	for(long j=1;j<n;j++)
 	{
 		// first bottom-left triangle
 		b0=B->vcthr(i0+j);	c0=C->vcthr(i0+j);	d0=D->vcthr(id+j);
-		if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(id+j+di);
+		if(difr)	d0 = (mreal(2)-b0)*d0-c0*D->vcthr(id+j+di);
 		aa[0] = -c0/b0;	bb[0] = d0/b0;
 		for(long i=1;i<=j;i++)
 		{
-			register long ii=i0+j+di*i, dd=id+j+di*i;
+			long ii=i0+j+di*i, dd=id+j+di*i;
 			dual a=A->vcthr(ii),b=B->vcthr(ii),c=C->vcthr(ii);
-			dual d=difr?-a*D->vcthr(dd-di)+(2.-b)*D->vcthr(dd)-c*D->vcthr(dd+di):D->vcthr(dd);
+			dual d=difr?-a*D->vcthr(dd-di)+(mreal(2)-b)*D->vcthr(dd)-c*D->vcthr(dd+di):D->vcthr(dd);
 			aa[i] = -c/(b+a*aa[i-1]);
 			bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 		}
@@ -1377,13 +1411,13 @@ void MGL_NO_EXPORT mgl_progonka_hc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, lo
 		// next top-right triangle
 		long j1=n-1-j;
 		b0=B->vcthr(i1+j1);	c0=C->vcthr(i1+j1);	d0=D->vcthr(d1+j1);
-		if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(d1+j1-di);
+		if(difr)	d0 = (mreal(2)-b0)*d0-c0*D->vcthr(d1+j1-di);
 		aa[0] = -c0/b0;	bb[0] = d0/b0;
 		for(long i=1;i<=j;i++)
 		{
-			register long ii=i1+j1-di*i, dd=d1+j1-di*i;
+			long ii=i1+j1-di*i, dd=d1+j1-di*i;
 			dual a=A->vcthr(ii),b=B->vcthr(ii),c=C->vcthr(ii);
-			dual d=difr?-a*D->vcthr(dd+di)+(2.-b)*D->vcthr(dd)-c*D->vcthr(dd-di):D->vcthr(dd);
+			dual d=difr?-a*D->vcthr(dd+di)+(mreal(2)-b)*D->vcthr(dd)-c*D->vcthr(dd-di):D->vcthr(dd);
 			aa[i] = -c/(b+a*aa[i-1]);
 			bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 		}
