@@ -17,6 +17,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+#include <algorithm>
 #include "mgl2/plot.h"
 #include "mgl2/eval.h"
 #include "mgl2/data.h"
@@ -33,105 +34,101 @@ void MGL_EXPORT mgl_fplot(HMGL gr, const char *eqY, const char *pen, const char
 	long n = (mgl_isnan(r) || r<=0) ? 100:long(r+0.5);
 	long nm = gr->FaceNum?gr->FaceNum*n:10000, nd = gr->FaceNum?gr->FaceNum*10:1000;
 
-	mreal *x = (mreal *)malloc(n*sizeof(mreal));
-	mreal *y = (mreal *)malloc(n*sizeof(mreal));
+	mglDataS x, y;
+	x.dat.reserve(nm);	y.dat.reserve(nm);
+
 	mglFormula *eq = new mglFormula(eqY);
-	mreal xs, ys, yr, ym=fabs(gr->Max.y - gr->Min.y)/nd;
 	// initial data filling
+	x.clear();	y.clear();
 	if(gr->Min.x>0 && gr->Max.x>100*gr->Min.x)
 	{
 		mreal d = log(2*gr->Max.x/gr->Min.x)/(n-1);
 		for(long i=0;i<n;i++)
-		{	x[i]=2*gr->Max.x*exp(d*i)/(2*gr->Max.x/gr->Min.x+exp(d*i));	y[i]=eq->Calc(x[i]);	}
+		{	mreal xx = 2*gr->Max.x*exp(d*i)/(2*gr->Max.x/gr->Min.x+exp(d*i));
+			x.dat.push_back(xx);	y.dat.push_back(eq->Calc(xx));	}
 	}
 	else if(gr->Max.x<0 && gr->Min.x<100*gr->Max.x)
 	{
 		mreal d = log(2*gr->Min.x/gr->Max.x)/(n-1);
 		for(long i=0;i<n;i++)
-		{	x[i]=2*gr->Min.x*exp(d*i)/(2*gr->Min.x/gr->Max.x+exp(d*i));	y[i]=eq->Calc(x[i]);	}
+		{	mreal xx = 2*gr->Min.x*exp(d*i)/(2*gr->Min.x/gr->Max.x+exp(d*i));
+			x.dat.push_back(xx);	y.dat.push_back(eq->Calc(xx));	}
 	}
 	else
 	{
 		mreal d = (gr->Max.x - gr->Min.x)/(n-1.);
 		for(long i=0;i<n;i++)
-		{	x[i]=gr->Min.x + i*d;	y[i]=eq->Calc(x[i]);	}
+		{	mreal xx = gr->Min.x + i*d;
+			x.dat.push_back(xx);	y.dat.push_back(eq->Calc(xx));	}
 	}
 
-	for(long i=0;i<n-1 && n<nm;)
+	bool check=true;
+	mreal ym=fabs(gr->Max.y - gr->Min.y)/nd;
+	while(check && long(x.dat.size())<nm)
 	{
-		if((i&0xfff)==0 && gr->NeedStop())
-		{	free(x);	free(y);	delete eq;	return;	}
-		xs=(x[i]+x[i+1])/2;
-		ys=(y[i]+y[i+1])/2;	yr=eq->Calc(xs);
-		if(fabs(yr-ys)>ym)	// bad approximation here
+		if(gr->NeedStop())	{	delete eq;	return;	}
+		check = false;
+		for(long i=1;i<long(x.size());i++)
 		{
-			x = (mreal *)realloc(x,(n+1)*sizeof(mreal));
-			y = (mreal *)realloc(y,(n+1)*sizeof(mreal));
-			memmove(x+i+2,x+i+1,(n-i-1)*sizeof(mreal));
-			memmove(y+i+2,y+i+1,(n-i-1)*sizeof(mreal));
-			x[i+1] = xs;	y[i+1] = yr;	n++;
+			mreal xs=(x[i]+x[i-1])/2;
+			mreal ys=(y[i]+y[i-1])/2, yr=eq->Calc(xs);
+			if(fabs(yr-ys)>ym)	// bad approximation here
+			{
+				x.dat.insert(x.dat.begin()+i,xs);
+				y.dat.insert(y.dat.begin()+i,yr);
+				check = true;	i++;
+			}
 		}
-		else i++;
 	}
-
-	delete eq;
-	mglData yy(y,n),xx(x,n);
-	free(x);	free(y);
-	mgl_plot_xy(gr,&xx,&yy,pen,0);
+	mgl_plot_xy(gr,&x,&y,pen,0);
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_fplot_xyz(HMGL gr, const char *eqX, const char *eqY, const char *eqZ, const char *pen, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long n = (mgl_isnan(r) || r<=0) ? 100:long(r+0.5);
+	long nm = gr->FaceNum?gr->FaceNum*n:10000, nd = gr->FaceNum?gr->FaceNum*10:1000;
 
-	mreal *x = (mreal *)malloc(n*sizeof(mreal));
-	mreal *y = (mreal *)malloc(n*sizeof(mreal));
-	mreal *z = (mreal *)malloc(n*sizeof(mreal));
-	mreal *t = (mreal *)malloc(n*sizeof(mreal));
+	mglDataS x, y, z, t;
+	x.dat.reserve(nm);	y.dat.reserve(nm);
+	z.dat.reserve(nm);	t.dat.reserve(nm);
 	mglFormula *ex, *ey, *ez;
 	ex = new mglFormula(eqX ? eqX : "0");
 	ey = new mglFormula(eqY ? eqY : "0");
 	ez = new mglFormula(eqZ ? eqZ : "0");
-	mreal ts, xs, ys, zs, xr, yr, zr, xm=fabs(gr->Max.x - gr->Min.x)/1000, ym=fabs(gr->Max.y - gr->Min.y)/1000, zm=fabs(gr->Max.z - gr->Min.z)/1000;
+	t.clear();	x.clear();	y.clear();	z.clear();
 	for(long i=0;i<n;i++)	// initial data filling
 	{
-		t[i] = i/(n-1.);
-		x[i] = ex->Calc(0,0,t[i]);
-		y[i] = ey->Calc(0,0,t[i]);
-		z[i] = ez->Calc(0,0,t[i]);
+		mreal tt = i/(n-1.);	t.push_back(tt);
+		x.push_back(ex->Calc(0,0,t[i]));
+		y.push_back(ey->Calc(0,0,t[i]));
+		z.push_back(ez->Calc(0,0,t[i]));
 	}
 
-	for(long i=0;i<n-1 && n<10000;)
+	bool check=true;
+	mreal xm=fabs(gr->Max.x-gr->Min.x)/nd, ym=fabs(gr->Max.y-gr->Min.y)/nd, zm=fabs(gr->Max.z-gr->Min.z)/nd;
+	while(check && long(x.dat.size())<nm)
 	{
-		if((i&0xfff)==0 && gr->NeedStop())
+		if(gr->NeedStop())	{	delete ex;	delete ey;	delete ez;	return;	}
+		check = false;
+		for(long i=1;i<long(t.size());i++)
 		{
-			free(x);	free(y);	free(z);	free(t);
-			delete ex;	delete ey;	delete ez;	return;
+			mreal ts=(t[i]+t[i-1])/2;
+			mreal xs=(x[i]+x[i-1])/2, xr=ex->Calc(0,0,ts);
+			mreal ys=(y[i]+y[i-1])/2, yr=ey->Calc(0,0,ts);
+			mreal zs=(z[i]+z[i-1])/2, zr=ez->Calc(0,0,ts);
+			if(fabs(xr-xs)>xm || fabs(yr-ys)>ym || fabs(zr-zs)>zm)	// bad approximation here
+			{
+				t.dat.insert(t.dat.begin()+i,ts);
+				x.dat.insert(x.dat.begin()+i,xr);
+				y.dat.insert(y.dat.begin()+i,yr);
+				z.dat.insert(z.dat.begin()+i,zr);
+				check = true;	i++;
+			}
 		}
-		ts=(t[i]+t[i+1])/2;
-		xs=(x[i]+x[i+1])/2;	xr=ex->Calc(0,0,ts);
-		ys=(y[i]+y[i+1])/2;	yr=ey->Calc(0,0,ts);
-		zs=(z[i]+z[i+1])/2;	zr=ez->Calc(0,0,ts);
-		if(fabs(xr-xs)>xm || fabs(yr-ys)>ym || fabs(zr-zs)>zm)	// bad approximation here
-		{
-			z = (mreal *)realloc(z,(n+1)*sizeof(mreal));
-			t = (mreal *)realloc(t,(n+1)*sizeof(mreal));
-			x = (mreal *)realloc(x,(n+1)*sizeof(mreal));
-			y = (mreal *)realloc(y,(n+1)*sizeof(mreal));
-			memmove(x+i+2,x+i+1,(n-i-1)*sizeof(mreal));
-			memmove(y+i+2,y+i+1,(n-i-1)*sizeof(mreal));
-			memmove(z+i+2,z+i+1,(n-i-1)*sizeof(mreal));
-			memmove(t+i+2,t+i+1,(n-i-1)*sizeof(mreal));
-			t[i+1]=ts;	x[i+1]=xr;	y[i+1]=yr;	z[i+1]=zr;	n++;
-		}
-		else i++;
 	}
 	delete ex;	delete ey;	delete ez;
-
-	mglData xx(x,n),yy(y,n),zz(z,n);
-	free(x);	free(y);	free(z);	free(t);
-	mgl_plot_xyz(gr,&xx,&yy,&zz,pen,0);
+	mgl_plot_xyz(gr,&x,&y,&z,pen,0);
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_fplot_(uintptr_t *gr, const char *fy, const char *stl, const char *opt, int ly, int ls, int lo)
@@ -167,7 +164,7 @@ void MGL_EXPORT mgl_radar(HMGL gr, HCDT a, const char *pen, const char *opt)
 	{
 		for(long i=0;i<n;i++)
 		{
-			register mreal v = a->v(i,j);
+			mreal v = a->v(i,j);
 			x.a[i+(n+1)*j] = (r+v)*co[i];
 			y.a[i+(n+1)*j] = (r+v)*co[i+n];
 		}
@@ -221,7 +218,7 @@ void MGL_EXPORT mgl_candle_xyv(HMGL gr, HCDT x, HCDT v1, HCDT v2, HCDT y1, HCDT
 	mreal dv=nx>n?1:0;
 	if(mglchr(pen,'<'))	dv = 1;
 	if(mglchr(pen,'^'))	dv = 0;
-	if(mglchr(pen,'>'))	dv = -1;
+	if(mglchr(pen,'>'))	dv =-1;
 	mreal zm = gr->AdjustZMin();
 	mreal c1,c2;	c2=c1=gr->NextColor(pal);
 	bool col2 = (gr->GetNumPal(pal)==2 && !sh);
@@ -250,7 +247,8 @@ void MGL_EXPORT mgl_candle_xyv(HMGL gr, HCDT x, HCDT v1, HCDT v2, HCDT y1, HCDT
 		gr->line_plot(n4,n2);	gr->line_plot(n4,n3);
 		if(m1>m2 || (col2 && !wire))	gr->quad_plot(n1,n2,n3,n4);
 	}
-	if(d1)	delete y1;	if(d2)	delete y2;
+	if(d1)	delete y1;
+	if(d2)	delete y2;
 	gr->EndGroup();
 }
 //-----------------------------------------------------------------------------
@@ -289,6 +287,43 @@ void MGL_EXPORT mgl_candle_(uintptr_t *gr, uintptr_t *y, uintptr_t *y1, uintptr_
 //	Plot series
 //
 //-----------------------------------------------------------------------------
+struct mglPointA {	mglPoint p;	bool orig;	mglPointA(const mglPoint &pp, bool o) : p(pp), orig(o) {}	};
+std::vector<mglPointA> MGL_NO_EXPORT mgl_pnt_prepare(const mglPoint &p1, const mglPoint &p2, HCDT xx, HCDT yy, HCDT zz, HCDT cc)
+{
+	std::vector<mglPointA> out;
+	long n = xx->GetNx();
+	mglPoint p(xx->v(0),yy->v(0),zz->v(0),cc?cc->v(0):0);
+	if(p>p1 && p<p2)	out.push_back(mglPointA(p,true));
+	else	out.push_back(mglPointA(mglPoint(NAN),true));
+	for(long i=1;i<n;i++)
+	{
+		mglPoint q(xx->v(i),yy->v(i),zz->v(i),cc?cc->v(i):0);
+		mreal x1,x2,y1,y2,z1,z2,t;
+		x1=mgl_d(p1.x, p.x, q.x);	x2=mgl_d(p2.x, p.x, q.x);	if(x2<x1)	{	t=x1;	x1=x2;	x2=t;	}
+		y1=mgl_d(p1.y, p.y, q.y);	y2=mgl_d(p2.y, p.y, q.y);	if(y2<y1)	{	t=y1;	y1=y2;	y2=t;	}
+		z1=mgl_d(p1.z, p.z, q.z);	z2=mgl_d(p2.z, p.z, q.z);	if(z2<z1)	{	t=z1;	z1=z2;	z2=t;	}
+		mreal d1 = mgl_isnum(x1)?x1:0, d2 = mgl_isnum(x2)?x2:1;
+		if(y1>d1)	d1=y1;
+		if(y2<d2)	d2=y2;
+		if(z1>d1)	d1=z1;
+		if(z2<d2)	d2=z2;
+		if(d1>0 && d1<1)	out.push_back(mglPointA(p+d1*(q-p),false));
+		if(d2>0 && d2<1)	out.push_back(mglPointA(p+d2*(q-p),false));
+		if(d1<1 && d2>=1)	out.push_back(mglPointA(q,true));
+		else if(i==n-1)	out.push_back(mglPointA(mglPoint(NAN),true));
+		p = q;
+	}
+	return out;
+}
+std::vector<mglPointA> MGL_NO_EXPORT mgl_pnt_copy(HCDT xx, HCDT yy, HCDT zz, HCDT cc)
+{
+	std::vector<mglPointA> out;
+	long n = xx->GetNx();
+	for(long i=0;i<n;i++)
+		out.push_back(mglPointA(mglPoint(xx->v(i),yy->v(i),zz->v(i),cc?cc->v(i):0),true));
+	return out;
+}
+//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_mark(HMGL gr, double x, double y, double z,const char *mark);
 void MGL_EXPORT mgl_plot_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, const char *opt)
 {
@@ -313,48 +348,31 @@ void MGL_EXPORT mgl_plot_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 	gr->SaveState(opt);
 	long m = x->GetNy() > y->GetNy() ? x->GetNy() : y->GetNy();	m = z->GetNy() > m ? z->GetNy() : m;
 	char mk=gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
-	mglPoint p1,nn,p2,pt;
-	long n1=-1,n2=-1,n3=-1;
-	bool sh = mglchr(pen,'!');
+	bool sh = mglchr(pen,'!'), orig = !mglchr(pen,'a');
 
+	int d = gr->MeshNum>0 ? gr->MeshNum+1 : n, dx = n>d?n/d:1;
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
 		long mx = j<x->GetNy() ? j:0, my = j<y->GetNy() ? j:0, mz = j<z->GetNy() ? j:0;
 		gr->NextColor(pal);
-		bool t1 = false, t2 = false;
-		for(long i=0;i<n;i++)
+		mglDataR xx(x,mx), yy(y,my), zz(z,mz);
+		const std::vector<mglPointA> &pp = orig ? mgl_pnt_copy(&xx, &yy, &zz, 0) :
+			mgl_pnt_prepare(gr->Min, gr->Max, &xx, &yy, &zz, 0);
+		long n1=-1, n2=-1;
+		
+		for(size_t i=0;i<pp.size();i++)
 		{
-			if(i>0)	{	n2=n1;	p2 = p1;	t2=t1;	}
-			p1.Set(x->v(i,mx), y->v(i,my), z->v(i,mz));
 			mreal c = sh ? gr->NextColor(pal,i):gr->CDef;
-			n1 = gr->AddPnt(p1,c);	t1 = n1>=0;
-			if(mk && t1)	gr->mark_plot(n1,mk);
-			if(t1 && t2)
+			n2 = n1;	n1 = gr->AddPnt(pp[i].p, c);
+			if(mk && i%dx==0 && n1>=0 && pp[i].orig)	gr->mark_plot(n1,mk);
+			if(n1>=0 && n2>=0)
 			{
 				gr->line_plot(n1,n2);
 				if(i==1)	gr->arrow_plot(n2,n1,gr->Arrow1);
-				if(i==n-1)	gr->arrow_plot(n1,n2,gr->Arrow2);
-			}
-
-			if(i>0 && ((t1 && !t2) || (t2 && !t1)))	// do smoothing
-			{
-				mreal i1=0, i2=1, ii;
-				mglPoint q(NAN),p;
-				do {
-					ii = (i1+i2)/2;
-					pt.x = p1.x*ii+p2.x*(1-ii);
-					pt.y = p1.y*ii+p2.y*(1-ii);
-					pt.z = p1.z*ii+p2.z*(1-ii);	p=pt;
-					bool t3 = gr->ScalePoint(gr->GetB(),p,q,false);
-					if((t1 && t3) || (t2 && !t3))	i2 = ii;
-					else	i1 = ii;
-				} while(fabs(i2-i1)>1e-3);
-				n3 = gr->AddPnt(pt,c,q,-1,2);
-				if(t2)	gr->line_plot(n3,n2);
-				else 	gr->line_plot(n1,n3);
 			}
 		}
+		if(n1>=0 && n2>=0)	gr->arrow_plot(n1,n2,gr->Arrow2);
 	}
 	gr->EndGroup();
 }
@@ -368,7 +386,7 @@ void MGL_EXPORT mgl_plot_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const char
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_plot(HMGL gr, HCDT y, const char *pen, const char *opt)
 {
-	register long n=y->GetNx();
+	long n=y->GetNx();
 	if(n<2)	{	gr->SetWarn(mglWarnLow,"Plot");	return;	}
 	gr->SaveState(opt);
 	mglDataV x(n), z(n);
@@ -405,48 +423,29 @@ void MGL_EXPORT mgl_tens_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, const char
 	m = x->GetNy() > y->GetNy() ? x->GetNy() : y->GetNy();	m = z->GetNy() > m ? z->GetNy() : m;
 	char mk=gr->SetPenPal(pen, &pal);	gr->Reserve(2*n*m);
 	long ss=gr->AddTexture(pen);
-	mglPoint p1,p2,pt,nn;
-	long n1=-1,n2=-1,n3=-1;
+	bool orig = !mglchr(pen,'a');
 
+	int d = gr->MeshNum>0 ? gr->MeshNum+1 : n, dx = n>d?n/d:1;
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
 		long mx = j<x->GetNy() ? j:0, my = j<y->GetNy() ? j:0;
 		long mz = j<z->GetNy() ? j:0, mc = j<c->GetNy() ? j:0;
-		register long i;
-		bool t1 = false, t2 = false;
-		for(i=0;i<n;i++)
+		mglDataR xx(x,mx), yy(y,my), zz(z,mz), cc(c,mc);
+		const std::vector<mglPointA> &pp = orig ? mgl_pnt_copy(&xx, &yy, &zz, &cc) :
+			mgl_pnt_prepare(gr->Min, gr->Max, &xx, &yy, &zz, &cc);
+		long n1=-1, n2=-1;
+		for(size_t i=0;i<pp.size();i++)
 		{
-			if(i>0)	{	n2=n1;	p2=p1;	t2=t1;	}
-			p1.Set(x->v(i,mx), y->v(i,my), z->v(i,mz), c->v(i,mc));
-			n1 = gr->AddPnt(p1,gr->GetC(ss,p1.c));	t1 = n1>=0;
-			if(mk && t1)	gr->mark_plot(n1,mk);
-			if(t1 && t2)
+			n2 = n1;	n1 = gr->AddPnt(pp[i].p, gr->GetC(ss,pp[i].p.c));
+			if(mk && i%dx==0 && n1>=0 && pp[i].orig)	gr->mark_plot(n1,mk);
+			if(n1>=0 && n2>=0)
 			{
 				gr->line_plot(n1,n2);
 				if(i==1)	gr->arrow_plot(n2,n1,gr->Arrow1);
-				if(i==n-1)	gr->arrow_plot(n1,n2,gr->Arrow2);
-			}
-
-			if(i>0 && ((t1 && !t2) || (t2 && !t1)))	// do smoothing
-			{
-				mreal i1=0, i2=1, ii;
-				mglPoint q(NAN),p;
-				do {
-					ii = (i1+i2)/2;
-					pt.x = p1.x*ii+p2.x*(1-ii);
-					pt.y = p1.y*ii+p2.y*(1-ii);
-					pt.z = p1.z*ii+p2.z*(1-ii);	p=pt;
-					pt.c = p1.c*ii+p2.c*(1-ii);
-					bool t3 = gr->ScalePoint(gr->GetB(),p,q,false);
-					if((t1 && t3) || (t2 && !t3))	i2 = ii;
-					else	i1 = ii;
-				} while(fabs(i2-i1)>1e-3);
-				n3 = gr->AddPnt(pt,gr->GetC(ss,pt.c),q,-1,2);
-				if(t2)	gr->line_plot(n3,n2);
-				else 	gr->line_plot(n1,n3);
 			}
 		}
+		if(n1>=0 && n2>=0)	gr->arrow_plot(n1,n2,gr->Arrow2);
 	}
 	gr->EndGroup();
 }
@@ -460,7 +459,7 @@ void MGL_EXPORT mgl_tens_xy(HMGL gr, HCDT x, HCDT y, HCDT c, const char *pen, co
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tens(HMGL gr, HCDT y, HCDT c, const char *pen, const char *opt)
 {
-	register long n=y->GetNx();
+	long n=y->GetNx();
 	if(n<2)	{	gr->SetWarn(mglWarnLow,"Tens");	return;	}
 	gr->SaveState(opt);
 	mglDataV x(n), z(n);
@@ -495,31 +494,32 @@ void MGL_EXPORT mgl_area_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("Area3",cgid++);
 	m = x->GetNy() > y->GetNy() ? x->GetNy() : y->GetNy();	m = z->GetNy() > m ? z->GetNy() : m;
-	bool sh = mglchr(pen,'!');
-	bool wire = mglchr(pen,'#');
+	bool sh = mglchr(pen,'!'), wire = mglchr(pen,'#'), orig = !mglchr(pen,'a');
 
 	mreal z0=gr->GetOrgZ('x');
-	mreal c1,c2;
-	mglPoint p1,p2,p3,p4,nn;
 	gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
-		c2=c1=gr->NextColor(pal);
+		mreal c1=gr->NextColor(pal), c2=c1;
 		if(gr->GetNumPal(pal)==2*m && !sh)	c2 = gr->NextColor(pal);
 		long mx = j<x->GetNy() ? j:0, my = j<y->GetNy() ? j:0, mz = j<z->GetNy() ? j:0;
 
-		nn.Set(-y->dvx(0,my),x->dvx(0,mx));
-		mglPoint p(x->v(0,mx),y->v(0,my),z->v(0,mz));
-		long n1 = gr->AddPnt(p,c1,nn,-1,27);	p.z = z0;
-		long n2 = gr->AddPnt(p,c2,nn,-1,27);
-		for(long i=1;i<n;i++)
+		mglDataR xx(x,mx), yy(y,my), zz(z,mz);
+		std::vector<mglPointA> pp = orig ? mgl_pnt_copy(&xx, &yy, &zz, 0) :
+			mgl_pnt_prepare(gr->Min, gr->Max, &xx, &yy, &zz, 0);
+		size_t np = pp.size();
+		long n1=-1, n2=-1;
+		mglPoint nn(pp[0].p.y-pp[1].p.y, pp[1].p.x-pp[0].p.x);
+		for(size_t i=0;i<np;i++)
 		{
 			long n3=n1, n4=n2;
-			nn.Set(-y->dvx(i,my),x->dvx(i,mx));
-			p.Set(x->v(i,mx),y->v(i,my),z->v(i,mz));
 			if(sh)	c2=c1=gr->NextColor(pal,i);
-			n1 = gr->AddPnt(p,c1,nn,-1,27);	p.z = z0;	n2 = gr->AddPnt(p,c2,nn,-1,27);
+			if(i>0 && i<np-1)	{	nn.x=(pp[i-1].p.y-pp[i+1].p.y)/2;	nn.y=(pp[i+1].p.x-pp[i-1].p.x)/2;	}
+			else if(i==np-1)	{	nn.x=pp[np-2].p.y-pp[np-1].p.y;	nn.y=pp[np-1].p.x-pp[np-2].p.x;	}
+			n1 = gr->AddPnt(pp[i].p, c1,nn,-1,27);	pp[i].p.z = z0;
+			n2 = gr->AddPnt(pp[i].p, c2,nn,-1,27);
+			if(gr->SamePnt(n1,n3) || gr->SamePnt(n2,n4))	continue;
 			if(wire)
 			{
 				gr->line_plot(n1,n2);	gr->line_plot(n3,n4);
@@ -540,30 +540,31 @@ void MGL_EXPORT mgl_area_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const char
 	static int cgid=1;	gr->StartGroup("Curve",cgid++);
 	mreal zm = gr->AdjustZMin();
 	mreal y0=gr->GetOrgY('x');
-	mreal c1,c2;
 	mglPoint nn(0,0,1);
-	bool sh = mglchr(pen,'!');
-	bool wire = mglchr(pen,'#');
+	bool sh = mglchr(pen,'!'), wire = mglchr(pen,'#'), orig = !mglchr(pen,'a');
 
 	gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
-//	long s=gr->AddTexture(pen,1);
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
-		c2=c1=gr->NextColor(pal);
+		mreal c1=gr->NextColor(pal), c2=c1;
 		if(gr->GetNumPal(pal)==2*m && !sh)	c2=gr->NextColor(pal);
 		long mx = j<x->GetNy() ? j:0, my = j<y->GetNy() ? j:0;
 		mreal z0 = zm + (m-1-j)*(gr->Max.z-zm)/m;
 
-		mglPoint p(x->v(0,mx),y->v(0,my),z0);
-		long n1 = gr->AddPnt(p,c1,nn,-1,27);	p.y = y0;
-		long n2 = gr->AddPnt(p,c2,nn,-1,27);
-		for(long i=1;i<n;i++)
+		mglDataR xx(x,mx), yy(y,my);	mglDataV zz(n,1,1,z0);
+		std::vector<mglPointA> pp = orig ? mgl_pnt_copy(&xx, &yy, &zz, 0) :
+			mgl_pnt_prepare(gr->Min, gr->Max, &xx, &yy, &zz, 0);
+		size_t np = pp.size();
+		long n1=-1, n2=-1;
+		for(size_t i=0;i<np;i++)
 		{
 			long n3=n1, n4=n2;
-			p.Set(x->v(i,mx),y->v(i,my),z0);
 			if(sh)	c2=c1=gr->NextColor(pal,i);
-			n1 = gr->AddPnt(p,c1,nn,-1,27);	p.y = y0;	n2 = gr->AddPnt(p,c2,nn,-1,27);
+			n1 = gr->AddPnt(pp[i].p, c1,nn,-1,27);	pp[i].p.y = y0;
+			n2 = gr->AddPnt(pp[i].p, c2,nn,-1,27);
+			if(n1<0 || n2<0)	{	n1=n2=-1;	continue;	}
+			if(gr->SamePnt(n1,n3) || gr->SamePnt(n2,n4))	continue;
 			if(wire)
 			{
 				gr->line_plot(n1,n2);	gr->line_plot(n3,n4);
@@ -601,6 +602,63 @@ void MGL_EXPORT mgl_area_(uintptr_t *gr, uintptr_t *y, const char *pen, const ch
 //	Region series
 //
 //-----------------------------------------------------------------------------
+struct mglPointB {	mglPoint p1, p2;	bool orig;
+	mglPointB(const mglPoint &pp1, const mglPoint &pp2, bool o) : p1(pp1), p2(pp2), orig(o) {}	};
+std::vector<mglPointB> MGL_NO_EXPORT mgl_pnt_prepare(const mglPoint &a1, const mglPoint &a2, HCDT xx1, HCDT yy1, HCDT zz1, HCDT xx2, HCDT yy2, HCDT zz2)
+{
+	std::vector<mglPointB> out;
+	long n = xx1->GetNx();
+	mglPoint p1(xx1->v(0),yy1->v(0),zz1->v(0)), p2(xx2->v(0),yy2->v(0),zz2->v(0));
+	if(p1>a1 && p1<a2 && p2>a1 && p2<a2)	out.push_back(mglPointB(p1,p2,true));
+	else	out.push_back(mglPointB(mglPoint(NAN),mglPoint(NAN),true));
+	for(long i=1;i<n;i++)
+	{
+		mglPoint q1(xx1->v(i),yy1->v(i),zz1->v(i)), q2(xx2->v(i),yy2->v(i),zz2->v(i));
+		mreal x1,x2,y1,y2,z1,z2,t;
+		x1=mgl_d(a1.x, p1.x, q1.x);	x2=mgl_d(a2.x, p1.x, q1.x);	if(x2<x1)	{	t=x1;	x1=x2;	x2=t;	}
+		y1=mgl_d(a1.y, p1.y, q1.y);	y2=mgl_d(a2.y, p1.y, q1.y);	if(y2<y1)	{	t=y1;	y1=y2;	y2=t;	}
+		z1=mgl_d(a1.z, p1.z, q1.z);	z2=mgl_d(a2.z, p1.z, q1.z);	if(z2<z1)	{	t=z1;	z1=z2;	z2=t;	}
+		mreal d11 = mgl_isnum(x1)?x1:0, d12 = mgl_isnum(x2)?x2:1;
+		if(y1>d11)	d11=y1;
+		if(y2<d12)	d12=y2;
+		if(z1>d11)	d11=z1;
+		if(z2<d12)	d12=z2;
+		x1=mgl_d(a1.x, p2.x, q2.x);	x2=mgl_d(a2.x, p2.x, q2.x);	if(x2<x1)	{	t=x1;	x1=x2;	x2=t;	}
+		y1=mgl_d(a1.y, p2.y, q2.y);	y2=mgl_d(a2.y, p2.y, q2.y);	if(y2<y1)	{	t=y1;	y1=y2;	y2=t;	}
+		z1=mgl_d(a1.z, p2.z, q2.z);	z2=mgl_d(a2.z, p2.z, q2.z);	if(z2<z1)	{	t=z1;	z1=z2;	z2=t;	}
+		mreal d21 = mgl_isnum(x1)?x1:0, d22 = mgl_isnum(x2)?x2:1;
+		if(y1>d21)	d21=y1;
+		if(y2<d22)	d22=y2;
+		if(z1>d21)	d21=z1;
+		if(z2<d22)	d22=z2;
+		
+		std::vector<mreal> dd;
+		if(d11>0 && d11<1)	dd.push_back(d11);
+		if(d21>0 && d21<1)	dd.push_back(d21);
+		if(d12>0 && d12<1)	dd.push_back(d12);
+		if(d22>0 && d22<1)	dd.push_back(d22);
+		// now add all intersections to be sure
+		x1=mgl_d(0, p2.x-p1.x, q2.x-q1.x);	if(x1>0 && x1<1)	dd.push_back(x1);
+		y1=mgl_d(0, p2.y-p1.y, q2.y-q1.y);	if(y1>0 && y1<1)	dd.push_back(y1);
+		z1=mgl_d(0, p2.z-p1.z, q2.z-q1.z);	if(z1>0 && z1<1)	dd.push_back(z1);
+		std::sort(dd.begin(),dd.end());
+		for(size_t j=0;j<dd.size();j++)
+			out.push_back(mglPointB(p1+dd[j]*(q1-p1), p2+dd[j]*(q2-p2), false));
+		if((d11<1 && d12>=1) || (d21<1 && d22>=1))	out.push_back(mglPointB(q1,q2,true));
+		else if(i==n-1)	out.push_back(mglPointB(mglPoint(NAN),mglPoint(NAN),true));
+		p1 = q1;	p2 = q2;
+	}
+	return out;
+}
+std::vector<mglPointB> MGL_NO_EXPORT mgl_pnt_copy(HCDT xx1, HCDT yy1, HCDT zz1, HCDT xx2, HCDT yy2, HCDT zz2)
+{
+	std::vector<mglPointB> out;
+	long n = xx1->GetNx();
+	for(long i=0;i<n;i++)
+		out.push_back(mglPointB(mglPoint(xx1->v(i),yy1->v(i),zz1->v(i)), mglPoint(xx2->v(i),yy2->v(i),zz2->v(i)), true));
+	return out;
+}
+//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_region_3d(HMGL gr, HCDT x1, HCDT y1, HCDT z1, HCDT x2, HCDT y2, HCDT z2, const char *pen, const char *opt)
 {
 	long n=y1->GetNx(), m, pal;
@@ -619,7 +677,7 @@ void MGL_EXPORT mgl_region_3d(HMGL gr, HCDT x1, HCDT y1, HCDT z1, HCDT x2, HCDT
 	mglPoint nn(0,0,1);
 	mreal zm = gr->AdjustZMin();
 //	bool inside = (mglchr(pen,'i'));	// NOTE: check if 'i' is free (used here for inside flag)
-	bool sh = mglchr(pen,'!');
+	bool sh = mglchr(pen,'!'), wire = mglchr(pen,'#'), orig = !mglchr(pen,'a');
 
 	gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
 	for(long j=0;j<m;j++)
@@ -631,15 +689,37 @@ void MGL_EXPORT mgl_region_3d(HMGL gr, HCDT x1, HCDT y1, HCDT z1, HCDT x2, HCDT
 		long mz = (zhave && j<z1->GetNy()) ? j:0;
 		mreal z0 = zm + (m-1-j)*(gr->Max.z-zm)/m;
 
-		long n1 = gr->AddPnt(mglPoint(x1->v(0,mx),y1->v(0,my),zhave?z1->v(0,mz):z0),c1,nn,-1,27);
-		long n2 = gr->AddPnt(mglPoint(x2->v(0,mx),y2->v(0,my),zhave?z2->v(0,mz):z0),c2,nn,-1,27);
-		for(long i=1;i<n;i++)
+		mglDataR xx1(x1,mx), yy1(y1,my), xx2(x2,mx), yy2(y2,my);
+		mglDataV zz0(n,1,1,z0);
+		std::vector<mglPointB> pp;
+		if(zhave)
+		{
+			mglDataR zz1(z1,mz), zz2(z2,mz);
+			pp = orig ? mgl_pnt_copy(&xx1, &yy1, &zz1, &xx2, &yy2, &zz2) :
+				mgl_pnt_prepare(gr->Min, gr->Max, &xx1, &yy1, &zz1, &xx2, &yy2, &zz2);
+		}
+		else
+		{
+			pp = orig ? mgl_pnt_copy(&xx1, &yy1, &zz0, &xx2, &yy2, &zz0) :
+				mgl_pnt_prepare(gr->Min, gr->Max, &xx1, &yy1, &zz0, &xx2, &yy2, &zz0);
+		}
+
+		long n1=-1, n2=-1;
+		size_t np = pp.size();
+		for(size_t i=0;i<np;i++)
 		{
 			long n3=n1, n4=n2;
 			if(sh)	c2=c1=gr->NextColor(pal,i);
-			n1 = gr->AddPnt(mglPoint(x1->v(i,mx),y1->v(i,my),zhave?z1->v(i,mz):z0),c1,nn,-1,27);
-			n2 = gr->AddPnt(mglPoint(x2->v(i,mx),y2->v(i,my),zhave?z2->v(i,mz):z0),c2,nn,-1,27);
-			gr->quad_plot(n1,n2,n3,n4);
+			n1 = gr->AddPnt(pp[i].p1, c1,nn,-1,27);
+			n2 = gr->AddPnt(pp[i].p2, c2,nn,-1,27);
+			if(n1<0 || n2<0)	{	n1=n2=-1;	continue;	}
+			if(gr->SamePnt(n1,n3) || gr->SamePnt(n2,n4))	continue;
+			if(wire)
+			{
+				gr->line_plot(n1,n2);	gr->line_plot(n3,n4);
+				gr->line_plot(n1,n3);	gr->line_plot(n4,n2);
+			}
+			else	gr->quad_plot(n1,n2,n3,n4);
 		}
 	}
 	gr->EndGroup();
@@ -656,11 +736,10 @@ void MGL_EXPORT mgl_region_xy(HMGL gr, HCDT x, HCDT y1, HCDT y2, const char *pen
 	mreal c1,c2;
 	mglPoint nn(0,0,1);
 	mreal zm = gr->AdjustZMin();
-	bool inside = (mglchr(pen,'i'));	// NOTE: check if 'i' is free (used here for inside flag)
-	bool sh = mglchr(pen,'!');
+	bool inside = mglchr(pen,'i');	// NOTE: check if 'i' is free (used here for inside flag)
+	bool sh = mglchr(pen,'!'), wire = mglchr(pen,'#'), orig = !mglchr(pen,'a');
 
 	gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
-//	long s=gr->AddTexture(pen,1);
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
@@ -669,18 +748,31 @@ void MGL_EXPORT mgl_region_xy(HMGL gr, HCDT x, HCDT y1, HCDT y2, const char *pen
 		long mx = j<x->GetNy() ? j:0;
 		mreal z0 = zm + (m-1-j)*(gr->Max.z-zm)/m;
 
-		mreal f1 = y1->v(0,j), f2 = y2->v(0,j), xx = x->v(0,mx);
-		long n1 = gr->AddPnt(mglPoint(xx,f1,z0),c1,nn,-1,27);
-		long n2 = gr->AddPnt(mglPoint(xx,f2,z0),c2,nn,-1,27);
-		for(long i=1;i<n;i++)
+		mglDataR xx(x,mx), yy1(y1,j), yy2(y2,j);
+		mglDataV zz0(n,1,1,z0);
+		std::vector<mglPointB> pp = orig ? mgl_pnt_copy(&xx, &yy1, &zz0, &xx, &yy2, &zz0) :
+			mgl_pnt_prepare(gr->Min, gr->Max, &xx, &yy1, &zz0, &xx, &yy2, &zz0);
+
+		mreal f1=0, f2=0;
+		long n1=-1, n2=-1;
+		size_t np = pp.size();
+		for(size_t i=0;i<np;i++)
 		{
 			long n3=n1, n4=n2;
 			mreal f3=f1, f4=f2;
-			f1 = y1->v(i,j);	f2 = y2->v(i,j);	xx = x->v(i,mx);
+			f1 = pp[i].p1.y;	f2 = pp[i].p2.y;
 			if(sh)	c2=c1=gr->NextColor(pal,i);
-			n1 = gr->AddPnt(mglPoint(xx,f1,z0),c1,nn,-1,27);
-			n2 = gr->AddPnt(mglPoint(xx,f2,z0),c2,nn,-1,27);
-			if(!inside || (f2>f1 && f4>f3))	gr->quad_plot(n1,n2,n3,n4);
+			n1 = gr->AddPnt(pp[i].p1, c1,nn,-1,27);
+			n2 = gr->AddPnt(pp[i].p2, c2,nn,-1,27);
+			if(n1<0 || n2<0)	{	n1=n2=-1;	continue;	}
+			if(inside && (f2<f1 || f4<f3))	continue;
+			if(gr->SamePnt(n1,n3) || gr->SamePnt(n2,n4))	continue;
+			if(wire)
+			{
+				gr->line_plot(n1,n2);	gr->line_plot(n3,n4);
+				gr->line_plot(n1,n3);	gr->line_plot(n4,n2);
+			}
+			else	gr->quad_plot(n1,n2,n3,n4);
 		}
 	}
 	gr->EndGroup();
@@ -723,7 +815,7 @@ void MGL_EXPORT mgl_step_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 	bool sh = mglchr(pen,'!');
 
 	char mk=gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
-	mglPoint p;
+	int d = gr->MeshNum>0 ? gr->MeshNum+1 : n, dx = n>d?n/d:1;
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
@@ -734,14 +826,14 @@ void MGL_EXPORT mgl_step_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 		for(long i=1;i<n;i++)
 		{
 			long n2 = n1;	// horizontal
-			p.Set(x->v(i,mx), y->v(i,my), z->v(i-1,mz));
+			mglPoint p(x->v(i,mx), y->v(i,my), z->v(i-1,mz));
 			mreal c = sh ? gr->NextColor(pal,i):gr->CDef;
 			n1 = gr->AddPnt(p,c);	gr->line_plot(n1,n2);
 			if(i==1)	gr->arrow_plot(n2,n1,gr->Arrow1);
 
 			n2 = n1;	// vertical
 			p.z = z->v(i,mz);	n1 = gr->AddPnt(p,c);
-			if(mk)	gr->mark_plot(n1,mk);
+			if(mk && i%dx==0)	gr->mark_plot(n1,mk);
 			gr->line_plot(n1,n2);
 			if(i==n-1)	gr->arrow_plot(n1,n2,gr->Arrow2);
 		}
@@ -752,36 +844,49 @@ void MGL_EXPORT mgl_step_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 void MGL_EXPORT mgl_step_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const char *opt)
 {
 	long m,n=y->GetNx(), pal;
-	if(mgl_check_dim1(gr,x,y,0,0,"Step"))	return;
+	if(mgl_check_dim1(gr,x,y,0,0,"Step",true))	return;
 
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("Step",cgid++);
 	m = x->GetNy() > y->GetNy() ? x->GetNy() : y->GetNy();
 	bool sh = mglchr(pen,'!');
+	bool same = x->GetNx()==n;
 
 	mreal zVal =gr->AdjustZMin();
 	char mk=gr->SetPenPal(pen,&pal);	gr->Reserve(2*n*m);
-	mglPoint p;
+	int d = gr->MeshNum>0 ? gr->MeshNum+1 : n, dx = n>d?n/d:1;
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
 		long mx = j<x->GetNy() ? j:0, my = j<y->GetNy() ? j:0;
 		gr->NextColor(pal);
-		long n1 = gr->AddPnt(mglPoint(x->v(0,mx), y->v(0,my), zVal));
+		mreal xx = x->v(0,mx);
+		long n1 = gr->AddPnt(mglPoint(same?xx:(xx+x->v(1,mx))/2, y->v(0,my), zVal));
 		if(mk)	gr->mark_plot(n1,mk);
+		if(!same)	n1 = gr->AddPnt(mglPoint(xx, y->v(0,my), zVal));
 		for(long i=1;i<n;i++)
 		{
 			long n2 = n1;	// horizontal
-			p.Set(x->v(i,mx), y->v(i-1,my), zVal);
+			xx = x->v(i,mx);
+			mglPoint p(xx, y->v(i-1,my), zVal);
 			mreal c = sh ? gr->NextColor(pal,i):gr->CDef;
 			n1 = gr->AddPnt(p,c);	gr->line_plot(n1,n2);
 			if(i==1)	gr->arrow_plot(n2,n1,gr->Arrow1);
 
 			n2 = n1;	// vertical
 			p.y = y->v(i,my);		n1 = gr->AddPnt(p,c);
-			if(mk)	gr->mark_plot(n1,mk);
 			gr->line_plot(n1,n2);
-			if(i==n-1)	gr->arrow_plot(n1,n2,gr->Arrow2);
+			if(same && i==n-1)	gr->arrow_plot(n1,n2,gr->Arrow2);
+			long nn = n1;
+			if(!same)	nn = gr->AddPnt(mglPoint((xx+x->v(i+1,mx))/2, y->v(i,my), zVal));
+			if(mk && i%dx==0)	gr->mark_plot(nn,mk);
+		}
+		if(!same)
+		{
+			mglPoint p(x->v(n,mx), y->v(n-1,my), zVal);
+			mreal c = sh ? gr->NextColor(pal,n-1):gr->CDef;
+			long n2 = gr->AddPnt(p,c);	gr->line_plot(n1,n2);
+			gr->arrow_plot(n2,n1,gr->Arrow2);
 		}
 	}
 	gr->EndGroup();
@@ -909,7 +1014,7 @@ void MGL_EXPORT mgl_bars_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 	m = z->GetNy() > m ? z->GetNy() : m;
 	bool sh = mglchr(pen,'!');
 
-	bool wire = mglchr(pen,'#');
+	bool wire = mglchr(pen,'#'), fixed = mglchr(pen,'F');
 	bool above = mglchr(pen,'a'), fall = mglchr(pen,'f');
 	if(above)	fall = false;
 	mreal c1,c2;
@@ -919,6 +1024,18 @@ void MGL_EXPORT mgl_bars_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 	if(mglchr(pen,'>'))	dv = -1;
 	memset(dd,0,n*sizeof(mreal));
 
+	mreal dc=INFINITY;
+	if(fixed)	for(long j=0;j<m;j++)
+	{
+		long mx = j<x->GetNy() ? j:0, my = j<y->GetNy() ? j:0;
+		for(long i=0;i<n-1;i++)
+		{
+			mreal cx = hypot(x->v(i+1,mx)-x->v(i,mx), y->v(i+1,my)-y->v(i,my));
+			if(cx<dc)	dc=cx;
+		}
+	}
+	if(dc==0)	fixed=false;	// NOTE: disable fixed width if it is zero
+
 	gr->SetPenPal(pen,&pal);
 	gr->Reserve(4*n*m);
 	for(long j=0;j<m;j++)
@@ -931,10 +1048,12 @@ void MGL_EXPORT mgl_bars_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 		for(long i=0;i<n;i++)
 		{
 			if(sh)	c2=c1=gr->NextColor(pal,i);
-			mreal vv = x->v(i,mx), d = i<nx-1 ? x->v(i+1,mx)-vv : vv-x->v(i-1,mx), zz;
-			mreal x1 = vv + d/2*(dv-gr->BarWidth), x2 = x1 + gr->BarWidth*d;
-			vv = y->v(i,my);	d = i<ny-1 ? y->v(i+1,my)-vv : vv-y->v(i-1,my);
-			mreal y1 = vv + d/2*(dv-gr->BarWidth), y2 = y1 + gr->BarWidth*d;
+			mreal vv = x->v(i,mx), dx = i<nx-1 ? x->v(i+1,mx)-vv : vv-x->v(i-1,mx), dy, zz;
+			vv = y->v(i,my);	dy = i<ny-1 ? y->v(i+1,my)-vv : vv-y->v(i-1,my);
+			if(fixed)
+			{	mreal ff = dc/hypot(dx,dy);	dx *= ff;	dy *= ff;	}
+			mreal x1 = vv + dx/2*(dv-gr->BarWidth), x2 = x1 + gr->BarWidth*dx;
+			mreal y1 = vv + dy/2*(dv-gr->BarWidth), y2 = y1 + gr->BarWidth*dy;
 			vv = zz = z->v(i,mz);
 			if(!above)
 			{
@@ -972,7 +1091,7 @@ void MGL_EXPORT mgl_bars_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const char
 	m = x->GetNy() > y->GetNy() ? x->GetNy() : y->GetNy();
 	bool sh = mglchr(pen,'!');
 
-	bool wire = mglchr(pen,'#');
+	bool wire = mglchr(pen,'#'), fixed = mglchr(pen,'F');
 	bool above = mglchr(pen,'a'), fall = mglchr(pen,'f');
 	if(above)	fall = false;
 	mreal c1,c2;
@@ -983,6 +1102,18 @@ void MGL_EXPORT mgl_bars_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const char
 	mreal zm = gr->AdjustZMin();
 	memset(dd,0,n*sizeof(mreal));
 
+	mreal dx=INFINITY;
+	if(fixed)
+	{
+		long nn=x->GetNy();
+		for(long i=0;i<n-1;i++)	for(long j=0;j<nn;j++)
+		{
+			mreal cx = fabs(x->v(i+1,j)-x->v(i,j));
+			if(cx<dx)	dx=cx;
+		}
+	}
+	if(dx==0)	fixed=false;	// NOTE: disable fixed width if it is zero
+
 	gr->SetPenPal(pen,&pal);
 	gr->Reserve(4*n*m);
 	for(long j=0;j<m;j++)
@@ -996,6 +1127,7 @@ void MGL_EXPORT mgl_bars_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const char
 		{
 			if(sh)	c2=c1=gr->NextColor(pal,i);
 			mreal vv = x->v(i,mx), d = i<nx-1 ? x->v(i+1,mx)-vv : vv-x->v(i-1,mx), yy;
+			if(fixed)	d = dx;
 			mreal x1 = vv + d/2*(dv-gr->BarWidth), x2 = x1 + gr->BarWidth*d;
 			vv = yy = y->v(i,my);
 			if(!above)
@@ -1056,7 +1188,7 @@ void MGL_EXPORT mgl_barh_yx(HMGL gr, HCDT y, HCDT v, const char *pen, const char
 	m = y->GetNy() > v->GetNy() ? y->GetNy() : v->GetNy();
 	bool sh = mglchr(pen,'!');
 
-	bool wire = mglchr(pen,'#');
+	bool wire = mglchr(pen,'#'), fixed = mglchr(pen,'F');
 	bool above = mglchr(pen,'a'), fall = mglchr(pen,'f');
 	if(above)	fall = false;
 	mreal *dd=new mreal[n], x0,xp,dv=ny>n?1:0;
@@ -1066,6 +1198,18 @@ void MGL_EXPORT mgl_barh_yx(HMGL gr, HCDT y, HCDT v, const char *pen, const char
 	mreal zm = gr->AdjustZMin();
 	memset(dd,0,n*sizeof(mreal));
 
+	mreal dy=INFINITY;
+	if(fixed)
+	{
+		long nn=y->GetNy();
+		for(long i=0;i<n-1;i++)	for(long j=0;j<nn;j++)
+		{
+			mreal cx = fabs(y->v(i+1,j)-y->v(i,j));
+			if(cx<dy)	dy=cx;
+		}
+	}
+	if(dy==0)	fixed=false;	// NOTE: disable fixed width if it is zero
+
 	gr->SetPenPal(pen,&pal);
 	gr->Reserve(4*n*m);
 	for(long j=0;j<m;j++)
@@ -1079,6 +1223,7 @@ void MGL_EXPORT mgl_barh_yx(HMGL gr, HCDT y, HCDT v, const char *pen, const char
 		{
 			if(sh)	c2=c1=gr->NextColor(pal,i);
 			mreal vv = y->v(i,my), d = i<ny-1 ? y->v(i+1,my)-vv : vv-y->v(i-1,my), xx;
+			if(fixed)	d = dy;
 			mreal y1 = vv + d/2*(dv-gr->BarWidth), y2 = y1 + gr->BarWidth*d;
 			vv = xx = v->v(i,mx);
 			if(!above)
@@ -1153,7 +1298,7 @@ void MGL_EXPORT mgl_ohlc_x(HMGL gr, HCDT x, HCDT open, HCDT high, HCDT low, HCDT
 			x1 = vv + dd/2*(dv-gr->BarWidth);	x2 = x1 + gr->BarWidth*dd;
 			x2 = (x2-x1)/m;		x1 += j*x2;		x2 += x1;	vv = (x2+x1)/2;
 			if(sh)	c1=c2=gr->NextColor(pal,i);
-			register long n1,n2;
+			long n1,n2;
 
 			dd = close->v(i,j);
 			mreal c = (i==0 || dd>=close->v(i-1,j)) ? c1:c2;
@@ -1216,8 +1361,8 @@ void MGL_EXPORT mgl_boxplot_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const c
 	mreal *d = new mreal[m];
 	for(long i=0;i<n;i++)	// find quartiles by itself
 	{
-		register long mm,k,j;
-		for(mm=j=0;j<m;j++)
+		long mm=0,k;
+		for(long j=0;j<m;j++)
 		{
 			mreal vv = y->v(i,j);
 			if(mgl_isnum(vv))	{	d[mm]=vv;	mm++;	}
@@ -1240,7 +1385,7 @@ void MGL_EXPORT mgl_boxplot_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const c
 		x1 = vv + dd/2*(dv-gr->BarWidth);
 		x2 = x1 + gr->BarWidth*dd;
 		mreal c = sh ? gr->NextColor(pal,i):gr->CDef;
-		register long n1,n2;
+		long n1,n2;
 
 		n1=gr->AddPnt(mglPoint(x1,b[i],zVal),c);	// horizontal lines
 		n2=gr->AddPnt(mglPoint(x2,b[i],zVal),c);
@@ -1521,7 +1666,7 @@ void MGL_EXPORT mgl_chart(HMGL gr, HCDT a, const char *cols, const char *opt)
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("Chart",cgid++);
 	bool wire = mglchr(cols,'#');	// draw edges
-	register long n=a->GetNx(),i,j=0,len=cols?long(strlen(cols)):0;
+	long n=a->GetNx(),i,j=0,len=cols?long(strlen(cols)):0;
 	if(cols)	for(i=0;i<len;i++)
 		if(strchr(MGL_COLORS,cols[i]) || cols[i]==' ')	j++;
 	if(j==0)	{	cols = MGL_DEF_PAL;	len=long(strlen(cols));	}
@@ -1584,13 +1729,14 @@ void MGL_EXPORT mgl_mark_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT r, const char
 	if(mk==0)	mk='.';
 	bool sh = mglchr(pen,'!');
 
+	int d = gr->MeshNum>0 ? gr->MeshNum+1 : n, dx = n>d?n/d:1;
 	for(long j=0;j<m;j++)
 	{
 		if(gr->NeedStop())	break;
 		gr->NextColor(pal);
 		long mx = j<x->GetNy() ? j:0, my = j<y->GetNy() ? j:0;
 		long mz = j<z->GetNy() ? j:0, mr = j<r->GetNy() ? j:0;
-		for(long i=0;i<n;i++)
+		for(long i=0;i<n;i+=dx)
 		{
 			mreal c = sh ? gr->NextColor(pal,i):gr->CDef;
 			gr->mark_plot(gr->AddPnt(mglPoint(x->v(i,mx),y->v(i,my),z->v(i,mz)),c), mk, fabs(r->v(i,mr)));
@@ -1608,7 +1754,7 @@ void MGL_EXPORT mgl_mark_xy(HMGL gr, HCDT x, HCDT y, HCDT r, const char *pen, co
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_mark_y(HMGL gr, HCDT y, HCDT r, const char *pen, const char *opt)
 {
-	register long n=y->GetNx();
+	long n=y->GetNx();
 	gr->SaveState(opt);
 	mglDataV x(n), z(n);
 	x.Fill(gr->Min.x,gr->Max.x);	z.Fill(gr->AdjustZMin());
@@ -1669,8 +1815,8 @@ void MGL_EXPORT mgl_tube_xyzr(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT r, const cha
 			mreal c = sh ? gr->NextColor(pal,i):gr->CDef;
 			for(long k=0;k<num;k++)
 			{
-				register int kk = k*360/(num-1);
-				register float  co = mgl_cos[(kk)%360], si = mgl_cos[(270+kk)%360];
+				int kk = k*360/(num-1);
+				float  co = mgl_cos[(kk)%360], si = mgl_cos[(270+kk)%360];
 				p = q + t*(rr*co) + u*(rr*si);
 				d = (t*si - u*co)^(l + t*(dr*co) + u*(dr*si));
 				nn[k+num]=nn[k];	nn[k] = gr->AddPnt(p,c,wire?mglPoint(NAN,NAN):d,-1,3);
@@ -1692,7 +1838,7 @@ void MGL_EXPORT mgl_tube_xyr(HMGL gr, HCDT x, HCDT y, HCDT r, const char *pen, c
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tube_r(HMGL gr, HCDT y, HCDT r, const char *pen, const char *opt)
 {
-	register long n=y->GetNx();
+	long n=y->GetNx();
 	if(n<2)	{	gr->SetWarn(mglWarnLow,"Tube");	return;	}
 	gr->SaveState(opt);
 	mglDataV x(n), z(n);
@@ -1702,7 +1848,7 @@ void MGL_EXPORT mgl_tube_r(HMGL gr, HCDT y, HCDT r, const char *pen, const char
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tube(HMGL gr, HCDT y, double rr, const char *pen, const char *opt)
 {
-	register long n=y->GetNx();
+	long n=y->GetNx();
 	if(n<2)	{	gr->SetWarn(mglWarnLow,"Tube");	return;	}
 	gr->SaveState(opt);
 	mglDataV x(n), r(n), z(n);
@@ -1713,7 +1859,7 @@ void MGL_EXPORT mgl_tube(HMGL gr, HCDT y, double rr, const char *pen, const char
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tube_xy(HMGL gr, HCDT x, HCDT y, double rr, const char *pen, const char *opt)
 {
-	register long n=y->GetNx();
+	long n=y->GetNx();
 	if(n<2)	{	gr->SetWarn(mglWarnLow,"Tube");	return;	}
 	gr->SaveState(opt);
 	mglDataV r(n), z(n);
@@ -1798,8 +1944,7 @@ void MGL_EXPORT mgl_tape_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, c
 			if(zo)	{	n3 = gr->AddPnt(p2,c2,q1,-1,3);	n4 = gr->AddPnt(p2+rr*q2,c2,q1,-1,3);	}
 		}
 		else		{	n1=n2=n3=n4=-1;	}
-		register long i;
-		for(i=1;i<n;i++)
+		for(long i=1;i<n;i++)
 		{
 			p1 = p2;	p2.Set(x->v(i,mx), y->v(i,my), z->v(i,mz));
 			l = p2-p1;		l /= mgl_norm(l);
@@ -1837,7 +1982,7 @@ void MGL_EXPORT mgl_tape_xy(HMGL gr, HCDT x, HCDT y, const char *pen, const char
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tape(HMGL gr, HCDT y, const char *pen, const char *opt)
 {
-	register long n=y->GetNx();
+	long n=y->GetNx();
 	if(n<2)	{	gr->SetWarn(mglWarnLow,"Plot");	return;	}
 	gr->SaveState(opt);
 	mglDataV x(n), z(n);
@@ -1906,7 +2051,7 @@ void MGL_EXPORT mgl_pmap_xy(HMGL gr, HCDT x, HCDT y, HCDT r, const char *pen, co
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_pmap(HMGL gr, HCDT y, HCDT r, const char *pen, const char *opt)
 {
-	register long n=y->GetNx();
+	long n=y->GetNx();
 	gr->SaveState(opt);
 	mglDataV x(n), z(n);
 	x.Fill(gr->Min.x,gr->Max.x);	z.Fill(gr->AdjustZMin());
