@@ -25,6 +25,8 @@
 #include <wx/menu.h>
 #include <wx/scrolwin.h>
 
+#undef _
+
 #include "mgl2/canvas_wnd.h"
 #include "mgl2/wx.h"
 //-----------------------------------------------------------------------------
@@ -191,7 +193,7 @@ void wxMathGL::Update()
 	if(draw_func || draw_cl)
 	{
 		if(mgl_get_flag(gr,MGL_CLF_ON_UPD))	mgl_set_def_param(gr);
-		mgl_reset_frames(gr);
+		mgl_set_def_param(gr);		mgl_reset_frames(gr);
 		mgl_set_alpha(gr,alpha);	mgl_set_light(gr,light);
 		if(draw_func)	draw_func(gr, draw_par);	// drawing itself
 		else 	if(draw_cl)	{	mglGraph g(gr);	draw_cl->Draw(&g);	}
@@ -279,8 +281,10 @@ void wxMathGL::OnMouseMove(wxMouseEvent &ev)
 			mreal ff = 240/sqrt(mreal(w*h));
 			phi += int((x0-xe)*ff);
 			tet += int((y0-ye)*ff);
-			if(phi>180)	phi-=360;		if(phi<-180)	phi+=360;
-			if(tet>180)	tet-=360;		if(tet<-180)	tet+=360;
+			if(phi>180)		phi-=360;
+			if(phi<-180)	phi+=360;
+			if(tet>180)		tet-=360;
+			if(tet<-180)	tet+=360;
 //			Update();
 		}
 		if(ev.ButtonDown(wxMOUSE_BTN_RIGHT))	// zoom and perspective
@@ -290,7 +294,8 @@ void wxMathGL::OnMouseMove(wxMouseEvent &ev)
 			x1 = cx+(x1-cx)*exp(-ff);	x2 = cx+(x2-cx)*exp(-ff);
 			y1 = cy+(y1-cy)*exp(-ff);	y2 = cy+(y2-cy)*exp(-ff);
 			per = per + gg;
-			if(per<0)	per = 0;	if(per>=1)	per = 0.9999;
+			if(per<0)	per = 0;
+			if(per>=1)	per = 0.9999;
 //			Update();
 		}
 		if(ev.ButtonDown(wxMOUSE_BTN_MIDDLE))	// shift
@@ -351,11 +356,7 @@ void wxMathGL::ExportBPS(wxString fname)
 	if(fname.IsEmpty())	fname = ScriptName;
 	if(fname.IsEmpty())	wxMessageBox(appName, wxT("No filename."),wxOK|wxICON_ERROR ,this);
 	else
-	{
-		setlocale(LC_ALL, "C");
 		mgl_write_bps(gr,mglw_str(mglSetExtension(fname,"eps")), mglw_str(appName));
-		setlocale(LC_ALL, "");
-	}
 }
 //-----------------------------------------------------------------------------
 void wxMathGL::ExportEPS(wxString fname)
@@ -363,11 +364,7 @@ void wxMathGL::ExportEPS(wxString fname)
 	if(fname.IsEmpty())	fname = ScriptName;
 	if(fname.IsEmpty())	wxMessageBox(appName, wxT("No filename."),wxOK|wxICON_ERROR ,this);
 	else
-	{
-		setlocale(LC_ALL, "C");
 		mgl_write_eps(gr,mglw_str(mglSetExtension(fname,"eps")), mglw_str(appName));
-		setlocale(LC_ALL, "");
-	}
 }
 //-----------------------------------------------------------------------------
 void wxMathGL::ExportSVG(wxString fname)
@@ -375,11 +372,7 @@ void wxMathGL::ExportSVG(wxString fname)
 	if(fname.IsEmpty())	fname = ScriptName;
 	if(fname.IsEmpty())	wxMessageBox(appName, wxT("No filename."),wxOK|wxICON_ERROR ,this);
 	else
-	{
-		setlocale(LC_ALL, "C");
 		mgl_write_svg(gr,mglw_str(mglSetExtension(fname,"eps")), mglw_str(appName));
-		setlocale(LC_ALL, "");
-	}
 }
 //-----------------------------------------------------------------------------
 void wxMathGL::Copy()
